// DCConfig.cpp: Data collection configuration reader/writer

/*
// CodeAnalyst for Open Source
// Copyright 2006 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "stdafx.h"
#include "DCConfigAPI.h"
#include <qstringlist.h>

#ifdef _DEBUG
#include <assert.h>
#include <iostream>
#endif


////////////////////////////////////////////////////////////////////////////////////
// CDCConfig class definition
////////////////////////////////////////////////////////////////////////////////////

// Constants for initializing private data members
#define DefaultConfigName      "No name"
#define DefaultInterval        1.0
#define DefaultMultiplexPeriod 1
#define DefaultProcCore        "Opteron"
#define DefaultMultiplier      10
#define DefaultMaxToTrace      10
#define DefaultMaxCount        0

//
// Simple class constructor
//
DCCONFIGUREFILE_API CDCConfig::CDCConfig()
{
	m_configType = DCConfigInvalid ;
	m_configName = DefaultConfigName ;
	m_toolTip = "" ;
	m_description = "" ;
	// Time-based profiling configuration attributes
	m_interval = DefaultInterval ;
	// Event-based profiling configuration attributes
	m_multiplexPeriod = 0 ;
	m_numberOfGroups = 0 ;
	m_eventGroups = NULL ;
	// Simulation configuration attributes
	m_simConfig.warmupCaches = false ;
	m_simConfig.runSimAfter = false ;
	m_simConfig.maxToTrace = DefaultMaxToTrace ;
	m_simConfig.multiplier = DefaultMultiplier ;
	m_simConfig.saveTrace = false ;
	m_simConfig.traceFileName = "" ;
	m_simConfig.procCore = DefaultProcCore ;
	// IBS configuration attributes
	m_IBSConfig.fetchSampling = false ;
	m_IBSConfig.fetchMaxCount = DefaultMaxCount ;
	m_IBSConfig.opSampling = false ;
	m_IBSConfig.opMaxCount = DefaultMaxCount ;
	m_IBSConfig.opCycleCount = true;
	m_IBSConfig.dcMissInfoEnabled = false;
	m_IBSConfig.fetchList.clear();
	m_IBSConfig.opList.clear();
}

//
// Class copy constructor
//
DCCONFIGUREFILE_API CDCConfig::CDCConfig(const CDCConfig& original)
{
	m_configType = original.m_configType ;
	m_configName = original.m_configName ;
	m_toolTip = original.m_toolTip ;
	m_description = original.m_description ;
	// Time-based profiling configuration attributes
	m_interval = original.m_interval ;
	// Event-based profiling configuration attributes
	m_multiplexPeriod = original.m_multiplexPeriod ;
	m_numberOfGroups = original.m_numberOfGroups ;
	m_eventGroups = NULL ;
	if (NULL != original.m_eventGroups) {
		m_eventGroups = new EventGroup[m_numberOfGroups] ;
		memcpy(m_eventGroups, original.m_eventGroups, m_numberOfGroups*sizeof(EventGroup)) ;
	}
	// Simulation configuration attributes
	m_simConfig.warmupCaches = original.m_simConfig.warmupCaches ;
	m_simConfig.runSimAfter = original.m_simConfig.runSimAfter ;
	m_simConfig.maxToTrace = original.m_simConfig.maxToTrace ;
	m_simConfig.multiplier = original.m_simConfig.multiplier ;
	m_simConfig.saveTrace = original.m_simConfig.saveTrace ;
	m_simConfig.traceFileName = original.m_simConfig.traceFileName ;
	m_simConfig.procCore = original.m_simConfig.procCore ;
	// IBS configuration attributes
	m_IBSConfig.fetchSampling = original.m_IBSConfig.fetchSampling ;
	m_IBSConfig.fetchMaxCount = original.m_IBSConfig.fetchMaxCount ;
	m_IBSConfig.opSampling = original.m_IBSConfig.opSampling ;
	m_IBSConfig.opMaxCount = original.m_IBSConfig.opMaxCount ;
	m_IBSConfig.opCycleCount = original.m_IBSConfig.opCycleCount;
	m_IBSConfig.dcMissInfoEnabled = original.m_IBSConfig.dcMissInfoEnabled;
	m_IBSConfig.fetchList = original.m_IBSConfig.fetchList;
	m_IBSConfig.opList = original.m_IBSConfig.opList;

	// This copy constructor does *not* copy the XML handling state
	// variables. A copy should not occur when a file is being read.
}

//
// Class destructor
//
DCCONFIGUREFILE_API CDCConfig::~CDCConfig()
{
	if (m_eventGroups != NULL) delete [] m_eventGroups ;
	m_eventGroups = NULL ;
}

//
// Class assignment operator
//
const CDCConfig& CDCConfig::operator =(const CDCConfig& rhs)
{
	if (this != &rhs) {
		m_configType = rhs.m_configType ;
		m_configName = rhs.m_configName ;
		m_toolTip = rhs.m_toolTip ;
		m_description = rhs.m_description ;
		// Time-based profiling configuration attributes
		m_interval = rhs.m_interval ;
		// Event-based profiling configuration attributes
		m_multiplexPeriod = rhs.m_multiplexPeriod ;
		m_numberOfGroups = rhs.m_numberOfGroups ;
		if (m_eventGroups != NULL) {
			delete [] m_eventGroups ;
			m_eventGroups = NULL ;
		}
		if (NULL != rhs.m_eventGroups) {
			m_eventGroups = new EventGroup[m_numberOfGroups] ;
			memcpy(m_eventGroups, rhs.m_eventGroups, m_numberOfGroups*sizeof(EventGroup)) ;
		}
		// Simulation configuration attributes
		m_simConfig.warmupCaches = rhs.m_simConfig.warmupCaches ;
		m_simConfig.runSimAfter = rhs.m_simConfig.runSimAfter ;
		m_simConfig.maxToTrace = rhs.m_simConfig.maxToTrace ;
		m_simConfig.multiplier = rhs.m_simConfig.multiplier ;
		m_simConfig.saveTrace = rhs.m_simConfig.saveTrace ;
		m_simConfig.traceFileName = rhs.m_simConfig.traceFileName ;
		m_simConfig.procCore = rhs.m_simConfig.procCore ;
		// IBS configuration attributes
		m_IBSConfig.fetchSampling = rhs.m_IBSConfig.fetchSampling ;
		m_IBSConfig.fetchMaxCount = rhs.m_IBSConfig.fetchMaxCount ;
		m_IBSConfig.opSampling = rhs.m_IBSConfig.opSampling ;
		m_IBSConfig.opMaxCount = rhs.m_IBSConfig.opMaxCount ;
		m_IBSConfig.opCycleCount = rhs.m_IBSConfig.opCycleCount;
		m_IBSConfig.dcMissInfoEnabled = rhs.m_IBSConfig.dcMissInfoEnabled;
		m_IBSConfig.fetchList = rhs.m_IBSConfig.fetchList;
		m_IBSConfig.opList = rhs.m_IBSConfig.opList;
		// This assignment operator does *not* copy the XML handling state
		// variables. An assignment should not occur when a file is being read.
	}
	return( *this ) ;
}

////////////////////////////////////////////////////////////////////////////////////
// DCConfig mutator functions
////////////////////////////////////////////////////////////////////////////////////

//
// Set the data collection configuration type.
//
DCCONFIGUREFILE_API void CDCConfig::SetConfigType(DCConfigType configType)
{
	m_configType = configType ;
}

//
// Set the data collection configuration name.
//
DCCONFIGUREFILE_API void CDCConfig::SetConfigName(const QString configName)
{
	m_configName = configName ;
}

//
// Set the data collection configuration tool tip.
//
DCCONFIGUREFILE_API void CDCConfig::SetToolTip(const QString toolTip)
{
	m_toolTip = toolTip ;
}

//
// Set the data collection configuration description.
//
DCCONFIGUREFILE_API void CDCConfig::SetDescription(const QString description)
{
	m_description = description ;
}

//
// Set the timer interval. TBP only.
//
DCCONFIGUREFILE_API void CDCConfig::SetTimerInterval(float interval)
{
	m_interval = interval ;
}

//
// Set the event information. EBP only.
//
DCCONFIGUREFILE_API void CDCConfig::SetEventInfo(
	EventGroup *pEventGroup, int numberOfGroups,
	UINT64 multiplexPeriod)
{
	m_numberOfGroups = numberOfGroups ;
	// If there is only one event group, force multiplex period to zero
	if (numberOfGroups == 1) {
		m_multiplexPeriod = 0 ;
	} else {
		m_multiplexPeriod = multiplexPeriod ;
	}

	// Delete existing event group array to avoid memory leak
	if (m_eventGroups != NULL) delete [] m_eventGroups ;
	m_eventGroups = new EventGroup[numberOfGroups] ;
#ifdef _DEBUG
	assert((pEventGroup != NULL) && (m_eventGroups != NULL)) ;
#endif
	memcpy(m_eventGroups, pEventGroup, sizeof(EventGroup)*numberOfGroups) ;
}

//
// Set the simulation configuration information. Sim only.
//
DCCONFIGUREFILE_API void CDCConfig::SetSimInfo(SimConfig *pSim)
{
#ifdef _DEBUG
	assert(pSim != NULL) ;
#endif
	if (pSim != NULL)
	{
		m_simConfig.warmupCaches = pSim->warmupCaches ;
		m_simConfig.runSimAfter = pSim->runSimAfter ;
		m_simConfig.maxToTrace = pSim->maxToTrace ;
		m_simConfig.multiplier = pSim->multiplier ;
		m_simConfig.saveTrace = pSim->saveTrace ;
		m_simConfig.procCore = pSim->procCore ;
		m_simConfig.traceFileName = pSim->traceFileName ;
	}
}

//
// Set the IBS configuration information. Sim only.
//
DCCONFIGUREFILE_API void CDCConfig::SetIBSInfo(IBSConfig *pIBS)
{
#ifdef _DEBUG
	assert(pIBS != NULL) ;
#endif
	if (pIBS != NULL)
	{
		m_IBSConfig.fetchSampling 	= pIBS->fetchSampling ;
		m_IBSConfig.fetchMaxCount 	= pIBS->fetchMaxCount ;
		m_IBSConfig.opSampling 		= pIBS->opSampling ;
		m_IBSConfig.opMaxCount 		= pIBS->opMaxCount ;
		m_IBSConfig.opCycleCount 	= pIBS->opCycleCount;
		m_IBSConfig.dcMissInfoEnabled 	= pIBS->dcMissInfoEnabled;
		m_IBSConfig.fetchList 		= pIBS->fetchList;
		m_IBSConfig.opList 		= pIBS->opList;
	}
}
//
// Write the tool tip element to the XML output stream.
//
void CDCConfig::WriteTooltip(QTextStream& xmlStream, QString& text)
{
	xmlStream << "    <tool_tip>" << text << "</tool_tip>\n" ;
}

//
// Write the description element to the XML output stream.
//
void CDCConfig::WriteDescription(QTextStream& xmlStream, QString& text)
{
	xmlStream << "    <description>" << text << "</description>\n" ;
}

//
// Write a string-valued attribute to the XML output stream.
//
void CDCConfig::WriteStringAttr(QTextStream& xmlStream, const char* attr, QString& value)
{
	xmlStream << ' ' << attr << "=\"" << value << "\"" ;
}

//
// Write an integer attribute in decimal to the XML output stream.
//
void CDCConfig::WriteDecimalAttr(QTextStream& xmlStream, const char* attr, unsigned long long int value)
{
	QString decimal_value ;
	decimal_value.setNum(value, 10) ;
	xmlStream << ' ' << attr << "=\"" ;
	xmlStream << decimal_value << "\"" ;
}

//
// Write an integer attribute in hexdecimal to the XML output stream.
//
void CDCConfig::WriteHexAttr(QTextStream& xmlStream, const char* attr, unsigned long long int value)
{
	QString hex_value ;
	hex_value.setNum(value, 16) ;
	xmlStream << ' ' << attr << "=\"" ;
	xmlStream << hex_value << "\"" ;
}

//
// Write a float attribute in decimal to the XML output stream.
//
void CDCConfig::WriteFloatAttr(QTextStream& xmlStream, const char* attr, float value)
{
	QString decimal_value ;
	decimal_value.setNum(value, 'f', 4) ;
	xmlStream << ' ' << attr << "=\"" ;
	xmlStream << decimal_value << "\"" ;
}

//
// Write an Boolean attribute as "T" or "F" to the XML output stream.
//
void CDCConfig::WriteBoolAttr(QTextStream& xmlStream, const char* attr, bool value)
{
	xmlStream << ' ' << attr ;
	if (value) {
		xmlStream << "=\"T\"" ;
	} else {
		xmlStream << "=\"F\"" ;
	}
}

//
// Write a TBP configuration element to the XML output stream.
//
void CDCConfig::WriteTBP(QTextStream& xmlStream)
{
	xmlStream << "  <tbp" ;
	WriteStringAttr(xmlStream, "name", m_configName) ;
	WriteFloatAttr(xmlStream, "interval", m_interval) ;
	xmlStream << ">\n" ;
	WriteTooltip(xmlStream, m_toolTip) ;
	WriteDescription(xmlStream, m_description) ;
	xmlStream << "  </tbp>\n" ;
}

//
// Write an event group element to the XML output stream. The event
// group contains an event element for each valid event in the group.
//
void CDCConfig::WriteEventGroup(QTextStream& xmlStream, EventGroup* pEG)
{
	// Attribute "valid" is not read. It is written as a debugging aid.
	xmlStream << "    <group" ; 
	WriteDecimalAttr(xmlStream, "valid", pEG->numberOfValidEvents) ;
	xmlStream << ">\n" ;

	for (int i = 0 ; i < pEG->numberOfValidEvents ; i++) {
		xmlStream << "      <event" ;
		WriteHexAttr(xmlStream, "select", pEG->eventSelect[i]) ;
		WriteHexAttr(xmlStream, "mask", pEG->eventUnitMask[i]) ;
		WriteBoolAttr(xmlStream, "os", pEG->eventOS[i]) ;
		WriteBoolAttr(xmlStream, "user", pEG->eventUser[i]) ;
		WriteBoolAttr(xmlStream, "edge_detect", pEG->edgeDetect[i]) ;
		WriteBoolAttr(xmlStream, "host", pEG->eventHost[i]) ;
		WriteBoolAttr(xmlStream, "guest", pEG->eventGuest[i]) ;
		WriteDecimalAttr(xmlStream, "count", pEG->eventCount[i]) ;
		xmlStream << "></event>\n" ;
	}

	xmlStream << "    </group>\n" ;
}

//
// Write an EBP configuration element to the XML output stream.
//
void CDCConfig::WriteEBP(QTextStream& xmlStream)
{
	EventGroup* pEG ;

	xmlStream << "  <ebp" ;
	WriteStringAttr(xmlStream, "name", m_configName) ;
	WriteDecimalAttr(xmlStream, "mux_period", m_multiplexPeriod) ;
	// num_groups is not read. It is written as a debugging aid
	WriteDecimalAttr(xmlStream, "num_groups", m_numberOfGroups) ;
	xmlStream << ">\n" ;
#ifdef _DEBUG
	assert( m_eventGroups != NULL ) ;
	assert( m_numberOfGroups >= 1 ) ;
#endif
	pEG = m_eventGroups ;
	for (int i = 0 ; i < m_numberOfGroups ; i++) {
		WriteEventGroup(xmlStream, pEG) ;
		pEG++ ;
	}
	WriteTooltip(xmlStream, m_toolTip) ;
	WriteDescription(xmlStream, m_description) ;
	xmlStream << "  </ebp>\n" ;
}

//
// Write a Sim configuration element to the XML output stream.
//
void CDCConfig::WriteSim(QTextStream& xmlStream)
{
	xmlStream << "  <sim" ;
	WriteStringAttr(xmlStream, "name", m_configName) ;
	xmlStream << "\n      " ;
	WriteBoolAttr(xmlStream, "warmup_caches", m_simConfig.warmupCaches) ;
	WriteBoolAttr(xmlStream, "run_sim_after", m_simConfig.runSimAfter) ;
	WriteBoolAttr(xmlStream, "save_trace", m_simConfig.saveTrace) ;
	xmlStream << "\n      " ;
	WriteStringAttr(xmlStream, "proc_core", m_simConfig.procCore) ;
	WriteDecimalAttr(xmlStream, "multiplier", m_simConfig.multiplier) ;
	WriteDecimalAttr(xmlStream, "max_to_trace", m_simConfig.maxToTrace) ;
	xmlStream << "\n      " ;
	WriteStringAttr(xmlStream, "trace_file_name", m_simConfig.traceFileName) ;
	xmlStream << ">\n" ;
	WriteTooltip(xmlStream, m_toolTip) ;
	WriteDescription(xmlStream, m_description) ;
	xmlStream << "  </sim>\n" ;
}

//
// Write an IBS configuration element to the XML output stream.
//
void CDCConfig::WriteIBS(QTextStream& xmlStream)
{
	xmlStream << "  <ibs" ;
	WriteStringAttr(xmlStream, "name", m_configName) ;
	xmlStream << "\n      " ;
	WriteBoolAttr(xmlStream, "fetch_sampling", m_IBSConfig.fetchSampling) ;
	WriteDecimalAttr(xmlStream, "fetch_max_count", m_IBSConfig.fetchMaxCount) ;
	WriteBoolAttr(xmlStream, "op_sampling", m_IBSConfig.opSampling) ;
	WriteDecimalAttr(xmlStream, "op_max_count", m_IBSConfig.opMaxCount) ;
	WriteBoolAttr (xmlStream, "op_cycle_count", m_IBSConfig.opCycleCount);
	WriteBoolAttr (xmlStream, "dc_miss_info_enabled", m_IBSConfig.dcMissInfoEnabled);

	QString tmp; 
	QStringList fetchList;
	QStringList opList;
	QValueList<unsigned int>::iterator it;
	QValueList<unsigned int>::iterator it_end;
	it     = m_IBSConfig.fetchList.begin();
	it_end = m_IBSConfig.fetchList.end();
	for(; it != it_end ; it++)
	{
		fetchList.append(QString("0x") + QString::number(*it,16));
	}
	tmp = fetchList.join(",");
	WriteStringAttr(xmlStream, "fetchList", tmp) ;

	it     = m_IBSConfig.opList.begin();
	it_end = m_IBSConfig.opList.end();
	for(; it != it_end ; it++)
	{
		opList.append(QString("0x") + QString::number(*it,16));
	}
	tmp = opList.join(",");
	WriteStringAttr(xmlStream, "opList", tmp) ;

	xmlStream << ">\n" ;
	WriteTooltip(xmlStream, m_toolTip) ;
	WriteDescription(xmlStream, m_description) ;
	xmlStream << "  </ibs>\n" ;
}

//
// Write the data configuration element to the XML output stream. Call
// the appropriate configuration type-specific private function to actually
// perform the writing.
//
DCCONFIGUREFILE_API bool CDCConfig::WriteConfigFile(wchar_t *configFileName)
{
	QFile* pFile = NULL ;
	QTextStream xmlStream ;

	if (m_configType == DCConfigInvalid) {
		// No valid configuration data
		return( false ) ;
	}

	pFile = new QFile( QString::fromUcs2((const short unsigned int*)configFileName) ) ;
	if (! pFile->open( IO_WriteOnly )) {
		// Fill open error
		return( false ) ;
	}
	xmlStream.setDevice( pFile ) ;
	// Set XML file character encoding
	xmlStream.setEncoding(QTextStream::UnicodeUTF8) ;
	//	xmlStream.setEncoding(QTextStream::Unicode) ;

	xmlStream << "<?xml version=\"1.0\"?>\n" ;
	xmlStream << "<!DOCTYPE dc_configuration SYSTEM \"dcconfig.dtd\">\n";
	xmlStream << "<dc_configuration>\n" ;
	switch( m_configType ) {
		case DCConfigTBP: 
			{
				WriteTBP( xmlStream ) ;
				break ;
			}
		case DCConfigEBP:
			{
				WriteEBP( xmlStream ) ;
				break ;
			}
		case DCConfigSim:
			{
				WriteSim( xmlStream ) ;
				break ;
			}
		case DCConfigIBS:
			{
				WriteIBS( xmlStream ) ;
				break ;
			}
		default:
			break;
	}
	xmlStream << "</dc_configuration>\n" ;

	// Close file and deallocate QFile explicitly
	xmlStream.unsetDevice() ;
	pFile->close() ;
	delete pFile ;
	return( true ) ;
}

////////////////////////////////////////////////////////////////////////////////////
// DCConfig accessor functions
////////////////////////////////////////////////////////////////////////////////////

//
// Return the data collection configuration type to the caller.
//
DCCONFIGUREFILE_API DCConfigType CDCConfig::GetConfigType()
{
	return( m_configType ) ;
}

//
// Return the configuration name to the caller.
//
DCCONFIGUREFILE_API void CDCConfig::GetConfigName(QString& configName)
{
	configName = m_configName ; 
}

//
// Return the tool tip string to the caller.
//
DCCONFIGUREFILE_API void CDCConfig::GetToolTip(QString& toolTip)
{
	toolTip = m_toolTip ; 
}

//
// Return the description string to the caller.
//
DCCONFIGUREFILE_API void CDCConfig::GetDescription(QString& description)
{
	description = m_description ;
}

//
// Return the timer interval to the caller. TBP only.
//
DCCONFIGUREFILE_API float CDCConfig::GetTimerInterval()
{
	return( m_interval ) ;
}

//
// Return the number of event groups to the caller. EBP only.
//
DCCONFIGUREFILE_API int CDCConfig::GetNumberOfGroups()
{
	return( m_numberOfGroups ) ;
}

//
// Return event configuration information to the caller. The caller
// must provide an array of EventGroup structs as the destination of
// the configuration data, i.e., storage management is the responsibility
// of the caller. EBP only.
//
DCCONFIGUREFILE_API void CDCConfig::GetEventInfo(
	EventGroup *pEventGroup, int numberOfGroups)
{
#ifdef _DEBUG
	assert(pEventGroup != NULL) ;
	assert(numberOfGroups > 0) ;
#endif
	if ((numberOfGroups > 0) && (pEventGroup != NULL)) {
		if (m_eventGroups != NULL) {
			memcpy(pEventGroup, m_eventGroups, sizeof(EventGroup)*numberOfGroups) ;
		}
	}
}

//
// Return the multiplexing period to the caller. EBP only.
//
DCCONFIGUREFILE_API UINT64 CDCConfig::GetMultiplexPeriod()
{
	return( m_multiplexPeriod ) ;
}

//
// Return simulation configuration information to the caller. The caller
// must provide a SimConfig struct as the destination of the configuration
// data. Sim only.
//
DCCONFIGUREFILE_API void CDCConfig::GetSimInfo(SimConfig *pSim)
{
#ifdef _DEBUG
	assert(pSim != NULL) ;
#endif
	if (pSim != NULL) {
		pSim->warmupCaches = m_simConfig.warmupCaches ;
		pSim->runSimAfter = m_simConfig.runSimAfter ;
		pSim->maxToTrace = m_simConfig.maxToTrace ;
		pSim->multiplier = m_simConfig.multiplier ;
		pSim->saveTrace = m_simConfig.saveTrace ;
		pSim->procCore = m_simConfig.procCore ;
		pSim->traceFileName = m_simConfig.traceFileName ;
	}
}

//
// Return IBS configuration information to the caller. The caller
// must provide an IBSConfig struct as the destination of the configuration
// data. IBS only.
//
DCCONFIGUREFILE_API void CDCConfig::GetIBSInfo(IBSConfig *pIBS)
{
#ifdef _DEBUG
	assert(pIBS != NULL) ;
#endif
	if (pIBS != NULL) {
		pIBS->fetchSampling 	= m_IBSConfig.fetchSampling ;
		pIBS->fetchMaxCount 	= m_IBSConfig.fetchMaxCount ;
		pIBS->opSampling 	= m_IBSConfig.opSampling ;
		pIBS->opMaxCount 	= m_IBSConfig.opMaxCount ;
		pIBS->opCycleCount 	= m_IBSConfig.opCycleCount;
		pIBS->dcMissInfoEnabled	= m_IBSConfig.dcMissInfoEnabled;
		pIBS->fetchList 	= m_IBSConfig.fetchList;
		pIBS->opList 		= m_IBSConfig.opList;
	}
}

////////////////////////////////////////////////////////////////////////////////////
// Read data collection confguration from XML file
////////////////////////////////////////////////////////////////////////////////////

//
// Set up the XML file for reading, then start the parse. The XML parse
// proceeds and calls the callback functions below. Return false if the
// parser detected an error.
//
DCCONFIGUREFILE_API bool CDCConfig::ReadConfigFile(wchar_t *configFileName)
{
	// Create a file, an XML input source and a simple reader
	QFile xmlFile( QString::fromUcs2((const short unsigned int*)configFileName) ) ;
	QXmlInputSource source( xmlFile ) ;
	QXmlSimpleReader reader ;
	// Connect this object's handler interface to the XML reader
	reader.setContentHandler( this ) ;
	// Return true if the parse succeeds and no XML semantic errors
	return( reader.parse( source ) && (!m_xmlSemanticError) ) ;
}

////////////////////////////////////////////////////////////////////////////////////
// Callback functions for reading a data collection configuration in XML
////////////////////////////////////////////////////////////////////////////////////

//
// startDocument() is called at the beginning of an XML document.
// Prepare to read a data collection configuration by clearing the
// current configuration.
//
DCCONFIGUREFILE_API bool CDCConfig::startDocument()
{
	m_configName = DefaultConfigName ;
	m_toolTip = "" ;
	m_description = "" ;
	// Clear all DC configuration info as if this was a new object
	m_configType = DCConfigInvalid ;
	// Time-based profiling configuration attributes
	m_interval = DefaultInterval ;
	// Event-based profiling configuration attributes
	m_multiplexPeriod = DefaultMultiplexPeriod ;
	m_numberOfGroups = 0 ;
	if (m_eventGroups != NULL) delete [] m_eventGroups ;
	m_eventGroups = NULL ;
	// Simulation configuration attributes
	m_simConfig.warmupCaches = false ;
	m_simConfig.runSimAfter = false ;
	m_simConfig.maxToTrace = DefaultMaxToTrace ;
	m_simConfig.multiplier = DefaultMultiplier ;
	m_simConfig.saveTrace = false ;
	m_simConfig.traceFileName = "" ;
	m_simConfig.procCore = DefaultProcCore ;
	// IBS configuration attributes
	m_IBSConfig.fetchSampling = false ;
	m_IBSConfig.fetchMaxCount = DefaultMaxCount ;
	m_IBSConfig.opSampling = false ;
	m_IBSConfig.opMaxCount = DefaultMaxCount ;
	m_IBSConfig.opCycleCount = true;
	m_IBSConfig.dcMissInfoEnabled = false;
	m_IBSConfig.fetchList.clear();
	m_IBSConfig.opList.clear();

	// Initialize all XML-related working variables
	m_toolTipIsOpen = false ;
	m_descriptionIsOpen = false ;
	m_groupIsOpen = false ;
	m_eventIndex = 0 ;
	m_xmlSemanticError = false ;
	m_eventGroupList.clear() ;

	return( true ) ;
}

//
// endDocument() is called at the beginning of an XML document.
//
DCCONFIGUREFILE_API bool CDCConfig::endDocument()
{
	m_eventGroupList.clear() ;
	return( true ) ;
}

//
// startElement() is called at the beginning of an XML element.
// Dispatch control using the element name. Handle an element-specific
// attributes. Set-up state variables to handle the contents of the
// element as the XML parse proceeds.
//
DCCONFIGUREFILE_API bool CDCConfig::startElement(
	const QString &namespaceURI, const QString &localName,
	const QString &qName, const QXmlAttributes &atts)
{
	Q_UNUSED(namespaceURI);
	Q_UNUSED(localName);
	QString boolValue ;
	QString elementName = qName.lower() ;

	if ( elementName == "ebp" ) {
		/////////////////////////////////////////////////////////
		// <ebp> Read and store event-based profiling parameters
		/////////////////////////////////////////////////////////
#ifdef _DEBUG
		assert( !atts.value("name").isNull() ) ;
		assert( !atts.value("mux_period").isNull() ) ;
#endif
		m_configType = DCConfigEBP ;
		// Use the default name when configuration name is missing
		if (! atts.value("name").isNull()) {
			m_configName = atts.value("name") ;
		}
		m_multiplexPeriod = atts.value("mux_period").toInt(0, 10) ;
	} else if ( elementName == "group" ) {
		/////////////////////////////////////////////////////////
		// <group> Create a new event group
		// and add it to the end of the group list
		/////////////////////////////////////////////////////////
		EventGroup eg ;  // Alocated on stack
		memset(&eg, 0, sizeof(EventGroup)) ;
		m_eventGroupList.push_back(eg) ;
		m_numberOfGroups++ ;
		m_eventIndex = 0 ;
		m_groupIsOpen = true ;
	} else if ( elementName == "event" ) {
		/////////////////////////////////////////////////////////
		// <event> Read and store
		// event parameters in the last event group on list
		/////////////////////////////////////////////////////////
#ifdef _DEBUG
		assert( !atts.value("select").isNull() ) ;
		assert( !atts.value("mask").isNull() ) ;
		assert( !atts.value("os").isNull() ) ;
		assert( !atts.value("user").isNull() ) ;
		assert( !atts.value("count").isNull() ) ;
		// "edge_detect" is an optional attribute
		// "host" is an optional attribute
		// "guest" is an optional attribute
#endif
		// Only store data for events for which there are counters
		if (m_eventIndex < NumberOfCounters) {
			if ( ! m_eventGroupList.empty() ) {
				// Retrieve and store EBP-related attribute values
				EventGroup& eg = m_eventGroupList.back() ;
				eg.numberOfValidEvents++ ;
				eg.eventSelect[m_eventIndex] = atts.value("select").toInt(0, 16) ;
				eg.eventUnitMask[m_eventIndex] = atts.value("mask").toInt(0, 16) ;
				boolValue = atts.value("os").upper() ;
				eg.eventOS[m_eventIndex] = (boolValue == "T") ;
				boolValue = atts.value("user").upper() ;
				eg.eventUser[m_eventIndex] = (boolValue == "T") ;
				eg.edgeDetect[m_eventIndex] = false ;
				eg.eventHost[m_eventIndex] = false ;
				eg.eventGuest[m_eventIndex] = false ;
				if (! atts.value("edge_detect").isNull()) 
				{
					boolValue = atts.value("edge_detect").upper() ;
					eg.edgeDetect[m_eventIndex] = (boolValue == "T") ;
				}
				if (! atts.value("host").isNull()) 
				{
					boolValue = atts.value("host").upper() ;
					eg.eventHost[m_eventIndex] = (boolValue == "T") ;
				}
				if (! atts.value("guest").isNull()) 
				{
					boolValue = atts.value("guest").upper() ;
					eg.eventGuest[m_eventIndex] = (boolValue == "T") ;
				}
				eg.eventCount[m_eventIndex] = atts.value("count").toInt(0, 10) ;
			}
		}
		m_eventIndex++ ;
	} else if ( elementName == "tool_tip" ) {
		/////////////////////////////////////////////////////////
		// <tool_tip> Set the current tooltip string to empty
		/////////////////////////////////////////////////////////
		m_toolTip = "" ;
		m_toolTipIsOpen = true ;
	} else if ( elementName == "description" ) {
		/////////////////////////////////////////////////////////
		// <description> Set the current description string to empty
		/////////////////////////////////////////////////////////
		m_description = "" ;
		m_descriptionIsOpen = true ;
	} else if ( elementName == "tbp" ) {
		/////////////////////////////////////////////////////////
		// <tbp> time-based profiling configuration parameters
		/////////////////////////////////////////////////////////
#ifdef _DEBUG
		assert( !atts.value("name").isNull() ) ;
		assert( !atts.value("interval").isNull() ) ;
#endif
		m_configType = DCConfigTBP ;
		// if (atts.value("name").isNull()) m_xmlSemanticError = true ;
		// Use the default name when configuration name is missing
		if (! atts.value("name").isNull()) {
			m_configName = atts.value("name") ;
		}
		m_interval = atts.value("interval").toFloat(0) ;
	} else if ( elementName == "sim" ) {
		/////////////////////////////////////////////////////////
		// <sim> simulation configuration parameters
		/////////////////////////////////////////////////////////
#ifdef _DEBUG
		assert( !atts.value("name").isNull() ) ;
		assert( !atts.value("warmup_caches").isNull() ) ;
		assert( !atts.value("run_sim_after").isNull() ) ;
		assert( !atts.value("save_trace").isNull() ) ;
		assert( !atts.value("proc_core").isNull() ) ;
		assert( !atts.value("multiplier").isNull() ) ;
		assert( !atts.value("max_to_trace").isNull() ) ;
#endif
		m_configType = DCConfigSim ;
		// Use the default name when configuration name is missing
		if (! atts.value("name").isNull()) {
			m_configName = atts.value("name") ;
		}
		// Use the default processor core name when attribute is missing
		if (! atts.value("proc_core").isNull()) {
			m_simConfig.procCore = atts.value("proc_core") ;
		}
		// Retrieve and store values for Sim-related attributes
		m_simConfig.maxToTrace = atts.value("max_to_trace").toInt(0, 10) ;
		m_simConfig.multiplier = atts.value("multiplier").toInt(0, 10) ;
		boolValue = atts.value("run_sim_after").upper() ;
		m_simConfig.runSimAfter = (boolValue == "T") ;
		boolValue = atts.value("save_trace").upper() ;
		m_simConfig.saveTrace = (boolValue == "T") ;
		boolValue = atts.value("warmup_caches").upper() ;
		m_simConfig.warmupCaches = (boolValue == "T") ;
		m_simConfig.traceFileName = atts.value("trace_file_name") ;
	} else if ( elementName == "ibs" ) {
		/////////////////////////////////////////////////////////
		// <ibs> IBS configuration parameters
		/////////////////////////////////////////////////////////
#ifdef _DEBUG
		assert( !atts.value("name").isNull() ) ;
		assert( !atts.value("fetch_sampling").isNull() ) ;
		assert( !atts.value("fetch_max_count").isNull() ) ;
		assert( !atts.value("op_sampling").isNull() ) ;
		assert( !atts.value("op_max_count").isNull() ) ;
#endif
		m_configType = DCConfigIBS ;
		// Use the default name when configuration name is missing
		if (! atts.value("name").isNull()) {
			m_configName = atts.value("name") ;
		}
		m_IBSConfig.fetchSampling = false ;
		m_IBSConfig.fetchMaxCount = DefaultMaxCount ;
		m_IBSConfig.opSampling = false ;
		m_IBSConfig.opMaxCount = DefaultMaxCount ;
		boolValue = atts.value("fetch_sampling").upper() ;
		m_IBSConfig.fetchSampling = (boolValue == "T") ;
		boolValue = atts.value("op_sampling").upper() ;
		m_IBSConfig.opSampling = (boolValue == "T") ;
		m_IBSConfig.fetchMaxCount = atts.value("fetch_max_count").toInt(0, 10) ;
		m_IBSConfig.opMaxCount = atts.value("op_max_count").toInt(0, 10) ;
		// Use the default when attribute is missing
		if (! atts.value("op_cycle_count").isNull()) {
			boolValue = atts.value("op_cycle_count").upper() ;
			m_IBSConfig.opCycleCount = (boolValue == "T") ;
		}
		
		boolValue = atts.value("dc_miss_info_enabled").upper() ;
		m_IBSConfig.dcMissInfoEnabled = (boolValue == "T") ;

		QStringList strList;
		// Handle fetchList
		strList = QStringList::split(QString(","),atts.value("fetchList"));
		if (!strList.empty()) {
			QStringList::iterator it = strList.begin();
			QStringList::iterator it_end = strList.end();
			for(; it != it_end; it++) {
				m_IBSConfig.fetchList.append(((QString)*it).toUInt(0,16));
			}
		}

		// Handle opList
		strList.clear();
		strList = QStringList::split(QString(","),atts.value("opList"));
		if (!strList.empty()) {
			QStringList::iterator it = strList.begin();
			QStringList::iterator it_end = strList.end();
			for(; it != it_end; it++) {
				m_IBSConfig.opList.append(((QString)*it).toUInt(0,16));
			}
		}

	} else if ( elementName == "dc_configuration" ) {
		/////////////////////////////////////////////////////////
		// <dc_configuration>
		/////////////////////////////////////////////////////////
	} else {
		// Unrecognized element name
		return( false ) ;
	}
	return( true ) ;
}

//
// endElement() is called at the end of an XML element. Dispatch
// control using the element name to perform element-specific
// finalization of the element's contents.
//
DCCONFIGUREFILE_API bool CDCConfig::endElement(
	const QString &namespaceURI, const QString &localName,
	const QString &qName)
{
	Q_UNUSED(namespaceURI);
	Q_UNUSED(localName);
	QString elementName = qName.lower() ;
	if ( elementName == "ebp" ) {
		/////////////////////////////////////////////////////////
		// </ebp> Copy event group list to array
		/////////////////////////////////////////////////////////
		EventGroupList::const_iterator eg ;
		EventGroup *pEG ;
#ifdef _DEBUG
		assert( m_numberOfGroups >= 1 ) ;
		assert( m_numberOfGroups == m_eventGroupList.size() ) ;
#endif
		// If only one event group, force multiplex period to zero
		if (m_numberOfGroups == 1) m_multiplexPeriod = 0 ;
		// Allocate array to hold event groups
		m_eventGroups = new EventGroup[m_numberOfGroups] ;
		// Copy event groups from list to the new array
		pEG = m_eventGroups ;
		for (eg = m_eventGroupList.begin() ; eg != m_eventGroupList.end() ; eg++) {
			pEG->numberOfValidEvents = eg->numberOfValidEvents ;
			for (int i = 0 ; i < NumberOfCounters ; i++) {
				pEG->eventSelect[i] = eg->eventSelect[i] ;
				pEG->eventUnitMask[i] = eg->eventUnitMask[i] ;
				pEG->eventOS[i] = eg->eventOS[i] ;
				pEG->eventUser[i] = eg->eventUser[i] ;
				pEG->edgeDetect[i] = eg->edgeDetect[i] ;
				pEG->eventGuest[i] = eg->eventGuest[i] ;
				pEG->eventHost[i] = eg->eventHost[i] ;
				pEG->eventCount[i] = eg->eventCount[i] ;
			}
			pEG++ ;
		}
		// Clear the event group list now that we're done with it
		m_eventGroupList.clear() ;
	} else if ( elementName == "group" ) {
		/////////////////////////////////////////////////////////
		// </group> Close the open group element
		/////////////////////////////////////////////////////////
		m_groupIsOpen = false ;
	} else if ( elementName == "tool_tip" ) {
		/////////////////////////////////////////////////////////
		// </tool_tip> Close the open tooltip element
		/////////////////////////////////////////////////////////
		m_toolTipIsOpen = false ;
	} else if ( elementName == "description" ) {
		/////////////////////////////////////////////////////////
		// </description> Close the open description element
		/////////////////////////////////////////////////////////
		m_descriptionIsOpen = false ;
	}
	return( true ) ;
}

//
// characters() is called to return text that is the content of
// a non-empty XML element. Tooltip and description elements are
// the only DC configuration elements that are non-empty.
//
DCCONFIGUREFILE_API bool CDCConfig::characters(const QString &ch)
{
	if (m_descriptionIsOpen) {
		// Add characters to the end of the description text
		m_description.append( ch.simplifyWhiteSpace() ) ;
	} else if (m_toolTipIsOpen) {
		// Add characters to the end of the tooltip text
		m_toolTip.append( ch.simplifyWhiteSpace() ) ;
	}
	return( true ) ;
}
