//$Id: EventCfg.h 16383 2009-03-23 17:03:41Z ssuthiku $
//  This is our derivative of the iEventCfg.ui

/**
*   (c) 2006 Advanced Micro Devices, Inc.
*  YOUR USE OF THIS CODE IS SUBJECT TO THE TERMS
*  AND CONDITIONS OF THE GNU GENERAL PUBLIC
*  LICENSE FOUND IN THE "GPL.TXT" FILE THAT IS
*  INCLUDED WITH THIS FILE AND POSTED AT
*  <http://www.gnu.org/licenses/gpl.html>
*  Support: codeanalyst@amd.com
*/

#ifndef EVENT_CFG_H
#define  EVENT_CFG_H

#include <qstring.h>
#include <qcheckbox.h>
#include <qlistbox.h>
#include "iEventCfg.h"
#include "atunefile.h"
#include "ProfileCollection.h"
#include "eventsfile.h"


const unsigned int UNIT_MASK_COUNT = 8;

class UnitCheckBox : public QCheckBox
{
	Q_OBJECT
private:
	int m_index;
public:
	UnitCheckBox (QWidget *parent, int index);
	~UnitCheckBox ();
signals:
	void checked (int index);
	private slots:
		void onChecked ();
};

class EventWithProps : public QListViewItem
{
public:
	EventWithProps (QListViewItem *pParent);
	EventWithProps (QListView *pParent, EventWithProps *pAfter);

	UINT64 m_count;
	unsigned int m_eventSelect ;   // Event select
	unsigned int m_eventUnitMask; // Event unit mask
	bool m_edgeDetect;
	bool m_eventOS;               // Event OS sampling control
	bool m_eventUser ;             // Event User sampling control
};

class EventListWithProps : public QListBoxText
{
public:
	EventListWithProps (QListBox *pParent, const QString & text = QString::null);
	virtual ~EventListWithProps();

	UINT64 m_count;
	unsigned int m_eventSelect ;   // Event select
	unsigned int m_eventUnitMask; // Event unit mask
	bool m_edgeDetect;
	bool m_eventOS;               // Event OS sampling control
	bool m_eventUser ;             // Event User sampling control
};


class EventCfgDlg : public IEventCfg
{ 
	Q_OBJECT
public:
	EventCfgDlg ( QWidget* parent = 0, const char* name = 0, bool modal = true,
		WFlags fl = WStyle_Customize | WStyle_NormalBorder | WStyle_Title );
	virtual ~EventCfgDlg ();

	void setProperties (EBP_OPTIONS *pSession, const char* path);
	void setProfile (ProfileCollection *pProfiles, QString name);
	bool wasModified ();

public slots:
	virtual void onOk ();
	void onModified ();
	void onConfigChanged (const QString & configName);

	void onAddConfig ();
	void onAddEvent ();
	void onNewGroup ();
	void onMoveEventUp ();
	void onMoveEventDown ();
	void onRemoveEvent ();

	void onEventChanged (QListViewItem *pEvent);
	void onUnitChecked (int index);
	void onEdgeChecked (bool on);
	void onUsrChecked (bool on);
	void onOsChecked (bool on);
	void onCountChanged (const QString & count);

private:
	void buildCheckBoxes ();
	void detectCpu (const char* path);
	int getCpuInfo(const char* path,
			QString &vendorId,
			QString &name,
			QString &family,
			QString &model,
			QString &stepping,
			QString &flags);

	void readEventsFile (QString fileName, int model);
	void updateMux ();
	void insertEvents (EBP_OPTIONS *pSession);
	bool getEvents (EBP_OPTIONS *pSession);
	void resetEventProps ();
	QString makeEventName (unsigned int evSelect);
	QListViewItem * ifNeedAddGroup (int minEventCount = MAX_EVENTNUM);
	unsigned int getMinCountForEvent(unsigned int event);

private:
	UnitCheckBox * m_pUnitMask[UNIT_MASK_COUNT];
	QString m_profileName;
	ProfileCollection *m_pProfiles;
	bool m_modified;
	CEventsFile m_eventsFile;
	bool m_bGeodeChip;
	bool m_bHasLocalApic;
	bool m_isViewProperty;
	QString m_oprofileEventFile;
};
#endif
