//$Id: ManageCfgs.cpp 15335 2008-10-01 21:12:47Z ssuthiku $

/**
*   (c) 2006 Advanced Micro Devices, Inc.
*  YOUR USE OF THIS CODE IS SUBJECT TO THE TERMS
*  AND CONDITIONS OF THE GNU GENERAL PUBLIC
*  LICENSE FOUND IN THE "GPL.TXT" FILE THAT IS
*  INCLUDED WITH THIS FILE AND POSTED AT
*  <http://www.gnu.org/licenses/gpl.html>
*  Support: codeanalyst@amd.com
*/

#include <qlistbox.h>
#include <qdir.h>
#include "ManageCfgs.h"
#include "TimerCfg.h"
#include "EventCfg.h"
#include "CfgIbs.h"


ManageCfgsDlg::ManageCfgsDlg ( ProfileCollection *pProfiles, QWidget* parent, 
							  const char* name, bool modal, WFlags fl) 
							  : IManageCfgs (parent, name, modal, fl)
{
	m_pCollection = pProfiles;
	refreshList();
	connect (m_pEdit, SIGNAL (clicked()), SLOT (onEdit ()));
	connect (m_pRemove, SIGNAL (clicked()), SLOT (onRemove ()));
	connect (m_pExport, SIGNAL (clicked()), SLOT (onExport ()));
	connect (m_pImport, SIGNAL (clicked()), SLOT (onImport ()));
}


ManageCfgsDlg::~ManageCfgsDlg ()
{
}


void ManageCfgsDlg::refreshList ()
{
	m_pCfgList->clear ();
	if (NULL == m_pCollection)
		return;
	m_pCfgList->insertStringList (m_pCollection->getListOfProfiles ());
}


void ManageCfgsDlg::onRemove ()
{
	//Ignore if nothing is selected.
	if (m_pCfgList->currentText().isEmpty())
	{
		QMessageBox::warning ( this, "CodeAnalyst Warning",
			"Please select profile configuration to remove.");
		return;
	}

	if(QMessageBox::warning(this,"Remove Confirmation",
			QString("CodeAnalyst is removing configuration \"")
				+ m_pCfgList->currentText() + "\"\n\n"
				+ "Confirm removal?\n",
				QMessageBox::Yes,QMessageBox::No)
		!= QMessageBox::Yes)
	{
		return;
	}

	if (!m_pCollection->removeProfile (m_pCfgList->currentText()))
	{
		QMessageBox::warning (this, "CodeAnalyst warning",
			"Cannot remove predefined profie configuration.");
		return;
	}

	refreshList ();
}


void ManageCfgsDlg::onEdit ()
{
	int ret;
	//
	QString profile = m_pCfgList->currentText();

	if(profile.isEmpty())
	{
		QMessageBox::warning ( this, "CodeAnalyst Warning",
			"Please select profile configuration to edit.");
		return;
	}

	switch (m_pCollection->getProfileType (profile))
	{
	case TIMER_TRIGGER: {
		TimerCfgDlg *pTimer = new TimerCfgDlg (this);
		RETURN_IF_NULL (pTimer, this);
		pTimer->setProfile (m_pCollection, profile);
		ret = pTimer->exec ();
		delete pTimer;
		break; 
						}
	case EVENT_TRIGGER: {
		EventCfgDlg *pEventDlg = new EventCfgDlg (this);
		RETURN_IF_NULL (pEventDlg, this);
		pEventDlg->setProfile (m_pCollection, profile);
		ret = pEventDlg->exec ();

		delete pEventDlg;
		break; 
						}
	case IBS_TRIGGER: {
		IbsCfgDlg *pIbs = new IbsCfgDlg (this);
		RETURN_IF_NULL (pIbs, this);
		pIbs->setProfile (m_pCollection, profile);
		ret = pIbs->exec ();
		delete pIbs;
		break; 
					  }
	default:
		ret = 0;
		// Ignore if nothing is selected.
		break;
	}
	if (QDialog::Accepted == ret)
	{
		refreshList ();
	}
}


void ManageCfgsDlg::onExport ()
{
	//Ignore if nothing is selected.
	if (m_pCfgList->currentText().isEmpty())
	{
		QMessageBox::warning ( this, "CodeAnalyst Warning",
			"Please select profile configuration to export.");
		return;
	}

	//Get file to export to
	QString exportFile = QFileDialog::getSaveFileName(
		//		QString::null, "Profile Configurations (*.xml);;All files (*.*)", this,
		m_pCollection->getFileName (m_pCfgList->currentText()), 
		"Profile Configurations (*.xml);;All files (*.*)", this,
		NULL, "Please enter the file name to export the configuration to" );
	if (exportFile.isEmpty())
	{
		return;
	}
	while (QFile::exists (exportFile))
	{
		if (QMessageBox::Yes == QMessageBox::question (this, 
			"CodeAnalyst Warning", "Do you want to overwrite the existing file?",
			QMessageBox::Yes, QMessageBox::No))
		{
			break;
		}
		exportFile = QFileDialog::getSaveFileName(
			QString::null, "Profile Configurations (*.xml);;All files (*.*)", 
			this, NULL, 
			"Please enter the file name to export the configuration to" );
	}
	if (!exportFile.endsWith (".xml", false))
	{
		exportFile += ".xml";
	}
	m_pCollection->exportProfile (m_pCfgList->currentText(), exportFile);
	refreshList ();
}


void ManageCfgsDlg::onImport ()
{
	QString importFile = QFileDialog::getOpenFileName(
		QString::null, "Profile Configurations (*.xml);;All files (*.*)", this,
		NULL, "Please find the file name to import" );
	if (importFile.isEmpty())
	{
		return;
	}

	switch ( m_pCollection->importProfile (importFile))
	{
	case PROFILE_FILE_NOT_FOUND:
		QMessageBox::critical (this,
			"CodeAnalyst error", "The specified file (" + importFile 
			+ ") is not a valid profile configuration.\n"
			"Please choose another file.");
		break;
	case PROFILE_DUPLICATE_NAME:
		QMessageBox::warning (this,
			"CodeAnalyst error", "The specified file (" + importFile 
			+ ") contains a configuration which already exists.\n"
			"Please choose another file.");
		break;
	default:
	case PROFILE_IMPORT_OKAY:
		break;
	}
	refreshList ();
}
