//$Id: ProfileCollection.h 14799 2008-06-24 21:58:10Z ssuthiku $
//This file defines the ProfileCollection Interface, which handles most of the
//	profile file interaction
/*
// CodeAnalyst for Open Source
// Copyright 2006-2007 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef PROFILE_COLLECTION_H
#define PROFILE_COLLECTION_H

#include "stdafx.h"
#include "DCConfigAPI.h"
#include "atunefile.h"

const QString CURRENT_EVENT = "Current event-based profile";
const QString CURRENT_IBS = "Current instruction-based profile";

class ProfileElementType
{
public:
	QString path;
	bool modifiable;
	CDCConfig * pConfigure;
	ProfileElementType () 
	{
		pConfigure = NULL;
	};
	~ProfileElementType()
	{
		if (NULL != pConfigure)
			delete pConfigure;
	};
};

enum {
	PROFILE_IMPORT_OKAY = 0,
	PROFILE_FILE_NOT_FOUND,
	PROFILE_DUPLICATE_NAME
};

class ProfileCollection 
{
public:
	ProfileCollection ();
	~ProfileCollection ();

	void readAvailableProfiles();
	QStringList getListOfProfiles ();
	QStringList getListOfEbpProfiles ();

	bool profileExists (QString name);
	bool getProfileTexts (QString name, QString *pToolTip, QString *pDescrip);
	TRIGGER getProfileType (QString name);
	QString getLastModified ();
	void clearLastModified ();
	//Note that the correct type needs to be passed in...
	bool getProfileConfig (QString name, SESSION_OPTIONS * pConfig);
	bool setProfileConfig (QString name, SESSION_OPTIONS * pConfig);

	bool exportProfile (QString name, QString fileName);
	int importProfile (QString fileName);
	bool removeProfile (QString name);
	QString getFileName (QString name);

private:
	void verifyAMDPlatform ();

	typedef QMap<QString, ProfileElementType> ConfigMap;
	ConfigMap m_configs;
	QString m_lastModified;
};
#endif
