//$Id: SessionOptionsDlg.cpp 16686 2009-06-11 15:17:57Z ssuthiku $
//  This is the implementation for the SessionOptionsDlg class, which derives from
//	ISessionOptions and has different functionality for express vs normal mode

/**
*   (c) 2006 Advanced Micro Devices, Inc.
*  YOUR USE OF THIS CODE IS SUBJECT TO THE TERMS
*  AND CONDITIONS OF THE GNU GENERAL PUBLIC
*  LICENSE FOUND IN THE "GPL.TXT" FILE THAT IS
*  INCLUDED WITH THIS FILE AND POSTED AT
*  <http://www.gnu.org/licenses/gpl.html>
*  Support: codeanalyst@amd.com
*/
#include <unistd.h>
#include <qtextedit.h>
#include <qcombobox.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>

#include "SessionOptionsDlg.h"
#include "auxil.h"
#include "TimerCfg.h"
#include "EventCfg.h"
#include "CfgIbs.h"
#include "atuneoptions.h"
#include "xp.h"
#include "stdafx.h"
#include "ProcessFilterDlg.h"
#include "elf.h"

SessionOptionsDlg::SessionOptionsDlg (ATuneFile *pProject, QWidget* parent,
					const char* name, bool modal, WFlags fl)
					: ISessionOptions (parent, name, modal, fl)
{
	m_pProject = pProject;
	m_pProfiles = NULL;
	m_lastDuration = "20";
	m_lastDurationChecked = false;
	m_pAffinityDlg  = NULL;

	m_pRunControl = new IRunControl (this);
	setExtension (m_pRunControl);
	setOrientation (Vertical);

	//Handle run control enabling with launch
	connect (this, SIGNAL (enableLaunchItems (bool)), 
		m_pRunControl->m_pStopOnExit, SLOT (setEnabled (bool)));
	connect (this, SIGNAL (enableLaunchItems (bool)), 
		m_pRunControl->m_pTerminateAfter, SLOT (setEnabled (bool)));
	connect (m_pRunControl->m_pStopOnExit, SIGNAL (toggled (bool)), 
		this, SLOT (onToggleStopOnExit (bool)));
	connect (m_pRunControl->m_pProfileDuration, SIGNAL (clicked()),
		SLOT (onCheckDuration()));
	connect (m_pRunControl->m_pAffinityBtn, SIGNAL (clicked ()), 
		SLOT(onCpuAffinity()));
	connect (m_pRunControl->m_pApplyFilter, SIGNAL (toggled(bool)), 
		SLOT(onToggleApplyFilter(bool)));
	connect (m_pRunControl->m_pApplyFilterAdvanceBtn, SIGNAL (clicked ()), 
		SLOT(onSetAdvanceFilter()));

	connect (m_pShowRunControl,SIGNAL (toggled (bool)), 
		SLOT (onClickShowOptions (bool)));
	connect (m_pLaunch, SIGNAL (textChanged (const QString &)), 
		SLOT (onLaunchChange (const QString &)));
	connect (m_pProfileType, SIGNAL (activated (const QString &)), 
		SLOT (onProfileChange (const QString &)));
	connect (m_pOk, SIGNAL (clicked()), SLOT (onOk()));
	connect (m_pCancel, SIGNAL (clicked()), SLOT (reject()));
	connect (m_pBrowseLaunch, SIGNAL (clicked ()), 
		SLOT (onBrowseLaunch()));
	connect (m_pBrowseWorkDir, SIGNAL (clicked ()), 
		SLOT (onBrowseWorkDir()));
	connect (m_pEditProfile, SIGNAL (clicked ()),
		SLOT (onClickEditProfile()));

	CATuneOptions opts;
	onClickShowOptions (opts.getShowRunOptions() );

} //SessionOptionsDlg::SessionOptionsDlg


SessionOptionsDlg::~SessionOptionsDlg()
{
	
}

void SessionOptionsDlg::onToggleStopOnExit (bool b) {
	if (b) {
		m_pRunControl->m_pProfileDuration->setEnabled (b);
			m_pRunControl->m_pProfileDuration->setChecked(m_lastDurationChecked);	
			onCheckDuration();
	} else {
		if (m_pRunControl->m_pProfileDuration->isChecked()) {
			m_lastDurationChecked = true;
			m_pRunControl->m_pProfileDuration->setChecked(false);	
			onCheckDuration();
		}else{
			m_lastDurationChecked = false;
		}
		m_pRunControl->m_pProfileDuration->setEnabled (b);
	}
}

void SessionOptionsDlg::onClickEditProfile ()
{
	//Get index from m_pProfileType
	m_pProfiles->clearLastModified ();
	QString profile = m_pProfileType->currentText();
	TRIGGER type = m_pProfiles->getProfileType (profile);
	int ret = QDialog::Rejected;
	switch (type)
	{
	case TIMER_TRIGGER: {
		TimerCfgDlg *pTimer = new TimerCfgDlg (this);
		RETURN_IF_NULL (pTimer, this);
		pTimer->setProfile (m_pProfiles, profile);
		ret = pTimer->exec ();
		delete pTimer;
		break; 
						}
	case EVENT_TRIGGER: {
		EventCfgDlg *pEventDlg = new EventCfgDlg (this);
		RETURN_IF_NULL (pEventDlg, this);
		pEventDlg->setProfile (m_pProfiles, profile);
		ret = pEventDlg->exec ();
		delete pEventDlg;
		break; 
						}
	case IBS_TRIGGER: {
		IbsCfgDlg *pIbs = new IbsCfgDlg (this);
		RETURN_IF_NULL (pIbs, this);
		pIbs->setProfile (m_pProfiles, profile);
		ret = pIbs->exec ();
		delete pIbs;
		break; 
					  }
	default:
		QMessageBox::information (this, "CodeAnalyst Error", "Profile selected is corrupted. Please select new profile.");
		break;
	}
	if (QDialog::Accepted == ret)
	{
		m_pProfileType->clear();
		m_pProfileType->insertStringList (m_pProfiles->getListOfProfiles ());
		QString curProfile = m_pProfiles->getLastModified ();
		if (!curProfile.isEmpty())
		{
			// Search if exists
			for (int i = m_pProfileType->count()-1 ; i >= 0 ; i--)
			{
				if(m_pProfileType->text(i) == curProfile)
				{
					m_pProfileType->setCurrentItem (i);
				}
			}
		} else {
			// Search if exists
			for (int i = m_pProfileType->count()-1 ; i >= 0 ; i--)
			{
				if(m_pProfileType->text(i) == profile)
				{
					m_pProfileType->setCurrentItem (i);
				}
			}
		}
	}
} //SessionOptionsDlg::onClickEditProfile


void SessionOptionsDlg::onClickShowOptions (bool toggled)
{
	showExtension (toggled);
	if (toggled)
	{
		m_pShowRunControl->setText ("Hide O&ptions <<<");
		m_pShowRunControl->setOn(true);
	} else {
		m_pShowRunControl->setText ("Show O&ptions >>>");
		m_pShowRunControl->setOn(false);
	}
}


void SessionOptionsDlg::onBrowseWorkDir ()
{
	QString     ProjPath;

	//ProjPath = QFileDialog::getExistingDirectory();
	//If we want to use the shortcuts, we'll need to use SHBrowseForFolder
	ProjPath = QFileDialog::getExistingDirectory ( m_pWorkDir->currentText(), 
		this, NULL, "Please select a working directory.");

	// Make sure the project directory format is valid
	if( !ProjPath.isNull() )
	{
		ProjPath.replace ('\\', '/');
		m_pWorkDir->setEditText ( ProjPath );
	}
}

void SessionOptionsDlg::onBrowseLaunch ()
{
	QString start;
	if (!m_pLaunch->currentText().isEmpty())
	{
		start = m_pLaunch->currentText().section ('\"', 1, 1);
	}
	QString Prog = QFileDialog::getOpenFileName( start, 
		"All files (*)", this, "FileBrowse", 
		"Select an application to launch" );

	if( Prog.isEmpty() ) 
		return;
	/*just in case, all paths use / instead of \ */
	Prog.replace ("\\", "/");
	m_pLaunch->setEditText( "\"" + Prog + "\"" );
}


void SessionOptionsDlg::onLaunchChange (const QString & string)
{
	if (!string.isEmpty())
	{
		QString temp = string.section ('\"', 1, 1);
		if (m_pWorkDir->currentText().isEmpty())
			m_pWorkDir->setEditText (temp.section ('/', 0, -2));
		if (QFile::exists (temp))
		{
			emit enableLaunchItems (true);
		} else {
			emit enableLaunchItems (false);
		}
	} else {
		m_pRunControl->m_pStopOnExit->setChecked (false);
		m_pRunControl->m_pTerminateAfter->setChecked (false);
		m_pRunControl->m_pProfileDuration->setChecked (false);
	
		onCheckDuration ();
		emit enableLaunchItems (false);
	}

	// Reset filter when launch application is changed, 
	m_filterList.clear();

	// If filter is enable, reset process filter to the launch application.
	if(isApplyProcessFilter())
	{
		onToggleApplyFilter(true);
	}

}


void SessionOptionsDlg::onProfileChange (const QString &profile)
{
	Q_UNUSED(profile);
	QString description;
	if (m_pProfiles->getProfileTexts (m_pProfileType->currentText (), NULL, &description))
	{
		m_pDescription->setText (description);
	} else {
		m_pDescription->setText ("This description is not currently available.");
	}
}


void SessionOptionsDlg::onOk ()
{
	//Save launch, working dir in history
	if (! checkForValid())
	{
		return;
	}

	// Sanity check for applying filter
	if(m_pRunControl->m_pApplyFilter->isChecked())
	{
		// Check if process filter is empty and warn
		if ( m_filterList.size() == 0 
		   ||( m_filterList.size() == 1 && m_filterList[0].isEmpty()))
		{
			unsigned int answer = 
				QMessageBox::warning(this,QString("CodeAnalyst Warning"),
				QString("Apply Process Filter is selected, but Process Filter List is empty.\n")
					+ "CodeAnalyst will disable process filtering.\n"
					+ "Are you sure you want to continue?\n",
				QMessageBox::Yes, QMessageBox::No);	
			if(answer != QMessageBox::Yes )
			{
				// Reconfigure
				m_pRunControl->m_pApplyFilter->setFocus();	
				return;
			}else{
				m_pRunControl->m_pApplyFilter->setChecked(false);	
			}
		}else{
			// Sanity check to see if the launch string is in the process filter list
			QString txt = m_pLaunch->currentText().section("\"",1,1);
			bool foundLaunch = false;
			// Insert each filter path to the view
			QStringList::iterator it = m_filterList.begin();
			QStringList::iterator it_end = m_filterList.end();
			for(; it != it_end ; it++)
			{
				if(txt.compare(*it) == 0)
				{
					foundLaunch = true;
					break;
				}
			}

			if(!foundLaunch)	
			{
				unsigned int answer = 
				QMessageBox::warning(this,QString("CodeAnalyst Warning"),
				QString("Process Filter List does not contain the application to be launched.\n")
					+ "This might result in no profiling data shown.\n"
					+ "Are you sure you want to continue?\n",
				QMessageBox::Yes, QMessageBox::No);	
				if(answer == QMessageBox::No )
				{
					// Reconfigure
					m_pRunControl->m_pApplyFilter->setFocus();	
					return;
				}
			}
		}
	}

	CATuneOptions opts;
	opts.setShowRunOptions (m_pRunControl->isVisible ());
	saveToProject();
	accept();
}


void SessionOptionsDlg::onCheckDuration()
{
	if (m_pRunControl->m_pProfileDuration->isChecked())
	{
		m_lastDuration = m_pRunControl->m_pDuration->text();
		m_pRunControl->m_pDuration->setText ("0");
		m_pRunControl->m_pDuration->setEnabled (false);
	} else {
		m_pRunControl->m_pDuration->setEnabled (true);
		if (m_pRunControl->m_pDuration->text().toUInt() > 0)
			return;	
		else if (m_lastDuration.toUInt() > 0)
			m_pRunControl->m_pDuration->setText (m_lastDuration);
		else
			m_pRunControl->m_pDuration->setText ("20");
	}
}


void SessionOptionsDlg::initFromProject (ProfileCollection * pProfiles)
{
	//Fill in launch / work dir history
	m_pLaunch->insertStringList ( m_pProject->getLaunchHistory ());
	m_pWorkDir->insertStringList (m_pProject->getWorkDirHistory ());

	//Fill in run control
	RUN_OPTIONS runOpts;
	m_pProject->getRunOptions (&runOpts);
	m_pSessionName->setText (runOpts.sessionName);
	m_pLaunch->setCurrentText (runOpts.launch);
	m_pWorkDir->setCurrentText (runOpts.workDir);

	m_pRunControl->m_pStopOnExit->setChecked (runOpts.stopOnAppExit);
	m_pRunControl->m_pTerminateAfter->setChecked (runOpts.termAppOnDur);
	m_pRunControl->m_pStartPaused->setChecked (runOpts.startPaused);
	m_pRunControl->m_pProfileDuration->setChecked (
			(runOpts.duration == 0) && (!runOpts.launch.isEmpty()));	
	m_pRunControl->m_pDuration->setText (QString::number (runOpts.duration));
	onCheckDuration ();
	m_pRunControl->m_pStartDelay->setText ((runOpts.startDelay < 0)? "0" : QString::number (runOpts.startDelay));

	// Fill in proc filter information	
	m_filterList = m_pProject->getProcessFilter();
	if(m_pProject->isApplyProcessFilter())
	{
		m_pRunControl->m_pApplyFilter->setChecked(true);
		m_pRunControl->m_pApplyFilterAdvanceBtn->setEnabled(true);
	}else{
		m_pRunControl->m_pApplyFilter->setChecked(false);
		m_pRunControl->m_pApplyFilterAdvanceBtn->setEnabled(false);
	}

	/*
	* NOTE: Here, we have to constrain the affinity mask 
	*	 to only available number of CPUs on this system.
	*/
	AffinityMask_t afMask = m_pProject->getAffinityMask();
	AffinityMask_t tmpMask = (unsigned int) GET_AFFINITYMASK_FOR_X_CPU( XpGetNumCpus() );
	afMask = afMask & tmpMask;	

	QString afStr;
	if(afMask == 0)
		afStr = QString("0x") + QString::number(tmpMask,16);
	else	
		afStr = QString("0x") + QString::number(afMask,16);
	m_pRunControl->m_pAffinityValue->setText(afStr);

	//fill in profiles
	fillProfiles (pProfiles);
	m_pOk->setFocus();
}


/*virtual */ 
void SessionOptionsDlg::fillProfiles (ProfileCollection * pProfiles)
{
	m_pProfiles = pProfiles;
	m_pProfileType->insertStringList (m_pProfiles->getListOfProfiles ());
	QString curProfile = m_pProject->getLastProfile ();
	if (!curProfile.isEmpty())
	{
		// Search if exists
		for (int i = m_pProfileType->count()-1 ; i >= 0 ; i--)
		{
			if(m_pProfileType->text(i) == curProfile)
			{
				m_pProfileType->setCurrentItem (i);
			}
		}
	} else {
		m_pProfileType->setCurrentItem (0);
	}
	curProfile = m_pProfileType->currentText ();

	QString description;
	if (m_pProfiles->getProfileTexts (curProfile, NULL, &description))
	{
		m_pDescription->setText (description);
	} else {
		m_pDescription->setText ("This description is not currently available.");
	}
}


void SessionOptionsDlg::saveToProject ()
{
	//Fill in run control
	RUN_OPTIONS runOpts;
	runOpts.sessionName = m_pSessionName->text();
	runOpts.launch = m_pLaunch->currentText ();
	runOpts.workDir = m_pWorkDir->currentText ();

	runOpts.stopOnAppExit = m_pRunControl->m_pStopOnExit->isChecked ();
	runOpts.termAppOnDur = m_pRunControl->m_pTerminateAfter->isChecked ();
	runOpts.startPaused = m_pRunControl->m_pStartPaused->isChecked ();
	runOpts.duration = m_pRunControl->m_pDuration->text ().toInt();
	runOpts.startDelay = m_pRunControl->m_pStartDelay->text ().toInt();
	runOpts.affinity = m_pRunControl->m_pAffinityValue->text ().toUInt(0,0);

	m_pProject->setRunOptions (runOpts);

	m_pProject->setLastProfile (m_pProfileType->currentText());

	m_pProject->setApplyProcessFilter(m_pRunControl->m_pApplyFilter->isChecked());

	m_pProject->setProcessFilter(m_filterList);
}


bool SessionOptionsDlg::checkForValid ()
{
	QString sessionName = m_pSessionName->text();
	if(sessionName.isEmpty())
	{
		QMessageBox::critical (this, "CodeAnalyst Error",
			"Please specify session name.");
		m_pSessionName->setFocus ();
		return false;
	}

	QRegExp rex ("[/:\\\\]");
	if (rex.search (sessionName) != -1)
	{
		QMessageBox::critical (this, "CodeAnalyst Error",
			"The session name must not contain the characters ':', '\\', or '/'");
		m_pSessionName->setFocus ();
		return false;
	}

	//get the part of the launch string between the first set of quotes
	QString launchString = m_pLaunch->currentText ();
	QString launchApp = launchString.section ('\"', 1, 1);
	if (!launchApp.isEmpty())
	{
		if (!QFile::exists (launchApp))
		{
			QMessageBox::critical (this, "CodeAnalyst Error", 
				"Unable to find the application to launch, please locate it.");
			return false;
		}

#ifndef DISABLE_CA_JVMTIA
		if (launchApp.endsWith ("java"))
		{
		    verifyJavaAgent (verifyJavaBitness(launchApp));
		}
#endif

	} else {
		//if there is no "" in the launch string, notify the user
		if (!launchString.isEmpty())
		{
			m_pLaunch->setFocus ();
			QMessageBox::critical (this, "CodeAnalyst Error", 
				"Your specified launch command needs to be formated like:\n"
				"\"<path>/<application>\" <arguments>");
			return false;

		}
	}
	if (!m_pWorkDir->currentText ().isEmpty())
	{
		QDir working (m_pWorkDir->currentText ());
		if (!working.exists())
		{
			QMessageBox::critical (this, "CodeAnalyst Error", 
				"Your specified working directory does not exist, please choose an existing directory to launch from.");
			return false;
		}
	}

	/*
	* Check if affinity is valid
	*/
	unsigned int affinityValue = m_pRunControl->m_pAffinityValue->text().toUInt(0,0);
	unsigned int tmp = XpGetNumCpus();
	if(affinityValue <= 0)
	{
		QMessageBox::critical (this, "CodeAnalyst Error", 
			"Invalid CPU Affinity Bitmask.");
		m_pRunControl->m_pAffinityValue->selectAll();
		m_pRunControl->m_pAffinityValue->setFocus ();
		return false;
	}
	// Check if Bitmask is > number of CPU
	else if(affinityValue > (unsigned int) (GET_AFFINITYMASK_FOR_X_CPU(tmp)))
	{
		QMessageBox::critical (this, "CodeAnalyst Error", 
			"CPU Affinity Bitmask (0x" + 
			QString::number(affinityValue,16) + 
			") is more than the number of available CPUs ("+
			QString::number(tmp) +").");

		m_pRunControl->m_pAffinityValue->setFocus ();
		return false;
	}

	/*
	 * Check if duration is valid
	 */
	bool isOk = false;
	if(!m_pRunControl->m_pProfileDuration->isChecked())
	{
		int duration = m_pRunControl->m_pDuration->text().toUInt(&isOk);

	 	if(!isOk || (duration == 0))
		{
			QMessageBox::critical(this, "CodeAnalyst Error",
				m_pRunControl->m_pDuration->text() + 
				" is an invalid input for profiling duration.");	
			
			m_pRunControl->m_pDuration->setFocus();
			return false;
		}
	}

	/*
	 * Check if delay is valid
	 */
	m_pRunControl->m_pStartDelay->text().toUInt(&isOk,0);
	if (!isOk)
	{
		QMessageBox::critical(this, "CodeAnalyst Error",
			m_pRunControl->m_pStartDelay->text() + 
			" is an invalid input for start delay value.");	
		
		m_pRunControl->m_pStartDelay->setFocus();
		return false;
	}

	return true;
} //SessionOptionsDlg::checkForValid


void SessionOptionsDlg::verifyJavaAgent (QString bitness)
{
	//get everything past the second quote
	QString arguments = m_pLaunch->currentText ().section ('\"', 2);

	//check for pi agent.
	if ((!arguments.contains ("CAJVMPIA", false)) &&
		(!arguments.contains ("CAJVMTIA", false)))
	{
		QString launch = m_pLaunch->currentText ();
		int secondquote	= launch.find ("\"", 1);

		QString CAJVMTIA_BIT_SO = CAJVMTIA + bitness + ".so";

		launch.insert (secondquote + 1, CAJVMTIA_BIT_SO);

		//Save so changing the launch string doesn't will the working dir
		QString saveWorkDir = m_pWorkDir->currentText ();
		m_pLaunch->setEditText (launch);
		m_pWorkDir->setEditText (saveWorkDir);
	}
} //SessionOptionsDlg::verifyJavaAgent

bool SessionOptionsDlg::isFile32Bit (const char* fileName)
{
    char elf_header[EI_NIDENT];
    FILE *fp;
    bool bRet = false;

    fp = fopen(fileName, "r");

    /**
 *      * The reporting libraries does not return correct path
 *           * of kernel module under 2.6.1.  This function will not
 *                * under this situation.
 *                     */
    if (NULL == fp) 
                goto system_default;

        fread(&elf_header, 16, 1, fp);
        if(ferror(fp))
        {
                bRet = false;
                fclose (fp);
                goto out;

        }
        fclose(fp);

        if (0 != strncmp(elf_header, ELFMAG, SELFMAG))
                goto system_default;

        switch (elf_header[EI_CLASS]) {
                case ELFCLASS32:
                        bRet = true; 
                        break;
                case ELFCLASS64:
                        bRet = false;
                        break;
        }
        goto out;

system_default:
#ifdef __x86_64__
        bRet = false;
#else
        bRet = true;
#endif

out:
    return bRet; 
} 

QString SessionOptionsDlg::verifyJavaBitness (QString javaFile)
{
	if(isFile32Bit(javaFile.data()))
	{
		return QString("32");	
	} else {
		return QString("64");	
	}
} //SessionOptionsDlg::verifyJavaBitness

void SessionOptionsDlg::onCpuAffinity()
{
	unsigned int numCpus = XpGetNumCpus();
	unsigned int numCpusPerSocket = XpGetNumCoresPerSocket();
	unsigned int numSockets = numCpus/numCpusPerSocket;


	m_pAffinityDlg = new CpuAffinityDlg(this);
	RETURN_IF_NULL(m_pAffinityDlg,this)

	m_pAffinityDlg->init(numSockets, numCpusPerSocket,
			m_pProject->getAffinityMask());

	if(QDialog::Accepted == m_pAffinityDlg->exec())
	{
		// If clicked OK, make changes to the session setting
		m_pProject->setAffinityMask(m_pAffinityDlg->getAffinityMask());
		QString afStr = QString("0x") + QString::number(m_pProject->getAffinityMask(),16);
		m_pRunControl->m_pAffinityValue->setText(afStr);
	}

	// Clean up
	if(m_pAffinityDlg)
	{
		delete m_pAffinityDlg;
		m_pAffinityDlg = NULL;
	}
}


void SessionOptionsDlg::onToggleApplyFilter(bool t)
{
	m_pRunControl->m_pApplyFilter->setChecked(t);
	m_pRunControl->m_pApplyFilterAdvanceBtn->setEnabled(t);

	if(t)
	{
		// If filter is enabled, we automatically put the application
		// in the filter by default if the list is empty.
		if ( m_filterList.size() == 0 
		||   ( m_filterList.size() == 1 && m_filterList[0].isEmpty() ))
		{
			m_filterList.clear();
			QString txt = m_pLaunch->currentText().section("\"",1,1);
			m_filterList.push_back(txt);
		}
	}
}

void SessionOptionsDlg::onSetAdvanceFilter()
{
	// Pop up process filter dialog
	ProcessFilterDlg *pfd = new ProcessFilterDlg(this, "");
	pfd->init(m_filterList);
	if( QDialog::Accepted == pfd->exec())
	{
		m_filterList.clear();
		m_filterList = pfd->getStringList();			
	}

	// Clean up
	if(pfd)
	{
		delete pfd;
		pfd = NULL;
	}
}

QStringList SessionOptionsDlg::getProcessFilter()
{
	return m_filterList;
}
	

bool SessionOptionsDlg::isApplyProcessFilter()
{
	return m_pRunControl->m_pApplyFilter->isChecked();
}
