//$Id: SessionOptionsDlg.h 16686 2009-06-11 15:17:57Z ssuthiku $
//  This is the interface for the SessionOptionsDlg class, which derives from
//	ISessionOptions and has different functionality for express vs normal mode

/**
*   (c) 2006 Advanced Micro Devices, Inc.
*  YOUR USE OF THIS CODE IS SUBJECT TO THE TERMS
*  AND CONDITIONS OF THE GNU GENERAL PUBLIC
*  LICENSE FOUND IN THE "GPL.TXT" FILE THAT IS
*  INCLUDED WITH THIS FILE AND POSTED AT
*  <http://www.gnu.org/licenses/gpl.html>
*  Support: codeanalyst@amd.com
*/

#ifndef SESSION_OPTIONS_DLG_H
#define SESSION_OPTIONS_DLG_H

#include <qstring.h>
#include "iSessionOptions.h"
#include "iruncontrol.h"
#include "atunefile.h"
#include "atuneoptions.h"
#include "ProfileCollection.h"
#include "CpuAffinityDlg.h"

const QString CAJVMPIA = " -XrunCAJVMPIA";
const QString CAJVMTIA_BIN_PATH = CA_BIN_DIR;
const QString CAJVMTIA = " -agentpath:" + CAJVMTIA_BIN_PATH + "/libCAJVMTIA";

class SessionOptionsDlg : public ISessionOptions
{ 
	Q_OBJECT

public:
	SessionOptionsDlg (ATuneFile *pProject, 
		QWidget* parent = 0, const char* name = 0, bool modal = true, 
		WFlags fl = WStyle_Customize | WStyle_DialogBorder | WStyle_Title 
		| WDestructiveClose);
	virtual ~SessionOptionsDlg();

	void initFromProject (ProfileCollection * pProfiles);
	QStringList getProcessFilter();
	bool isApplyProcessFilter();

private:
	void saveToProject ();
	virtual void fillProfiles (ProfileCollection * pProfiles);
	bool checkForValid ();
	void verifyJavaAgent (QString bitness);
	QString verifyJavaBitness (QString javaFile);
	bool isFile32Bit (const char* fileName);

private slots:
	void onClickEditProfile ();
	void onClickShowOptions (bool toggled);
	void onBrowseWorkDir ();
	void onBrowseLaunch ();
	void onLaunchChange (const QString & string);
	void onProfileChange (const QString &profile);
	void onOk ();
	void onCheckDuration ();
	void onCpuAffinity();
	void onToggleApplyFilter(bool t);
	void onSetAdvanceFilter();
	void onToggleStopOnExit(bool b);

signals:
	void enableLaunchItems (bool enable);

protected:
	ATuneFile *m_pProject;
	IRunControl *m_pRunControl;
	bool m_lastDurationChecked;
	QString m_lastDuration;
	ProfileCollection * m_pProfiles;
	CpuAffinityDlg *m_pAffinityDlg;
	QStringList m_filterList;
};
#endif
