//$Id: application.cpp,v 1.34 2006/05/15 22:09:22 jyeh Exp $
// The main gui that the user will be interacting with

/**
*   (c) 2003-2006 Advanced Micro Devices, Inc.
*  YOUR USE OF THIS CODE IS SUBJECT TO THE TERMS
*  AND CONDITIONS OF THE GNU GENERAL PUBLIC
*  LICENSE FOUND IN THE "GPL.TXT" FILE THAT IS
*  INCLUDED WITH THIS FILE AND POSTED AT
*  <http://www.gnu.org/licenses/gpl.html>
*  Support: codeanalyst@amd.com
*/

#include "stdafx.h"
#include <stdlib.h>
#include <qradiobutton.h>
#include <qtextedit.h>
#include <qimage.h>
#include <qfile.h>
#include <qvaluelist.h>
#include <assert.h>
#include <unistd.h>
#include <string.h>
#include <qstringlist.h>

#include "atuneoptionsdlg.h"
#include "application.h"
#include "auxil.h"
#include "xp.h"
#include "optionsdlg.h"
#include "ebpdef.h"
#include "sysInfoDlg.h"
#include "AboutDlg.h"
#include "oprofile_interface.h"
#include "translatedlg.h"
#include "tbsreader.h"
#include "ImportWizard.h"
#include "SessionOptionsDlg.h"
#include "TimerCfg.h"
#include "EventCfg.h"
#include "CfgIbs.h"
#include "ManageCfgs.h"
#include "ViewCfg.h"
#include "ViewCollection.h"
#include "EventMaskEncoding.h"
#include "NewDiffSessionDlg.h"

#include "catranslate_gui_display.h"
#include "opxmldata_handler.h"
#include "config.h"
#include "helperAPI.h"

#define _CODEANALYST_ICONS_
#include "xpm.h"
#undef  _CODEANALYST_ICONS_


#define DEFAULT_CAPTION "Advanced Micro Devices - CodeAnalyst Analyzer"
#define MAX_MRU_ITEMS   5
#define EMPTY_MRU_LIST  "E;E;E;E;E;"
#define CA_EXT "CodeAnalyst Workspace (*.caw)"

#define DIRSLASH "/"

#define NO_SAMPLE_MSG "Oprofile collected no samples.\n" \
		"Suggestion:\n" \
		"- Check the profiling duration\n" \
		"- Check number of counts for events\n" \
		"- Check \"Start delay period\" and \"Start with the profiling paused\" options.\n" 


extern EventMaskMap g_EventMaskMap;

ApplicationWindow::ApplicationWindow ()
:  QMainWindow (0, "example application main window", WType_TopLevel
				| WStyle_SysMenu | WDestructiveClose)
{

	// Check CPU family for the current running platform
	m_this_cpuFamily = 0;
	m_this_cpuModel  = 0;
	char VenderId[15];
	unsigned long stepping, features;
	CpuId(VenderId, &m_this_cpuFamily, &m_this_cpuModel, &stepping, &features);

	// Get oprofile events directory 
	QString file = helpGetEventFile(m_this_cpuFamily, m_this_cpuModel);

	m_eventFile.open(file);

	m_euid = geteuid();
	m_pATuneFile = 0;
	m_nDocNum = 1;
	m_bFirstDialog = TRUE;
	m_bLegacyTimer = TRUE;
	m_SamplingState = SS_IDLE;
	m_translate_progress_dlg = NULL;
	m_pSession = NULL;

	DWORD font_size = FONT_SIZE_DEFAULT;
	CATuneOptions ao;

	ao.getFontSize (&font_size);
	qApp->setFont (QFont (qApp->font().family(), font_size), TRUE);

	m_tick_per_sec = getTick ();


	//////////////////////////////////////////////////////

	m_profiles.readAvailableProfiles();

	QPixmap openIcon, saveIcon;
	QSize size;
	QIconSet icon;
	icon.setIconSize (QIconSet::Large, size);

	setIcon (QPixmap (amdArrowLogo));

	// ////////////////////////////////////////////////////////////////
	// Instantiate the workspace
	m_pSplitter = new QSplitter (this);
	m_pProjWnd = new ProjectWindow (m_pSplitter);
	m_pVBox = new QVBox (m_pSplitter);
	ws = new QWorkspace (m_pVBox);

	// When we cascade the sessions...
	ws->setScrollBarsEnabled (true);

	m_pVBox->setFrameStyle (QFrame::StyledPanel | QFrame::Sunken);
	m_pProjWnd->resize (200, m_pProjWnd->height ());

	QObject::connect (ws,
		SIGNAL (windowActivated (QWidget *)),
		SLOT (onWindowActivated (QWidget *)));

	connect( m_pProjWnd, SIGNAL (SessionDeletedAndReinit(TRIGGER, QString)),
		SLOT (onSessionDeleted (TRIGGER, QString)));
	connect( m_pProjWnd, SIGNAL (SessionDeletedOnly(TRIGGER, QString)),
		SLOT (onSessionDeletedOnly (TRIGGER, QString)));
	connect( m_pProjWnd, SIGNAL (reinitProjectWindow()),
		SLOT (initProjectWindow ()));


	QObject::connect( m_pProjWnd, SIGNAL(SessionDblClicked (TRIGGER, QString)),
		SLOT(onSessionDblClicked (TRIGGER, QString)) );
	QObject::connect( m_pProjWnd, SIGNAL(SessionRenamed (TRIGGER, QString, QString)),
		SLOT(onSessionRenamed (TRIGGER, QString, QString)) );

	QObject::connect (m_pProjWnd,
		SIGNAL (tbpSessionProperties (QString)),
		SLOT (onTbsSessionProperties (QString)));

	QObject::connect (m_pProjWnd,
		SIGNAL (ebpSessionProperties (QString)),
		SLOT (onEbsSessionProperties (QString)));

	QObject::connect (m_pProjWnd, 
		SIGNAL (ibsSessionProperties (QString)), 
		SLOT (onIbsSessionProperties (QString)));

	connect (m_pProjWnd, SIGNAL (CopySessionToCurrent (TRIGGER, QString)),
		SLOT (OnCopySessionToCurrent (TRIGGER, QString)));

	connect (m_pProjWnd, SIGNAL (openSessionInDiffAnalyst(DiffSessionInfo *)),
		SLOT (OnOpenDiffAnalyst(DiffSessionInfo *)));

	setCentralWidget (m_pSplitter);

	// /////////////////////////////////////////////////////////////////////
	// Fix our Status Bar
	m_pStatusMsg = new QLabel (statusBar (), "Hello There");

	// stretch is used to compute a suitable size for widget as the status
	// bar grows and shrinks.
	int stretch = 4;
	statusBar ()->addWidget (m_pStatusMsg, stretch);

	m_pStatusProgress = new QProgressBar (statusBar (), "Progress Bar");
	stretch = 1;
	statusBar ()->addWidget (m_pStatusProgress, stretch);

	m_pStatusProgress->setCenterIndicator (FALSE);

	setDockMenuEnabled (true);
	// /////////////////////////////////////////////////////////////////////
	// Add the File Tool Bar
	fileTools = new QToolBar (this, "File Tool Bar");
	addToolBar (fileTools, tr ("File Tool Bar"), Top, TRUE);

	new QToolButton (QIconSet (XpmFileNew), "New", QString::null, this,
		SLOT (onFileNew ()), fileTools, "New");
	new QToolButton (QIconSet (XpmFileOpen), "Open", QString::null, this,
		SLOT (onFileOpen ()), fileTools, "Open");

	m_SaveButt = new QToolButton (QIconSet (XpmFileSave), "Save",
		QString::null, this, SLOT (onFileSave ()),
		fileTools, "Save");
	m_SaveButt->setEnabled (FALSE);

	////////////////////////////////////////////////////////////////////////
	// Add the Tools Toolbar
	m_pToolsToolBar = new QToolBar( this, "Tools Toolbar" );
	addToolBar( m_pToolsToolBar, tr( "Tools Toolbar" ), Top, FALSE);

	icon = QIconSet (QPixmap(samplingsettings));
	m_ToolsToolId[TM_SETTINGS] = new QToolButton( icon, "Session Settings", QString::null, this,
		SLOT( OnSessionSettings() ), m_pToolsToolBar );

	QImage img;
	icon = QIconSet ( QPixmap(appopdialog));
	img.loadFromData( disappopdialog, sizeof( disappopdialog ), "PNG" );
	m_ToolsToolId[TM_CAOPTIONS] = new QToolButton( icon, "CodeAnalyst Options", QString::null, this,
		SLOT( onToolsATuneOptions() ), m_pToolsToolBar );

	icon = QIconSet (QPixmap (configdialog));
	img.loadFromData( disconfigdialog, sizeof( disconfigdialog ), "PNG" );
	m_ToolsToolId[TM_EDITPROFILES] = new QToolButton( icon, "Configuration Management", QString::null, this,
		SLOT( OnProfileManage() ), m_pToolsToolBar );

	icon = QIconSet (QPixmap (viewconfig));
	img.loadFromData( disviewconfig, sizeof( disviewconfig ), "PNG" );
	m_ToolsToolId[TM_EDITVIEWS] = new QToolButton( icon, "View Management", QString::null, this,
		SLOT( OnViewManage() ), m_pToolsToolBar );

	icon = QIconSet (QPixmap (sessionDiffIcon));
	m_ToolsToolId[TM_DIFFANALYST] = new QToolButton( icon, "Create Diff Session", QString::null, this,
		SLOT( OnOpenDiffAnalyst() ), m_pToolsToolBar );


	m_ToolsToolId[TM_SETTINGS]->setEnabled( FALSE );
	m_ToolsToolId[TM_DIFFANALYST]->setEnabled( FALSE );

	m_pToolsToolBar->show ();

	// ////////////////////////////////////////////////////////////////////
	// Add Sampling Tool Bar
	//
	pSamplingToolBar = new QToolBar (this, "Sampling Tool Bar");

	addToolBar (pSamplingToolBar, "Sampling Tool Bar", Top, FALSE);

	icon = QIconSet (QPixmap (samplingstart));
	m_SamplingToolId[SM_START] = new QToolButton (icon, "Start",
		QString::null, this,
		SLOT (onSamplingStart ()),
		pSamplingToolBar);

	icon = QIconSet (QPixmap (samplingpause));
	m_SamplingToolId[SM_PAUSE] = new QToolButton( icon, "Pause",
		QString::null, this,
		SLOT( onSamplingPause ()),
		pSamplingToolBar );

	icon = QIconSet (QPixmap (samplingstop));
	m_SamplingToolId[SM_STOP] = new QToolButton (icon, "Stop",
		QString::null, this,
		SLOT (onSamplingStop ()),
		pSamplingToolBar);

	m_SamplingToolId[SM_START]->setEnabled (FALSE);
	m_SamplingToolId[SM_PAUSE]->setEnabled (FALSE);
	m_SamplingToolId[SM_STOP]->setEnabled (FALSE);
	m_SamplingToolId[SM_PAUSE]->setToggleButton( TRUE );

	pSamplingToolBar->addSeparator ();
	m_pProfileBox = new QComboBox (pSamplingToolBar);
	m_pProfileBox->insertStringList (m_profiles.getListOfProfiles ());
	connect (m_pProfileBox, SIGNAL (activated (const QString &)),
		SLOT (OnProfileChange (const QString &)));
	m_pProfileBox->setEnabled (false);

	// ////////////////////////////////////////////////////////////////////
	// Insert Items under the file menu
	m_pFileMenu = new QPopupMenu (this);
	menuBar ()->insertItem ("&File", m_pFileMenu);

	m_pFileMenu->insertItem (QIconSet (QPixmap (XpmFileNew)), "&New", this,
		SLOT (onFileNew ()), CTRL + Key_N);
	m_pFileMenu->insertItem (QIconSet (QPixmap (XpmFileOpen)), "&Open",
		this, SLOT (onFileOpen ()), CTRL + Key_O);
	m_SaveId = m_pFileMenu->insertItem (QIconSet (QPixmap (XpmFileSave)),
		"&Save", this, SLOT (onFileSave ()),
		CTRL + Key_S);
	m_pFileMenu->setItemEnabled (m_SaveId, false);

	m_pFileMenu->insertSeparator ();

	// export data
	m_ExportDataId = m_pFileMenu->insertItem ("&Export...", this,
		SLOT (onExportData ()));
	m_pFileMenu->setItemEnabled (m_ExportDataId, false);

	// import data
	m_ImportDataId = m_pFileMenu->insertItem ("&Import...", this,
		SLOT (onImportData ()));
	m_pFileMenu->setItemEnabled (m_ImportDataId, false);

	m_pFileMenu->insertSeparator ();

	m_CloseId = m_pFileMenu->insertItem ("&Close", this,
		SLOT (onFileClose ()), CTRL + Key_W);
	m_pFileMenu->setItemEnabled (m_CloseId, false);

	m_pFileMenu->insertItem ("&Quit", this, SLOT (close ()), CTRL + Key_Q);

	QObject::connect (qApp, SIGNAL (aboutToQuit ()), this,
		SLOT (onAboutToQuit ()));

	QObject::connect (m_pFileMenu, SIGNAL (activated (int)),
		SLOT (onMruItemSelected (int)));

	insertMruItems ();

	// /////////////////////////////////////////////////////////////////////
	// Initialize the Sampling Menu Item
	m_pSamplingMenu = new QPopupMenu (this);

	menuBar ()->insertItem ("&Profile", m_pSamplingMenu);

	m_SamplingMenuId[SM_START]
	= m_pSamplingMenu->insertItem (QPixmap (samplingstart), "&Start",
		this, SLOT (onSamplingStart ()));

	m_pSamplingMenu->insertSeparator ();

	m_SamplingToolId[SM_PAUSE]->setToggleButton( TRUE );
	m_SamplingMenuId[SM_PAUSE]
	= m_pSamplingMenu->insertItem ( QPixmap (samplingpause), "&Pause",
		this, SLOT( onSamplingPause()  ));

	m_pSamplingMenu->insertSeparator ();
	m_SamplingMenuId[SM_STOP]
	= m_pSamplingMenu->insertItem (QPixmap (samplingstop), "S&top",
		this, SLOT (onSamplingStop ()));

	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_START], FALSE);
	m_pSamplingMenu->setItemEnabled( m_SamplingMenuId[SM_PAUSE], FALSE );
	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_STOP], FALSE);

	// /////////////////////////////////////////////////////////////////////
	// Initialize the Tools Menu
	m_pToolsMenu = new QPopupMenu (this);

	menuBar ()->insertItem ("&Tools", m_pToolsMenu);

	m_ToolsMenuId[TM_SETTINGS]  = m_pToolsMenu->insertItem( QPixmap(samplingsettings),
		"S&ession Settings", this, SLOT( OnSessionSettings()  ));

	m_ToolsMenuId[TM_CAOPTIONS] = m_pToolsMenu->insertItem( QPixmap(appopdialog), "&CodeAnalyst Options",
		this, SLOT( onToolsATuneOptions()) );
	m_pToolsMenu->insertSeparator();
	m_ToolsMenuId [TM_EDITPROFILES] =  m_pToolsMenu->insertItem( QPixmap (configdialog), "Configuration &Management",
		this, SLOT( OnProfileManage()) );
	m_ToolsMenuId [TM_EDITVIEWS] =  m_pToolsMenu->insertItem( QPixmap (viewconfig), "&View Management",
		this, SLOT( OnViewManage()) );
	m_ToolsMenuId [TM_DIFFANALYST] =  m_pToolsMenu->insertItem( QPixmap (sessionDiffIcon), "Create &Diff Session",
		this, SLOT( OnOpenDiffAnalyst()) );

	m_pToolsMenu->setItemEnabled( m_ToolsMenuId[TM_SETTINGS], FALSE );
	m_pToolsMenu->setItemEnabled( m_ToolsMenuId[TM_DIFFANALYST], FALSE );

	// /////////////////////////////////////////////////////////////////////
	// Insert items under the windows menu
	m_pWindowsMenu = new QPopupMenu (this);
	m_pWindowsMenu->setCheckable (TRUE);
	connect (m_pWindowsMenu, SIGNAL (aboutToShow ()), this,
		SLOT (windowsMenuAboutToShow ()));

	menuBar ()->insertItem ("&Windows", m_pWindowsMenu);

	// /////////////////////////////////////////////////////////////////////
	// Insert items under the help menu
	m_pHelpMenu = new QPopupMenu (this);
	menuBar ()->insertItem ("&Help", m_pHelpMenu);

	m_pHelpMenu->insertItem ("&About", this, SLOT (onHelpAbout ()), Key_F1);
	m_pHelpMenu->insertSeparator ();
	m_pHelpMenu->insertItem ("&Help", this, SLOT (onHelpContents ()));
	m_pHelpMenu->insertSeparator ();
	m_pHelpMenu->insertItem ("&System Info", this, SLOT (onHelpSysInfo ()));

	m_translate = NULL;
	m_ca_display = NULL;
	m_pProgress = NULL;
}


ApplicationWindow::~ApplicationWindow ()
{
	deleteOprofileTranslateObjects();
	DELETE_IF_NOT_NULL(m_pProgress);
}


void ApplicationWindow::deleteOprofileTranslateObjects()
{
	//    DELETE_IF_NOT_NULL(m_translate);
	//    DELETE_IF_NOT_NULL(m_ca_display);
}



bool ApplicationWindow::
createOprofileTranslateObjects()
{
	if (NULL != m_translate)
	{
		delete m_translate;
		m_translate = NULL;
	}
	m_translate = new CATranslate();
	RETURN_FALSE_IF_NULL (m_translate, this);

	if (NULL != m_ca_display)
	{
		delete m_ca_display;
		m_ca_display = NULL;
	}
	m_ca_display = new ca_translate_gui_display();
	RETURN_FALSE_IF_NULL (m_ca_display, this);

	return true;
}


/////////////////////////////////////////////////////////////////////////////
// customEvent()
// -------------
// Our Custom Event that we implement to notify the app that the sampling
// duration has expired
//
void ApplicationWindow::customEvent (QCustomEvent * pCustEvent)
{
	switch (pCustEvent->type())
	{
	case EVENT_DELAY_FINISH:
		if(!m_pStatusProgress) break;	
		m_pStatusProgress->reset ();

		if (SS_DELAY != m_SamplingState || !setSamplingState(SS_RUNNING))
			setSamplingState (SS_IDLE);
		if (m_pSession->startPaused)
			onSamplingPause();
		break;

	case EVENT_DURATION_FINISH:
		if(!m_pStatusProgress) break;	
		m_pStatusProgress->reset ();
		setSamplingState (SS_FINISHED);
		break;

	case EVENT_SET_PROGRESS:
		if(!m_pStatusProgress) break;	
		m_pStatusProgress->setProgress ((long) pCustEvent->data());
		break;

	case EVENT_RESET_PROGRESS:
		if(!m_pStatusProgress) break;	
		m_pStatusProgress->reset();
		break;

	case EVENT_SET_STEPS:
		if(!m_pStatusProgress) break;	
		m_pStatusProgress->setTotalSteps ((long) pCustEvent->data());
		break;

	case EVENT_LAUNCH_ERROR:
		{
			QString * temp = (QString *) pCustEvent->data();
			QMessageBox::critical (this, "CodeAnalyst profile error", *temp);
			delete temp;
			break;
		}
	case EVENT_UPDATE_PROGRESS_BEGIN:
		{
			m_translate_progress_dlg->m_pOk->setEnabled(false);
			m_translate_progress_dlg->canBeClosed = false;

			break;
		}
	case EVENT_UPDATE_PROGRESS_DONE:
		{
			m_translate_progress_dlg->m_pOk->setEnabled(true);
			m_translate_progress_dlg->m_pOk->setFocus();
			m_translate_progress_dlg->canBeClosed = true;
			break;
		}
	case EVENT_UPDATE_PROGRESS:
		{
			QString * temp = (QString *) pCustEvent->data();
			if (NULL != m_translate_progress_dlg)
			{
				m_translate_progress_dlg->translate_edit->append (*temp);
				m_translate_progress_dlg->translate_edit->scrollToBottom ();
			}
			delete temp;
			break;
		}
	case EVENT_UPDATE_ERROR:
		{
			QString * temp = (QString *) pCustEvent->data();
			if (NULL != m_translate_progress_dlg)
			{
				m_translate_progress_dlg->translate_edit->setColor (Qt::red);
				m_translate_progress_dlg->translate_edit->append (*temp);
				m_translate_progress_dlg->translate_edit->setColor (Qt::black);
				m_translate_progress_dlg->translate_edit->scrollToBottom ();
			}
			delete temp;
			break;
		}
	default:
		break;
	}
}


/*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ApplicationWindow::closeEvent()
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* This routine prevents the user from closing atune if a sampling session
* is taking place.
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
void ApplicationWindow::closeEvent (QCloseEvent * pCloseEvent)
{
	if (sampling ())
	{
		QMessageBox::information (this, "Hold on...", "Please wait "
			"for the sampling session to end, "
			"or end it manually.");
		pCloseEvent->ignore ();
	}
	else
	{
		pCloseEvent->accept ();
	}
}


/*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ApplicationWindow::sampling()
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Determines is a sampling session is going on
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
bool ApplicationWindow::sampling ()
{
	return (SS_IDLE != m_SamplingState);
}


////////////////////////////////////////////////////////////////////////////
// getCurrentFilePath()
// --------------------
//
QString ApplicationWindow::getCurrentFilePath ()
{
	return m_pATuneFile->getPath ();
}


///////////////////////////////////////////////////////////////////////////////
// onFileNew()
// -----------
// Display the Project Options Dialog to allow the user to select the
// appropriate options
//
void ApplicationWindow::onFileNew ()
{
	if( sampling() )
	{
		QMessageBox::warning (this, "Hold on...", 
			"Please wait for the profiling session to end, or end it manually." );
		return ;
	}

	// show the dialog
	m_bFirstDialog = FALSE;

	bool bKeepAsking = false;
	QString newProject;
	NewProjectDlg * pDlg = new NewProjectDlg (this);
	CATuneOptions ao;
	do
	{
		//If a default project directory is available use it,
		QString defaultProjectDir;
		ao.getDefaultProjectDir (defaultProjectDir);
		if (defaultProjectDir.isEmpty ())
		{
			//otherwise use the user's home directory
			defaultProjectDir = QDir::home ().path();
			defaultProjectDir.replace ('\\', '/');
			defaultProjectDir += "/AMD/CodeAnalyst";
		}
		pDlg->m_pProjectDir->setText (defaultProjectDir);

		pDlg->exec();
		if( pDlg->result() == QDialog::Rejected )
		{
			delete pDlg;
			return;
		}

		// Force projDir to end with "/"
		newProject = pDlg->m_pProjectDir->text();
		if (!newProject.endsWith ("/"))
		{
			newProject += "/";
		}

		// creates a directory based on the project name
		if (! newProject.endsWith (pDlg->m_pProjectName->text() + "/"))
			newProject += pDlg->m_pProjectName->text() + "/";

		// Append the projectFileName.caw 
		newProject += pDlg->m_pProjectName->text();
		if (! newProject.endsWith (".caw"))
		{
			newProject += ".caw";
		}

		bKeepAsking = false;
		if (QFile::exists (newProject))
		{
			if (QMessageBox::Yes  != QMessageBox::question (this, "File already exists",
				"The project file already exists, do you want to overwrite it?",
				QMessageBox::Yes, QMessageBox::No))
			{
				bKeepAsking = true;
			}
		}
	} while (bKeepAsking);

	closeAllWindows ();

	QString temp = pDlg->m_pProjectDir->text();
	ao.setDefaultProjectDir (temp );
	delete pDlg;

	//create  project
	if(!ATuneFile::create (newProject))
		return;	

	//Load the new file
	LoadFile (newProject);

	//Pop up profile dialog
	OnSessionSettings ();
}


/*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ApplicationWindow::onFileOpen()
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Event handler for selecting File-Open from the menu, or hitting the
* FileOpen button on the tool bar.
*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
void ApplicationWindow::onFileOpen ()
{
	QString dir = "";
	CATuneOptions ao;

	if( sampling() )
	{
		QMessageBox::warning (this, "Hold on...", 
			"Please wait for the profiling session to end, or end it manually." );
		return ;
	}

	ao.getDefaultProjectDir (dir);
	QString fn = QFileDialog::getOpenFileName (dir, CA_EXT, this);

	if (!fn.isEmpty ())
	{
		fn.replace ('\\', '/');					  //Make sure path has front slashes
		LoadFile (fn);
	}
}


///////////////////////////////////////////////////////////////////////////////
// initProjectWindow()
// -------------------
// Enumerates all tbs sessions listed and adds them to the project tree
//
void ApplicationWindow::initProjectWindow ()
{
	m_pProjWnd->deleteAllItems ();
	m_pProjWnd->initialize ();

	// Get the TBS Session Names
	QStringList sessions = m_pATuneFile->getSessionList (TIMER_TRIGGER);
	QStringList::Iterator it = sessions.begin ();

	for (unsigned int i = 0; i < sessions.count (); ++i, ++it)
	{
		m_pProjWnd->insertTbpSamplingSession ((*it).data ());
	}

	// Get the EBS Session Names
	sessions = m_pATuneFile->getSessionList (EVENT_TRIGGER);
	it = sessions.begin ();

	for (unsigned int i = 0; i < sessions.count (); ++i, ++it)
	{
		m_pProjWnd->insertEbpSamplingSession ((*it).data ());
	}

	// Get the IBS Session Names
	sessions = m_pATuneFile->getSessionList (IBS_TRIGGER);
	it = sessions.begin ();

	for (unsigned int i = 0; i < sessions.count (); ++i, ++it)
	{
		m_pProjWnd->insertIbsSamplingSession ((*it).data ());
	}

	// Set the Column caption for the window
	m_pProjWnd->setCaption (m_pATuneFile->getName ());
	m_pProjWnd->expandTbp ();
	m_pProjWnd->expandEbp ();
	m_pProjWnd->expandIbs ();
}


void ApplicationWindow::targetExit()
{
#ifdef _DEBUG_
	qDebug("target exists");
#endif
}


///////////////////////////////////////////////////////////////////////////////
// StartSampling()
// ---------------
//
void ApplicationWindow::onSamplingStart ()
{
	//sync up to reduce chance of io error
	qApp->syncX();

	/*************************  
	 * BEGIN: DO NOT CHANGE ORDER 
	 */
	/* 1 Check if user has permission */
	if(!m_opIf.havePermissionToProfile())
	{
		QString errMsg 	= QString("Error starting profile:\n\n")
				+ "This user does not have sufficient permission to run a profile.\n\n"
				+ "Please make sure :\n"
				+ "    - You are root or a member of \"amdca\" group.\n" 
				+ "    - amdca group has the permission to run\n\n"
				+ "         sudo " + CA_SBIN_DIR + "/ca_oprofile_controller\n\n"
				+ "Please use \"ca_user_manager\" to manage CodeAnalyst users.";
		QMessageBox::critical(this, "CodeAnalyst Error", errMsg);
		return;	
	}

	/* 2. Check to make sure that CodeAnalyst service is setup and ready.
	 * otherwise, we will try to start it.
	 */
	if (!m_opIf.checkOprofileDriverStatus() && !m_opIf.loadOprofileDriver())
	{
		QMessageBox::critical (this, "Error starting profile",
			"Checking OProfile driver (/dev/oprofile) returns failure\n"
			" and an attempt to load the driver also failed.\n."
			"CodeAnalyst will not be able to run profile.\n\n."
			"Please make sure :\n"
			"  - The user is a member of group amdca\n."
			"  - OProfle driver is installed and can be started properly\n."
		);
		return;
	}

	/* 3. Setup OProfile infrastructures. Must be done after driver loaded.*/
	if(!m_opIf.do_setup())  {
		QMessageBox::critical (this, "Error starting profile",
			QString("Failed to setup OProfile.\n"));
		return;
	}

	/* 4. This check is mainly for the case when running on older kernel that doesn't
	 * have support for family10 yet. In this case we suggest them to use the CAKM
	 */
	if(!isCorrectDriverForGH())
	{
		QString errMsg 	= QString("Error starting profile:\n\n")
				+ "Oprofile driver is not loaded.\n\n" 
				+ "or\n\n"
				+ "Incorrect Oprofile driver is loaded.\n"
				+ "CodeAnalyst requires a modified version\n"
				+ "of \"oprofile\" driver to be installed on this system.\n"  
				+ "Please see INSTALLATION instruction.";
		QMessageBox::critical(this, "CodeAnalyst Error", errMsg);
		return;	
	}
	/*************************  
	 * END: DO NOT CHANGE ORDER 
	 */

	// Get the options from the workspace file
	QString profile = m_pATuneFile->getLastProfile ();
	if(!m_profiles.profileExists(profile)) {
		// the DC configuration does not exist in available configs.
		// then use the current config.
		profile = m_pProfileBox->currentText();
		m_pATuneFile->setLastProfile(profile);
	}

	// set the current session info (for the duration of the profile) 
	// appropriately
	TRIGGER profileType = m_profiles.getProfileType (profile);
	QString file_ext = "";
	switch (profileType)
	{
	case TIMER_TRIGGER:
		m_pSession = static_cast<SESSION_OPTIONS *>(new TBP_OPTIONS ());
		if (NULL == m_pSession )
		{
			QMessageBox::critical (this, "CodeAnalyst Error", 
							DLG_ALLOC_ERROR_MSG);
			return;
		}
		file_ext = TBS_FILE_EXT;
		break;

	case EVENT_TRIGGER:
		if (!isCpuAMD ())
		{
			QMessageBox::critical (this, "CodeAnalyst Error",
				"Your system is not supported for this type of profile.");
			return;
		}
		m_pSession = static_cast<SESSION_OPTIONS *>(new EBP_OPTIONS ());
		if (NULL == m_pSession )
		{
			QMessageBox::critical (this, "CodeAnalyst Error", 
							DLG_ALLOC_ERROR_MSG);
			return;
		}
		file_ext = EBS_FILE_EXT;
		break;

	case IBS_TRIGGER:
		if (!isCpuIbsOk ())
		{
			QMessageBox::critical (this, "CodeAnalyst Error",
				"Your processor does not supported IBS profile.");
			return;
		}

		if (!isDriverIbsOk ())
		{
			QMessageBox::critical (this, "CodeAnalyst Error",
				"OProfile driver does not supported IBS profile.");
			return;
		}

		if (m_opIf.checkIbsSupportInDaemon() < 0 )
		{
			QMessageBox::critical (this, "CodeAnalyst Error",
				QString("Current OProfile daemon does not supported IBS profile\n")
				+ "or incompatible version of IBS support with the GUI.");
			return;
		}

		m_pSession = static_cast<SESSION_OPTIONS *>(new IBS_OPTIONS ());
		RETURN_IF_NULL (m_pSession, this);
		file_ext = EBS_FILE_EXT;
		break;

	default:
	case NO_TRIGGER:
		QMessageBox::information (this, "Error",
			"No sampling mechanism is available.\n"
			"The run will abort.");
		return;
	}

	// at here, profileType is always a valid type
	QString sessionName = m_pATuneFile->getNextSessionName (profileType);
	m_pSession->SessionFile = sessionName + file_ext;

	m_profiles.getProfileConfig (profile, m_pSession);

	// For MUX EBP
	if ((profileType == EVENT_TRIGGER)
	&& (static_cast <EBP_OPTIONS *>(m_pSession)->countEventGroups > 1 )){
		// check if driver is ok
		if (!m_opIf.checkMuxSupportInDriver() 
		||   m_opIf.getNumEventCounters() == MAX_EVENTNUM) {
			QMessageBox::warning(this, "CodeAnalyst Warning",QString("Warning:\n\n") +
			+ "Current CodeAnalyst Kernel Module does not support event multiplexing.\n"
			+ "CodeAnalyst will start profile only with events in the first group.\n\n"
			+ "Please update the CodeAnalyst Kernel Module in order to profile more than "
			+ "one event group.");
			static_cast <EBP_OPTIONS *>(m_pSession)->countEventGroups = 1; 
		}

		// check if daemon is ok
		if (!m_opIf.checkMuxSupportInDaemon()) {
			QMessageBox::warning(this, "CodeAnalyst Warning",QString("Warning:\n\n") +
			+ "Current OProfile daemon does not support event multiplexing.\n"
			+ "CodeAnalyst will start profile only with events in the first group.\n\n"
			+ "Please use CodeAnalyst provided OProfile in order to profile more than "
			+ "one event group.");
			static_cast <EBP_OPTIONS *>(m_pSession)->countEventGroups = 1; 
		}
	}       

	//Get the run control options
	m_pATuneFile->getRunOptions( m_pSession );
	//need this here, otherwise getRunOptions overwrites it
	m_pSession->sessionName = sessionName;

	setGUIButtonsStart();
	if( 0 != launchProgressManager())
	{
		// Clean up
		onSamplingStop();
	}
} // ApplicationWindow::onSamplingStart ()


void ApplicationWindow::setGUIButtonsStart()
{
	// Disable/Enable tool and menu items
	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_START], FALSE);
	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_PAUSE], TRUE);
	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_STOP], TRUE);
	m_pProfileBox->setEnabled (false);

	m_pToolsMenu->setItemEnabled (m_ToolsMenuId[TM_SETTINGS], FALSE);
	m_ToolsToolId[TM_SETTINGS]->setEnabled( FALSE );

	m_SamplingToolId[SM_START]->setEnabled (FALSE);
	m_SamplingToolId[SM_PAUSE]->setEnabled (TRUE);
	m_SamplingToolId[SM_STOP]->setEnabled (TRUE);

	// We do not allow import while profiling
	m_pFileMenu->setItemEnabled (m_ImportDataId, false);
}


int ApplicationWindow::launchProgressManager()
{
	// Sanity check	
	if( m_pSession->duration < 0 
	&& !m_pSession->stopOnAppExit 
	&&  m_pSession->startDelay < 0 ) {
		// This seems like user didn't setup the session properly,
		// force it to use default value
		m_pSession->duration = 20;
		m_pSession->startDelay = 0;
	}

	if(m_pSession->duration <= 0 && !m_pSession->stopOnAppExit) {
		QMessageBox::critical(this,"CodeAnalyst Error",
			QString("Invalid profiling duration (")
			+ QString::number(m_pSession->duration)
			+").\n"
			+"Please specify profiling duration greater than 0,\n"
			+"or specify \"Stop data collection when the app exits\"\n"	
			);	
		return -1;
	}

	if(m_pSession->startDelay < 0 ) {
		QMessageBox::critical(this,"CodeAnalyst Error",
			QString("Invalid start delay (")
			+ QString::number(m_pSession->startDelay)
			+").\n"
			+"Please specify start delay >= 0,\n"
			);	
		return -1;
	}

	// Spawn a thread to manage all the progress of the sampling session
	setSamplingState (SS_DELAY);

	m_pProgress = new ProgressMgr (m_pSession->startDelay,
		m_pSession->duration, this, m_pSession->launch,
		m_pSession->workDir, m_pSession->stopOnAppExit,
		m_pSession->termAppOnDur);
	if(!m_pProgress)
	{
		return -1;
	}

	if( m_pProgress->launch (m_pSession->affinity) != 0)
	{
		return -1;
	}

	QObject::connect(&(m_pProgress->m_launched), SIGNAL(processExited()),
		this, SLOT(targetExit()) );
	return 0;
}


///////////////////////////////////////////////////////////////////////////////
// PauseSampling()
// ---------------
//
void ApplicationWindow::onSamplingPause ()
{
	if (SS_PAUSED == m_SamplingState)
	{
		setSamplingState (SS_RESUMED);
	}
	else if (SS_IDLE == m_SamplingState)
	{
		setSamplingState (SS_IDLE);
	}
	else
	{
		setSamplingState (SS_PAUSED);
	}
}


//////////////////////////////////////////////////////////////////////////////
// StopSampling()
// --------------
//
void ApplicationWindow::onSamplingStop ()
{
	m_pSamplingMenu->setItemEnabled(m_SamplingMenuId[SM_STOP], FALSE);
	m_SamplingToolId[SM_STOP]->setEnabled(FALSE);
	m_pSamplingMenu->setItemEnabled(m_SamplingMenuId[SM_PAUSE], FALSE);
	m_SamplingToolId[SM_PAUSE]->setEnabled(FALSE);

	setSamplingState (SS_FINISHED);
}


void ApplicationWindow::onSessionDblClicked (TRIGGER trigger,
				 QString SessionName)
{
	SESSION_OPTIONS opts;

	if (!m_pATuneFile->getSessionCfg (trigger, SessionName, &opts))
		return;

	//session file does not include path
	QString  caption = SessionName + " - " + opts.SessionFile;

	QString sessionPath = m_pATuneFile->getDir()
		+ opts.SessionFile + DIR_FILE_EXT + "/" + opts.SessionFile;

	//If the profile session is currently shown
	QWidget *pWindow = findWindow( caption );
	if( pWindow != NULL )
	{
		//make it active
		pWindow->setFocus();
		return;
	}

	if ((NULL != ws->activeWindow()) && (ws->windowList().count() > 0))
	{
		ws->activeWindow()->showNormal();
	}

	qApp->setOverrideCursor (Qt::BusyCursor);

	QWidget *pNewWindow;
	CSessionNavigator *pSessNav = new CSessionNavigator( sessionPath, ws,
		sessionPath, WDestructiveClose );
	RETURN_IF_NULL (pSessNav, this);
	
	pNewWindow = pSessNav;

	QObject::connect( pSessNav,
		SIGNAL( DataViewChanged( QString, QListView *) ),
		SLOT( onDataViewChanged( QString, QListView *) ) );

	QObject::connect (this, SIGNAL (shownChanged (QString)), pSessNav, 
		SLOT(onGlobalShownChanged (QString)));

	QObject::connect (this, SIGNAL (densityVisibilityChanged ()), pSessNav, 
		SLOT (onDensityVisibilty ()));

	pSessNav->setCaption( caption );

	//this will be empty if timer, so the norms won't need to be calculated
	QString eventSessionName;
	if (EVENT_TRIGGER == trigger
	||  IBS_TRIGGER == trigger)
		eventSessionName = SessionName;

	if( !pSessNav->display (SessionName, m_pATuneFile, eventSessionName) )
	{
		QMessageBox::critical (this, "CodeAnalyst Error", QString("ERROR: Session : \"") + SessionName
					+ "\" not found.");
		qApp->restoreOverrideCursor();
		delete pSessNav;
		return;
	}

	setDefaultSize(pNewWindow);
	pNewWindow->showMaximized();

	qApp->restoreOverrideCursor ();

}//ApplicationWindow::onSessionDblClicked


// This function deletes the session from the project file, deletes
//  the module information file, and removes the window if it's showing...

void ApplicationWindow::onSessionDeletedOnly (TRIGGER trigger, QString SessionName)
{
	//Check for open window of session
	QString  caption ;
	QString fileName;
	QWidget *p_window;

	SESSION_OPTIONS opts;
	if (!m_pATuneFile->getSessionCfg (trigger, SessionName, &opts))
		return;
	fileName = opts.SessionFile;

	caption = SessionName + " - " + fileName;
	p_window = findWindow( caption );

	//If the session window is open
	if( NULL != p_window)
	{
		//close and delete the widget
		p_window->close(TRUE);
	}

	m_pATuneFile->deleteSession (trigger, SessionName );
} //ApplicationWindow::onSessionDeletedOnly

void ApplicationWindow::onSessionDeleted (TRIGGER trigger, QString SessionName)
{
	onSessionDeletedOnly(trigger,SessionName);	
	initProjectWindow();
}												  // ApplicationWindow::OnSessionDeleted


////////////////////////////////////////////////////////////////////////////
// onToolsOptions()
// ----------------
// The user has selected Tools | Options from the menu
// Return the Dialog code i.e. rejected() or accepted()
//
/*int ApplicationWindow::onToolsProjectOptions ()
{
GENERAL_OPTIONS gen;
OptionsDlg *pDlg = new OptionsDlg (FALSE, m_pATuneFile, this,
"ProjectOptions", TRUE,
m_bFirstDialog);

if (NULL != pDlg) {
m_bFirstDialog = FALSE;
pDlg->initialize ();
pDlg->exec ();

if (SS_IDLE == m_SamplingState)
setSamplingState (SS_IDLE);

m_pATuneFile->getGeneralOptions (&gen);
return pDlg->result ();
}
else {
QMessageBox::critical (this, "Error", DLG_ALLOC_ERROR_MSG);
return QDialog::Rejected;
}

}*/

///////////////////////////////////////////////////////////////////////////
// onToolsATuneOptions()
// ---------------------
//
int ApplicationWindow::onToolsATuneOptions ()
{
	int i_ret = QDialog::Rejected;
	CATuneOptionsDlg *pDlg = new CATuneOptionsDlg (this,
		"CodeAnalyst Options", TRUE);

	if (NULL != pDlg)
	{
		pDlg->exec ();
		i_ret = pDlg->result ();
		delete pDlg;
	}
	else
	{
		QMessageBox::critical (this, "ATuneOptions Error", DLG_ALLOC_ERROR_MSG);
	}

	return i_ret;
}


/////////////////////////////////////////////////////////////////////////////
// onFileSave()
// ------------
// User has clicked File | Save
//
void ApplicationWindow::onFileSave ()
{

	if (m_pATuneFile != NULL)
	{
		m_pATuneFile->saveAtuneFile ();
		statusBar()->message("Saved",3000);
	}
}

void ApplicationWindow::onFileClose (bool *isOk)
{
	if( sampling() )
	{
		QMessageBox::warning (this, "Hold on...", 
			"Please wait for the profiling session to end, or end it manually." );
		if(isOk)
			*isOk = false;
		return ;
	}

	closeAllWindows ();

	// Clear the Project Window
	m_pProjWnd->deleteAllItems ();
	m_pProjWnd->setCaption ("No Project Opened");
	setCaption (DEFAULT_CAPTION);

	// Disable sampling menu and Tools | Project Options
	m_pProfileBox->setEnabled (FALSE);
	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_START], FALSE);
	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_PAUSE], FALSE);
	m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_STOP], FALSE);

	m_pToolsMenu->setItemEnabled (m_ToolsMenuId[TM_SETTINGS], FALSE);
	m_pToolsMenu->setItemEnabled (m_ToolsMenuId[TM_DIFFANALYST], FALSE);
	m_pFileMenu->setItemEnabled (m_ExportDataId, false);
	m_pFileMenu->setItemEnabled (m_ImportDataId, false);
	m_pFileMenu->setItemEnabled (m_SaveId, false);
	m_pFileMenu->setItemEnabled (m_CloseId, false);
	m_SaveButt->setEnabled (FALSE);

	m_ToolsToolId[TM_SETTINGS]->setEnabled( FALSE );
	m_ToolsToolId[TM_DIFFANALYST]->setEnabled( FALSE );
	m_SamplingToolId[SM_START]->setEnabled (FALSE);
	m_SamplingToolId[SM_PAUSE]->setEnabled (FALSE);
	m_SamplingToolId[SM_STOP]->setEnabled (FALSE);

	if (m_pATuneFile)
		delete m_pATuneFile;
	m_pATuneFile = NULL;
	if(isOk)
		*isOk = true;
	return ;
}


// Shows version information for CodeAnalyst and any other sos
//
void ApplicationWindow::onHelpAbout ()
{
	AboutDlg *p_dlg = new AboutDlg (this, "About AMD CodeAnalyst", TRUE);

	if (NULL != p_dlg)
	{
		p_dlg->show ();
	} else
		QMessageBox::critical (this, "Error", DLG_ALLOC_ERROR_MSG);
}


void ApplicationWindow::onHelpContents ()
{
	QString invoke = LAUNCH_BROWSER + CA_DATA_DIR "/doc/cadoc/whnjs.htm";

	qApp->setOverrideCursor (QCursor (Qt::WaitCursor));
	system (invoke.data());
	qApp->restoreOverrideCursor ();
}


void ApplicationWindow::onHelpSysInfo ()
{
	SysInfoDlg *dlg = new SysInfoDlg (this, "System Information", TRUE);

	if (NULL != dlg)
	{
		dlg->initialize ();
		dlg->show ();
	} else
		QMessageBox::critical (this, "Error", DLG_ALLOC_ERROR_MSG);
}


//////////////////////////////////////////////////////////////////////////
// findWindow()
// -------------
//
QWidget *ApplicationWindow::findWindow (QString caption)
{
	QWidgetList windows = ws->windowList ();

	for (unsigned int i = 0; i < windows.count (); ++i)
	{
		QWidget *pWindow = windows.at (i);

		if (pWindow->caption ().compare (caption) == 0)
			return pWindow;
	}

	return NULL;
}


void ApplicationWindow::windowsMenuAboutToShow ()
{
	m_pWindowsMenu->clear ();

	int cascadeId =
		m_pWindowsMenu->insertItem ("&Cascade", this, SLOT (onCascade ()));
	int tileId = m_pWindowsMenu->insertItem ("&Tile", ws, SLOT (tile ()));

	int close_all_id = m_pWindowsMenu->insertItem ("Close Al&l", this,
		SLOT (closeAllWindows ()));

	m_toggleDensityId = m_pWindowsMenu->insertItem ("Show &Density charts",
		this, SLOT (onToggleChartDensity()));
	if (ws->windowList ().isEmpty ())
	{
		m_pWindowsMenu->setItemEnabled (cascadeId, FALSE);
		m_pWindowsMenu->setItemEnabled (tileId, FALSE);
		m_pWindowsMenu->setItemEnabled (close_all_id, FALSE);
	}

	CATuneOptions ao;
	DWORD density = SHOW_CHART_DENSITY;
	ao.getShowChartDensity (&density);

	if (SHOW_CHART_DENSITY == density)
		m_pWindowsMenu->setItemChecked (m_toggleDensityId, true);

	m_pWindowsMenu->insertSeparator ();

	QWidgetList windows = ws->windowList ();

	for (int i = 0; i < int (windows.count ()); ++i)
	{
		int id = m_pWindowsMenu->insertItem (windows.at (i)->caption (),
			this,
			SLOT (windowsMenuActivated (int)));

		m_pWindowsMenu->setItemParameter (id, i);
		m_pWindowsMenu->setItemChecked (id,
			ws->activeWindow () == windows.at (i));
	}
}


void ApplicationWindow::windowsMenuActivated (int id)
{
	QWidget *w = ws->windowList ().at (id);
	if (w)
	{
		w->show ();
		w->setFocus ();

	}
}

void ApplicationWindow::print_op_events()
{
	CpuEvent event;
	EBP_OPTIONS *pEventSession = static_cast <EBP_OPTIONS *> (m_pSession);

	for (int j=0; j<pEventSession->countEventGroups; j++)
	{
		for (int i=0; i<pEventSession->pEvents[j].numberOfValidEvents; i++)
		{
			CpuEvent event;
			EventGroup * pEventGroup = &pEventSession->pEvents[j];

			m_eventFile.findEventByValue(op_ebs_properties[i].eventmask, event);
			m_eventFile.findEventByValue(pEventGroup->eventSelect[i], event);
		}
	}
}

void ApplicationWindow:: enumerateAllOpEvents ( vector<string>& events, 
					   EventMaskEncodeMap & events_map)
{
	EBP_OPTIONS *pEventSession = static_cast <EBP_OPTIONS *> (m_pSession);
	EventMaskEncodeMap::iterator ev_it, ev_end;

	for (int j=0; j<pEventSession->countEventGroups; j++)
	{
		for (int i=0; i < pEventSession->pEvents[j].numberOfValidEvents; i++)
		{
			CpuEvent event;
			EventGroup * pEventGroup = &pEventSession->pEvents[j];

			m_eventFile.findEventByValue(pEventGroup->eventSelect[i], event);

			/* 
			* NOTE: Suravee	
			* Starting from CA 2.7, we are keeping track of the 
			* count and unitmask for each event
			*/
			QString op_name = QString("event:") + event.op_name;
			QString op_count = QString("count:") + 
						QString::number(pEventGroup->eventCount[i]);

			QString op_unitmask = QString("unit-mask:") + 
					QString::number(pEventGroup->eventUnitMask[i]);

			// This string will be passed to Oprofile	
			QString tmp = op_name + " " + op_count + " " + op_unitmask;
			string  key = string(tmp.data());
			// Check if this key is already exist.	
			ev_it = events_map.find(key);
			if(ev_it == events_map.end())
			{
				events.push_back(key);
				EventEncodeType eet;
				eet.sortedIndex = 0; 
				eet.weight = 1;
				eet.eventCount = pEventGroup->eventCount[i]; 
				eet.eventMask = EncodeEventMask(pEventGroup->eventSelect[i],
					pEventGroup->eventUnitMask[i]);
				events_map.insert(EventMaskEncodeMap::value_type(key, eet));

			} else {
				ev_it->second.weight++;
			}
		}
	}

	// change the index order of events;
	ev_it = events_map.begin();
	ev_end = events_map.end();
	for(int i = 0; ev_it != ev_end; ev_it++, i++) {
		ev_it->second.sortedIndex = i;
	}
}

void ApplicationWindow:: enumerateAllIbsEvents ( vector<string>& events, 
						EventMaskEncodeMap & events_map)
{
	/* 
	* We want IBS feth event when fetch is enabled,
	* IBS op event when op is enabled. 
	*/
	EventList evList;
	m_eventFile.findIbsEvent(evList);

	EventList::iterator ev_it = evList.begin();	
	EventList::iterator ev_end = evList.end();	

	for(;ev_it != ev_end; ev_it++) {
		IBS_OPTIONS *pIbsSession = static_cast <IBS_OPTIONS *> (m_pSession);

		bool bypass = true;
		QString op_name;
		if (pIbsSession->fetchSample && 
					(*ev_it).op_name.contains("IBS_FETCH_")) {
			bypass = false;
			QString t = QString::number(pIbsSession->fetchInterval, 10);
			op_name = QString("event:") + (*ev_it).op_name + 
				" count:" + t + " unit-mask:0";
		}

		if (pIbsSession->opSample && (*ev_it).op_name.contains("IBS_OP_")) {
			bypass = false;
			QString t = QString::number(pIbsSession->opInterval, 10);
			
			if (m_opIf.checkDispatchOpInDaemon() > 0) {
				op_name = QString("event:") + (*ev_it).op_name + 
					" count:" + t + " unit-mask:" 
					+ ((pIbsSession->opCycleCount)? "0" : "1") ;
			} else {
				op_name = QString("event:") + (*ev_it).op_name + 
					" count:" + t + " unit-mask:0";
			}
		}

		if (bypass)
			continue;

		// This string will be passed to Oprofile	
		string  key = string(op_name.data());
		// Check if this key is already exist.	
		if(events_map.find(key) == events_map.end()) {
			EventEncodeType eet;
			eet.sortedIndex = 0; 
			eet.weight = 1;

			if (pIbsSession->fetchSample 
			&&  (*ev_it).op_name.contains("IBS_FETCH_")) 
			{
				if (m_opIf.checkIbsSupportInDaemon() >= 2) {
					// If specify list and not found
					if (pIbsSession->fetchList.size() != 0
					&&  pIbsSession->fetchList.find((*ev_it).value) == pIbsSession->fetchList.end())
						continue;
				}

				eet.eventCount = pIbsSession->fetchInterval; 
				eet.eventMask = (*ev_it).value;
				events_map.insert( EventMaskEncodeMap::value_type(key, eet));
				events.push_back(key);
				continue;
			}

			if (pIbsSession->opSample 
			&& (*ev_it).op_name.contains("IBS_OP_")) 
			{
				if (m_opIf.checkIbsSupportInDaemon() >= 2) {
					// If specify list and not found
					if (pIbsSession->opList.size() != 0
					&&  pIbsSession->opList.find((*ev_it).value) == pIbsSession->opList.end())
						continue;						
				}

				eet.eventCount = pIbsSession->opInterval; 
				eet.eventMask = EncodeEventMask((*ev_it).value,
						((pIbsSession->opCycleCount)? 0 : 1));
				events_map.insert( EventMaskEncodeMap::value_type(key, eet));
				events.push_back(key);
				continue;
			}
		} // if
	} // for loop
	
	// change the index order of events;
	EventMaskEncodeMap::iterator em_it, em_end;
	em_it = events_map.begin();
	em_end = events_map.end();
	for(int i = 0; em_it != em_end; em_it++, i++) {
		em_it->second.sortedIndex = i;
	}
}



///////////////////////////////////////////////////////////////////////////////
// setSamplingState()
// ------------------
// Enables/Disables menu and tool bar items
//
bool ApplicationWindow::setSamplingState (SAMPLING_STATE State)
{
	vector < string > events_name;
	EventMaskEncodeMap events_map;
	QString ca_result_filename = "";
	QString ca_result_dirname = "";
	QCString fPath = "";
	QCString fpath = "";
	QString session = "";
	QCString f;
	int eventCount = 0;
	EventMaskMap::Iterator it;
	CATuneOptions ao;
	QString vmlinux_dir;
	QString tiFileName = "";
	QString vmlinuxName = "";
	QDir dir;

	switch (State)
	{
	case SS_RUNNING:
		{
			//Don't count time launching profiler towards sampling time.
			if(m_pProgress) m_pProgress->goToSleep();

			//Make sure events are taken care of, before trying a run
			qApp->processEvents();

			switch (m_pSession->Trigger)
			{
			case TIMER_TRIGGER:
				if (!runTbs ())
				{
					if(m_pProgress) m_pProgress->terminate ();
					return false;
				}
				break;

			case EVENT_TRIGGER:
			case EVENT_TRIGGER_PBM:
				if (!runEbs ())
				{
					if(m_pProgress) m_pProgress->terminate ();
					return false;
				}
				break;

			case IBS_TRIGGER:
				if( !runIbs () ) 
				{
					if(m_pProgress) m_pProgress->terminate ();
					return false;
				}
				break;

			default:
				QMessageBox::information (this, "Profile type error",
					"The application is in an unstable state, "
					"please restart CodeAnalyst");
				return false;
			}

			if(m_pProgress) 
			{
				m_pProgress->wakeUp();

				m_SamplingToolId[SM_PAUSE]->setEnabled (TRUE);
				m_pSamplingMenu->setItemEnabled(m_SamplingMenuId[SM_PAUSE], TRUE);

				m_pStatusMsg->setText ("Sampling Session Started.");

				m_SamplingState = SS_RUNNING;
			}else{
				// To handle the case when target application ended 
				// way too early then we expect.
				m_opIf.do_kill_daemon ();
				return false;
			}
			break;
		}// case SS_RUNNING:

	case SS_FINISHED:
		{
			QString caSession;

			/*
			* SS_FINISHED can be repeated set, ignore the repeated state
			* change if it has been set already.
			*/
			if (m_SamplingState == SS_FINISHED 
			||  m_SamplingState == SS_IDLE) 
			{
				goto SS_FINISHED_OUT;
			}

			setCursor(Qt::WaitCursor);
			m_pStatusMsg->setText ("Stopping daemon");
			m_opIf.do_kill_daemon ();
			unsetCursor();

			m_SamplingState = SS_IDLE;

			m_pStatusMsg->setText ("Processing sampled session");
			for( it = g_EventMaskMap.begin(); it != g_EventMaskMap.end(); ++it )
			{
				eventCount++;
			}

			// In the case of TBS, there is 1 event
			if (0 == eventCount)
				eventCount = 1;

			fpath = m_pATuneFile->getPath ();
			fpath.truncate (fpath.findRev (XPS_PATH_SLASH) + 1);

			if(m_pProgress)
			{
				m_pProgress->stopNow();
				if(m_pProgress->getExitStatus() != 0)
				{
					QString msg = QString(
						"WARNING: Target application exit with status ") + 
						QString::number(m_pProgress->getExitStatus()) + ".\n\n";
				/*	
					if(!m_pProgress->getStdout().isEmpty())
					{
						msg += "Stdout:\n" + m_pProgress->getStdout() + "\n";
					}
				*/	
					if(!m_pProgress->getStderr().isEmpty())
					{
						msg += "Stderr:\n" + m_pProgress->getStderr() + "\n";
					}

					msg += "Please check session setting and rerun the profile.\n";

					QMessageBox::warning(this, "CodeAnalyst Warning:", msg);
				}
				m_pProgress->terminate();
				DELETE_IF_NOT_NULL(m_pProgress);
				m_pProgress = NULL;
			}
			qApp->setOverrideCursor (QCursor (Qt::WaitCursor));
			qApp->restoreOverrideCursor ();

			// Make the new .tbp file
			f = m_pATuneFile->getDir ();

			/// get the name of the sampling file we ened to create
			ca_result_dirname = (QString) fpath + m_pSession->SessionFile;
			ca_result_dirname +=  DIR_FILE_EXT ;
			//////////////////////////////////////////
			dir.mkdir(ca_result_dirname );
			caSession = ca_result_dirname;
			ca_result_filename = ca_result_dirname + "/" + m_pSession->SessionFile;

			// Separate the type of samples
			switch (m_pSession->Trigger)
			{
			case TIMER_TRIGGER:
				{
					f.append(m_pSession->SessionFile);
					events_name.push_back(string("event:CPU_CLK_UNHALTED"));
			
					QString op_name = QString("event:CPU_CLK_UNHALTED");
					op_name += QString(" count:");
					op_name += QString::number(op_ebs_properties[0].count, 10);
					op_name += QString(" unit-mask:0");

					EventEncodeType eet;
					eet.sortedIndex = 0 ;
					eet.weight = 1;
					eet.eventCount = op_ebs_properties[0].count; 
					eet.eventMask = CPU_UNHALTED_CYCLES;
					events_map.insert(EventMaskEncodeMap::value_type(
								op_name.data(), eet));
				}
				break;
			case EVENT_TRIGGER:
			case EVENT_TRIGGER_PBM:
				f.append (m_pSession->SessionFile);
				enumerateAllOpEvents(events_name, events_map);
				break;
			case IBS_TRIGGER:
				f.append (m_pSession->SessionFile);
				enumerateAllIbsEvents(events_name, events_map);
				break;
			default:
				//should have already returned false...
				return false;
			}

			session = OP_SAMPLES_CURRENT_DIR;

			if (!createOprofileTranslateObjects()) {
				QMessageBox::information (this, "Translate Error",
					"failed to create translate object");
				return false;
			}

			/*****************************************************************
			* START: Translating Oprofile Samples Phase
			*/	
			m_translate->init(
				session,
				XpFileContains("/proc/cpuinfo", "processor"),
				caSession, events_map, m_ca_display);

			if (!m_translate)
			{
				QMessageBox::critical (this, "Error", DLG_ALLOC_ERROR_MSG);
				goto SS_FINISHED_OUT;
			}

			m_translate_progress_dlg = new TranslateDlg (this,
				"TranslateDlg", true);

			if (!m_translate_progress_dlg) {
				QMessageBox::critical (this, "Error", DLG_ALLOC_ERROR_MSG);
				delete m_translate;
				m_translate = NULL;
				goto SS_FINISHED_OUT;
			}

			m_translate_progress_dlg->setCaption ("Modules processed");
			connect (m_translate_progress_dlg, SIGNAL(dialogClosed()),
				this, SLOT (onModuleProcessingClosed()));

			m_ca_display->setDlgEvent (this);

			m_translate_progress_dlg->show();

			m_processFilter.clear();
			if(m_pATuneFile->isApplyProcessFilter())
			{
				m_processFilter = m_pATuneFile->getProcessFilter();
				
			}
			
			if (m_translate->translate_op_to_ca(m_this_cpuFamily,
								m_this_cpuModel,
								m_processFilter, 
								ca_result_filename))
			{
				// merge library = 1
				// exclude dependent = 0
				// Display the new session
				m_pSession->Cpus = XpGetNumCpus ();
				m_pATuneFile->addSession (m_pSession);
				m_pATuneFile->saveAtuneFile ();
				initProjectWindow ();

				// Display the system data
				onSessionDblClicked (m_pSession->Trigger, m_pSession->sessionName );
			}
			else
			{
				QMessageBox::information (this, "CodeAnalyst Warning",NO_SAMPLE_MSG);
				folderRemove(ca_result_dirname );
			}
			/*****************************************************************
			* END: Translating Oprofile Samples Phase
			*/	

SS_FINISHED_OUT:
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_START], FALSE);
			m_pSamplingMenu->setItemChecked (m_SamplingMenuId[SM_PAUSE], FALSE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_PAUSE], FALSE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_STOP], FALSE);

			m_SamplingToolId[SM_START]->setEnabled (FALSE);
			m_SamplingToolId[SM_PAUSE]->setOn (FALSE);
			m_SamplingToolId[SM_PAUSE]->setEnabled (FALSE);
			m_SamplingToolId[SM_STOP]->setEnabled (FALSE);

			// Now we allow seesion import.
			m_pFileMenu->setItemEnabled (m_ImportDataId, true);

			// Fall-through
		}// case SS_FINISHED:

	case SS_IDLE:
		{
			m_pProfileBox->setEnabled (true);
			m_ToolsToolId[TM_SETTINGS]->setEnabled( true );
			m_pToolsMenu->setItemEnabled (m_ToolsMenuId[TM_SETTINGS], true);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_START], TRUE);
			m_SamplingToolId[SM_START]->setEnabled (TRUE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_STOP], FALSE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_PAUSE], FALSE);
			m_SamplingToolId[SM_STOP]->setEnabled (FALSE);
			m_SamplingToolId[SM_PAUSE]->setEnabled (FALSE);
			m_pFileMenu->setItemEnabled (m_ImportDataId, true);

			// Reset progress
			m_pStatusProgress->reset ();

			m_pStatusMsg->setText ("Sampling Session Idle.");

			m_SamplingState = SS_IDLE;
			break;
		}// case SS_IDLE:

	case SS_PAUSED:
		{
			m_opIf.do_pause_daemon();

			//show controls...
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_STOP], TRUE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_PAUSE], TRUE);
			m_pSamplingMenu->setItemChecked (m_SamplingMenuId[SM_PAUSE], TRUE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_START], FALSE);
			m_SamplingToolId[SM_PAUSE]->setDown (TRUE);

			m_SamplingToolId[SM_STOP]->setEnabled (TRUE);
			m_SamplingToolId[SM_PAUSE]->setEnabled (TRUE);
			m_SamplingToolId[SM_PAUSE]->setDown (TRUE);
			m_SamplingToolId[SM_START]->setEnabled (FALSE);

			m_pStatusMsg->setText ("Sampling Session Paused...  Waiting for resume...");

			m_SamplingState = SS_PAUSED;
			break;
		}// case SS_PAUSED:

	case SS_RESUMED:
		{
			m_opIf.do_resume_daemon();	
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_STOP], TRUE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_PAUSE], TRUE);
			m_pSamplingMenu->setItemChecked (m_SamplingMenuId[SM_PAUSE], FALSE);
			m_pSamplingMenu->setItemEnabled (m_SamplingMenuId[SM_START], FALSE);
			m_SamplingToolId[SM_STOP]->setEnabled (TRUE);
			m_SamplingToolId[SM_PAUSE]->setEnabled (TRUE);
			m_SamplingToolId[SM_PAUSE]->setOn (FALSE);
			m_SamplingToolId[SM_PAUSE]->setDown (FALSE);
			m_SamplingToolId[SM_START]->setEnabled (FALSE);

			if(m_pProgress) m_pProgress->wakeUp();

			m_pStatusMsg->setText ("Sampling Session Resumed.");

			m_SamplingState = SS_RUNNING;
			break;
		}// case SS_RESUMED:

	case SS_DELAY:
		m_SamplingToolId[SM_PAUSE]->setEnabled (FALSE);
		m_pSamplingMenu->setItemEnabled(m_SamplingMenuId[SM_PAUSE], FALSE);

		m_pStatusMsg->setText (QString("Delay for ") 
			+ QString::number((unsigned int)m_pSession->startDelay)
			+" seconds before sampling started.");
		m_SamplingState = SS_DELAY;
		break;

	default:
		//for now, ignore other states
		break;
	}// switch (State)

	m_prevState = State;

	return true;
}


////////////////////////////////////////////////////////////////////////
// closeAllWindows()
// -----------------
// Closes all open windows in the workspace
//
void ApplicationWindow::closeAllWindows ()
{
	// Close all open windows
	QWidgetList windows = ws->windowList ();

	for (int i = 0; i < int (windows.count ()); ++i)
	{
		windows.at (i)->close ();
	}
}


///////////////////////////////////////////////////////////////////
// resizeAllWindows()
// ------------------
// Resizes all windows in the workspace to a predefined size.
// This is used when the cascade option has been selected.
//
void ApplicationWindow::resizeAllWindows (QSize NewSize)
{
	int numWnd = ws->windowList ().count ();

	for (int i = 0; i < numWnd; ++i)
	{
		QWidget *pWnd = ws->windowList ().at (i);

		pWnd->resize (NewSize);
	}
}


///////////////////////////////////////////////////////////////////////
// LoadFile()
// ----------
// Initializes the ATuneFile class and reads some of the information
// Initializes the Project Window with the names of the sampling sessions
//
void ApplicationWindow::LoadFile (QString FileName)
{
	bool isOk;
	onFileClose (&isOk);

	if(!isOk)
		return;	

	if (NULL != m_pATuneFile)
	{
		delete m_pATuneFile;
		m_pATuneFile = NULL;
	}

	m_pATuneFile = new ATuneFile (FileName, FALSE);

	if (NULL == m_pATuneFile)
	{
		QMessageBox::information (this, "Error",
			"Could not allocate memory for the ATuneFile");
		return;
	}

	if (m_pATuneFile->openAtuneFile ())
	{
		// Initialize the Project Window
		initProjectWindow ();

		m_pATuneFile->getPath ();

		// Format the Path name for windows if necessary
		QString caption;
		caption.sprintf ("Advanced Micro Devices - CodeAnalyst [%s]",
			m_pATuneFile->getPath ().data ());

		setCaption (caption);

		closeAllWindows ();

		// Enable Tools | Project Options Menu
		m_pToolsMenu->setItemEnabled (m_ToolsMenuId[TM_SETTINGS], TRUE);
		m_ToolsToolId[TM_SETTINGS]->setEnabled( TRUE );
		m_pToolsMenu->setItemEnabled (m_ToolsMenuId[TM_DIFFANALYST], TRUE);
		m_ToolsToolId[TM_DIFFANALYST]->setEnabled( TRUE );

		// Enable Import
		m_pFileMenu->setItemEnabled (m_ImportDataId, true);

		// Enable Save
		m_pFileMenu->setItemEnabled (m_SaveId, true);
		m_SaveButt->setEnabled (true);

		// Enable Close
		m_pFileMenu->setItemEnabled (m_CloseId, true);

		setSamplingState (SS_IDLE);

		// Update MRU file
		updateMruFile (m_pATuneFile->getPath ());
	}
	else
	{
		// Failed to load the file.
		QString message;

		if (QFile::exists (FileName))
		{
			message.sprintf ("%s\nCannot open file.\nBad Path\n",
				FileName.data ());
		}
		else
		{
			message.sprintf ("%s\nFile does not exist", FileName.data ());
		}

		QMessageBox::information (this, "Error", message);
		removeMruFile (FileName);
	}

	//get the last profileType
	QString profile = m_pATuneFile->getLastProfile ();
	if (!profile.isEmpty ())
	{
		for (int i = m_pProfileBox->count()-1 ; i >= 0 ; i--)
		{
			if(m_pProfileBox->text(i) == profile)
			{
				m_pProfileBox->setCurrentItem (i);
			}
		}
	}
}


////////////////////////////////////////////////////////////////////////////////////////
// ApplicationWindow::removeMruFile()
// ------------------------------------
//
void ApplicationWindow::removeMruFile (QString filePath)
{
	MruList::Iterator it;
	QString newMruList = "";

	// Remove all item from the menu
	for (it = m_MruList.begin (); it != m_MruList.end (); ++it)
	{
		int temp = (*it).index;
		m_pFileMenu->removeItem (temp);
	}

	// Remove the entry with filePath
	it = m_MruList.begin ();
	for (; it != m_MruList.end (); ++it)
	{
		if ((*it).filePath == filePath)
		{
			m_MruList.remove (it);
			break;
		}
	}

	if (!m_MruList.isEmpty ())
	{
		for (it = m_MruList.begin (); it != m_MruList.end (); ++it)
		{
			int temp = m_pFileMenu->insertItem ((*it).filePath);
			(*it).index = temp;
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////
// OnWindowShown()
// ---------------
//
void ApplicationWindow::onWindowActivated (QWidget * pWnd)
{
	CSessionNavigator *p_mdi = (CSessionNavigator *) pWnd;
	if (NULL == pWnd)
		m_pFileMenu->setItemEnabled (m_ExportDataId, false);
	else
	{
		if (p_mdi->isVisible ())
		{
			if (NULL == p_mdi->getCurrentView())
				m_pFileMenu->setItemEnabled (m_ExportDataId, false);
			else
				p_mdi->onCurrentTabChanged (p_mdi->getCurrentView ());
		}

		QWidgetList windows = ws->windowList ();
	}
}


void ApplicationWindow::onSessionRenamed (TRIGGER trigger, QString oldName,
										  QString newName )
{
	SESSION_OPTIONS opts;
	QString  caption ;
	QWidget *p_window;

	QRegExp rex ("[/:\\\\]");
	if (rex.search (newName) != -1)
	{
		QMessageBox::critical (this, "CodeAnalyst Error",
			"The session name must not contain the characters ':', '\\', or '/'");
		return;
	}

	if( m_pATuneFile->IsSessionNameTaken (trigger, newName) != 0 )
	{
		QMessageBox::information( this, "CodeAnalyst Error",
			"That session name is already taken." );
		return;
	}

	QString newSessionDir = m_pATuneFile->getDir() + newName;
	switch (trigger)
	{
	default:								  //should never happen
	case TIMER_TRIGGER:
		newSessionDir += TBS_FILE_EXT;
		break;
	case IBS_TRIGGER:
	case EVENT_TRIGGER:
		newSessionDir += EBS_FILE_EXT;
		break;
	}
	newSessionDir += DIR_FILE_EXT;
	if (QFile::exists (newSessionDir))
	{
		if (QMessageBox::No == QMessageBox::question (this, "CodeAnalyst Error",
			"The session directory alread exists, do you want to overwrite it?",
			QMessageBox::Yes, QMessageBox::No))
		{
			return;
		}
	}

	if (!m_pATuneFile->getSessionCfg (trigger, oldName, &opts))
		return;

	m_pATuneFile->renameSession (trigger, oldName, newName);

	caption = oldName + " - " + QString (opts.SessionFile);
	p_window = findWindow( caption );

	//This gets the renamed data
	m_pATuneFile->getSessionCfg (trigger, newName, &opts);

	//If the session window is open
	if( NULL != p_window)
	{
		p_window->close ();
		onSessionDblClicked (trigger, newName );
	}

	m_pATuneFile->saveAtuneFile();

	initProjectWindow();
}  //ApplicationWindow::onSessionRenamed


void ApplicationWindow::onTbsSessionProperties (QString SessionName)
{
	TBP_OPTIONS opts;

	TimerCfgDlg * p_dlg;
	if( !m_pATuneFile->getSessionDetails (SessionName, &opts ) )
	{
		QMessageBox::information( this, "Error",
			"Session Name could not be propogated to the file." );
		return;
	}
	p_dlg = new TimerCfgDlg ( this);
	RETURN_IF_NULL (p_dlg, this);

	p_dlg->setProperties (&opts);
	p_dlg->exec();

	delete p_dlg;
}


void ApplicationWindow::onEbsSessionProperties (QString SessionName)
{
	EBP_OPTIONS opts;

	EventCfgDlg * p_dlg;
	if( !m_pATuneFile->getSessionDetails (SessionName, &opts ) )
	{
		QMessageBox::information( this, "Error",
			"Session Name could not be propogated to the file." );
		return;
	}

	p_dlg = new EventCfgDlg ( this);
	RETURN_IF_NULL (p_dlg, this);

	QString session_cpuinfo = m_pATuneFile->getDir() 
		+ opts.sessionName 
		+ ".ebp"
		+ DIR_FILE_EXT 
		+ "/cpuinfo";

	if(QFile::exists(session_cpuinfo))
		p_dlg->setProperties (&opts,session_cpuinfo.data());
	else
		p_dlg->setProperties (&opts,"/proc/cpuinfo");

	p_dlg->exec();

	delete p_dlg;
}

void ApplicationWindow::onIbsSessionProperties( QString sessionName )
{
	IBS_OPTIONS opts;

	IbsCfgDlg * p_dlg;
	if( !m_pATuneFile->getSessionDetails (sessionName, &opts ) )
	{
		QMessageBox::information( this, "Error", 
			"Session Name could not be propogated to the file." );
		return;
	}
	p_dlg = new IbsCfgDlg ( this);
	RETURN_IF_NULL (p_dlg, this);

	p_dlg->setProperties (&opts);
	p_dlg->exec();

	delete p_dlg;
}

void ApplicationWindow::OnCopySessionToCurrent (TRIGGER trigger,
												QString SessionName)
{
	switch (trigger)
	{
	case TIMER_TRIGGER:
		{
			TBP_OPTIONS opts;
			if( m_pATuneFile->getSessionDetails (SessionName, &opts ) )
				m_profiles.setProfileConfig (CURRENT_TIME, &opts);
			//update current
			m_pProfileBox->setCurrentText (CURRENT_TIME);
			m_pATuneFile->setLastProfile (CURRENT_TIME);
			break;
		}
	case EVENT_TRIGGER:
		{
			EBP_OPTIONS opts;
			if( m_pATuneFile->getSessionDetails (SessionName, &opts ) )
				m_profiles.setProfileConfig (CURRENT_EVENT, &opts);
			//update current
			m_pProfileBox->setCurrentText (CURRENT_EVENT);
			m_pATuneFile->setLastProfile (CURRENT_EVENT);
			break;
		}
	case IBS_TRIGGER:
		{
			IBS_OPTIONS opts;
			if( m_pATuneFile->getSessionDetails (SessionName, &opts ) )
				m_profiles.setProfileConfig (CURRENT_IBS, &opts);
			//update current
			m_pProfileBox->setCurrentText (CURRENT_IBS);
			m_pATuneFile->setLastProfile (CURRENT_IBS);
			break;
		}
	default:
		break;
	}
}//ApplicationWindow::OnCopySessionToCurrent


void ApplicationWindow::OnProfileChange (const QString &profile)
{
	m_pATuneFile->setLastProfile (profile);
}


//called when the menu item is selected
void ApplicationWindow::OnViewManage ()
{
	//create new ViewCfgDlg
	ViewCfgDlg *pViewCfgDlg = new ViewCfgDlg (this);
	QString lastView = "";
	RETURN_IF_NULL (pViewCfgDlg, this);

	pViewCfgDlg->setCaption("Global View Management");	
	pViewCfgDlg->setViews (QString::null,m_this_cpuFamily,m_this_cpuModel,true);
	pViewCfgDlg->setCurrentViewNamePtr(&lastView);

	if (QDialog::Accepted == pViewCfgDlg->exec())
	{
		emit shownChanged(lastView);
	}
}

void ApplicationWindow::OnOpenDiffAnalyst(DiffSessionInfo *sess)
{
	QStringList list;
	NewDiffSessionDlg *pDiff = new NewDiffSessionDlg(m_pATuneFile->getPath(), this);
	pDiff->init(sess);
	if (QDialog::Accepted == pDiff->exec())
	{
		list = pDiff->getDiffSession();
		QString invoke = QString(CA_BIN_DIR) + "/DiffAnalyst -0 \"" + list[0] +
					"\" -1 \"" + list[1] + "\" &";

		qApp->setOverrideCursor (QCursor (Qt::WaitCursor));
		system (invoke.data());
		qApp->restoreOverrideCursor ();
	}
}

void ApplicationWindow::onDataViewChanged (QString exportString,
										   QListView * pList)
{
	if (exportString.isEmpty ())
	{
		m_pFileMenu->setItemEnabled( m_ExportDataId, false );
	} else {
		m_pFileMenu->changeItem( m_ExportDataId, exportString);
		m_pFileMenu->setItemEnabled( m_ExportDataId, true );
		m_pExportList = pList;
	}
}


void ApplicationWindow::onExportData ()
{
	QString txt;
	QString file_path = QFileDialog::getSaveFileName (m_pATuneFile->getDir (),
		"Comma Seperated Value (*.csv)",
		this);
	QFile *file = NULL;
	QTextStream stream;

	if (file_path.isEmpty ())
		return;
	if (NULL == m_pExportList)
		return;

	if (".csv" != file_path.right (4))
		file_path.append (".csv");

	file = new QFile (file_path);

	if (NULL == file)
	{
		QMessageBox::critical (this, "Memory error", "Insufficient memory");
		return;
	}
	// open the file for writing
	if (!file->open (IO_ReadWrite | IO_Translate | IO_Truncate))
	{
		QMessageBox::critical (this, "Error",
			"The file could not be written to.");
		return;
	}
	stream.setDevice (file);

	// put the column headers
	for (int i = 0; i < m_pExportList->columns (); ++i)
		stream << m_pExportList->columnText (i) << ",";

	// forgot to put this extra carriage return
	// after the column headers.
	stream << endl;

	// enumerate the list view data
	QListViewItem *pItem = m_pExportList->firstChild ();
	while (NULL != pItem)
	{
		for (int i = 0; i < pItem->listView ()->columns (); ++i)
		{
			stream << "\"" << pItem->text (i) << "\",";
		}

		stream << endl;
		pItem = pItem->itemBelow ();
	}

	stream.unsetDevice ();
	file->close ();

	delete file;
}


////////////////////////////////////////////////////////////////////////////
// ApplicationWindow::updateMruItemIndex()
// -----------------------------------
//
void ApplicationWindow::updateMruItemIndex (QString file, int index)
{
	MruList::Iterator it;

	for (it = m_MruList.begin (); it != m_MruList.end (); ++it)
	{
		// Remove the item if it is already on the list
		if ((*it).filePath == file)
		{
			(*it).index = index;
			break;
		}
	}
}


void ApplicationWindow::insertMruItems ()
{
	QString mru_list = 0;
	CATuneOptions ao;
	MruEntry entry;
	MruList::Iterator it;

	if (!ao.getMruFileList (mru_list))
	{
		mru_list = EMPTY_MRU_LIST;
	}

	parseMruString (mru_list.data ());

	if (m_MruList.isEmpty () == FALSE)
		m_pFileMenu->insertSeparator ();

	for (it = m_MruList.begin (); it != m_MruList.end (); ++it)
	{
		(*it).index = m_pFileMenu->insertItem ((*it).filePath);
	}
}


/*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ApplicationWindows()::parseMruString()
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* Takes a string of the form %s;%s;%s;%s;%s; and extracts the strings
* from the expression
*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*/
void ApplicationWindow::parseMruString (const char *mrustring)
{
	int lastdelimiter = 0;
	int nextdelimiter = 0;
	QString line = mrustring;
	QString onefile;
	MruEntry entry;

	for (int i = 0; i < MAX_MRU_ITEMS; i++)
	{
		// first string
		nextdelimiter = line.find (";", lastdelimiter);
		entry.filePath = line.mid (lastdelimiter,
			nextdelimiter - lastdelimiter);
		if (entry.filePath != "E")
		{
			entry.index = 0;
			m_MruList.append (entry);
		}

		lastdelimiter = nextdelimiter + 1;
	}
}


void ApplicationWindow::onAboutToQuit ()
{
	closeAllWindows ();
	saveMRUToReg ();

	setSamplingState (SS_IDLE);

	if (NULL != m_pATuneFile)
		delete m_pATuneFile;

	if (NULL != ws)
		delete ws;

	if (NULL != fileTools)
		delete fileTools;

	if (NULL != pSamplingToolBar)
		delete pSamplingToolBar;

	if (NULL != m_pWindowsMenu)
		delete m_pWindowsMenu;

	if (NULL != m_pSamplingMenu)
		delete m_pSamplingMenu;

	if (NULL != m_pToolsMenu)
		delete m_pToolsMenu;

	if (NULL != m_pHelpMenu)
		delete m_pHelpMenu;

	if (m_pFileMenu)
		delete m_pFileMenu;

	QString dir_name;
	dir_name.sprintf("%s", OP_JIT_DIR);
	QDir op_dir = QDir (dir_name);
	ApplicationWindow::folderRemove(op_dir);

	// Commented out for CA2.4
	//    if (NULL != m_pDataProcessing)
	//        delete m_pDataProcessing;
}


void ApplicationWindow::saveMRUToReg ()
{
	int i = 0;
	MruList::Iterator it = m_MruList.begin ();
	CATuneOptions ao;
	QString newMruList;

	for (i = 0, it = m_MruList.begin ();
		it != m_MruList.end () && i < MAX_MRU_ITEMS; ++it)
	{
		newMruList.append ((*it).filePath + QString (";"));
		++i;
	}

	// pad with empty entries
	while (i < (MAX_MRU_ITEMS))
	{
		newMruList.append ("E;");
		++i;
	}

	ao.setMruFileList (newMruList);
}


/*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
* ApplicationWindow::updateMruFile()
* ----------------------------------
* Adds the new path to the most-recently-used file list and
* saves the list out to the registry
*
* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/
void ApplicationWindow::updateMruFile (QString NewFilePath)
{
	MruEntry entry;

	MruList::Iterator it = m_MruList.begin ();
	int i = 0;

	if (0 != m_MruList.count ())
	{
		// Remove all item from the menu
		for (; it != m_MruList.end (); ++it)
		{
			m_pFileMenu->removeItem ((*it).index);
		}
	}
	else
	{
		m_pFileMenu->insertSeparator ();
	}

	// Check if NewFilePath is already in the list
	MruList::Iterator itDel;
	if (TRUE == mruFileContains (NewFilePath))
	{
		itDel = m_MruList.begin ();
		for (; itDel != m_MruList.end (); ++itDel)
		{
			// Remove the item if it is already on the list
			if ((*itDel).filePath == NewFilePath)
			{
				m_MruList.remove (itDel);
				break;
			}
		}
	}
	else
	{
		if (MAX_MRU_ITEMS <= m_MruList.count ())
		{
			itDel = m_MruList.fromLast ();
			if (itDel != m_MruList.end ())
				m_MruList.remove (itDel);
		}

	}

	// Add the new entry to the beginning of the list
	entry.filePath = NewFilePath;
	entry.index = 0;
	m_MruList.prepend (entry);

	for (it = m_MruList.begin ();
		it != m_MruList.end () && i < MAX_MRU_ITEMS; ++it)
	{
		(*it).index = m_pFileMenu->insertItem ((*it).filePath);
	}
}


/////////////////////////////////////////////////////////////////////////
// ApplicationWindow::onMruItemSelected()
// --------------------------------------
//
void ApplicationWindow::onMruItemSelected (int id)
{
	MruList::Iterator it = m_MruList.begin ();
	int i = 0;

	for (; it != m_MruList.end () && i < MAX_MRU_ITEMS; ++it, i++)
	{
		if ((*it).index == id)
		{
			LoadFile ((*it).filePath);

			break;
		}
	}
}


////////////////////////////////////////////////////////////////////////////////////////
// ApplicationWindow::mruFileContains()
// ------------------------------------
//
bool ApplicationWindow::mruFileContains (QString FilePath)
{
	MruList::Iterator it = m_MruList.begin ();

	while (it != m_MruList.end ())
	{
		if ((*it).filePath == FilePath)
			return TRUE;

		++it;
	}

	return FALSE;
}


float ApplicationWindow::translateMSToTicks (float ms)
{
	return m_tick_per_sec / 1000 * ms;
}


bool ApplicationWindow::runTbs ()
{
	TBP_OPTIONS *pTbpSession = static_cast <TBP_OPTIONS *> (m_pSession);
	BOOL rt;
	CATuneOptions ao;
	unsigned int novmlinux = true;
	QString vmlinux_dir = "";
	unsigned int buffer_size    = OP_DEFAULT_BUFFER_SIZE;
	unsigned int watershed_size = OP_DEFAULT_WATERSHED_SIZE;
	unsigned int cpu_buf_size   = OP_DEFAULT_CPU_BUFFER_SIZE;
	float count = translateMSToTicks (pTbpSession->msInterval);
	
	m_activeCounterMask  = 1;

	clearOPEBSProperties ();

	// Event "unhalted cycles" is used to simulate TBP.
	op_ebs_properties[0].eventmask = CPU_UNHALTED_CYCLES;

	// Converting from Float to unsigned long long
	op_ebs_properties[0].count = count;
	op_ebs_properties[0].unitmask = 0;
	op_ebs_properties[0].os = 1;
	op_ebs_properties[0].usr = 1;
	op_ebs_properties[0].op_name = "CPU_CLK_UNHALTED";

	m_opIf.set_op_events (op_ebs_properties);

	ao.getNoVmlinux (&novmlinux);
	ao.getVmlinuxDir (vmlinux_dir);
	ao.getBufferSize (&buffer_size);
	ao.getWatershedSize (&watershed_size);
	ao.getCpuBufSize (&cpu_buf_size);

	rt = startDaemon(novmlinux, vmlinux_dir,
		buffer_size, watershed_size, 
		cpu_buf_size, OP_EBP);

	return rt;
}


BOOL ApplicationWindow::startDaemon(unsigned int novmlinux,
				QString vmlinux_dir, 
				unsigned int buffer_size,
				unsigned int watershed_size,
				unsigned int cpu_buf_size,
				profiling_types ca_active_profiling)
{
	BOOL rt = false;
	int oie_rt = OIE_OK;
	QString errMsg = "Oprofile daemon failed to start.\nReason:\n\n";
	EBP_OPTIONS *pEventSession = static_cast <EBP_OPTIONS *> (m_pSession);
	
	/* Check watershed size*/
	if(buffer_size <= watershed_size) {
		errMsg += QString("Invalid OProfile event buffer configuration.\n")
			+ "The watershed size should be less than event buffer size.\n";
		oie_rt = -OIE_ERR;
	}

	if (oie_rt == OIE_OK) {
		if(m_pSession->Trigger == EVENT_TRIGGER)
			oie_rt = m_opIf.do_start_daemon (novmlinux, 
							vmlinux_dir,
							buffer_size, 
							watershed_size, 
							cpu_buf_size, 
							ca_active_profiling, 
							pEventSession->msMpxInterval);
		else
			oie_rt = m_opIf.do_start_daemon (novmlinux, 
							vmlinux_dir,
							buffer_size, 
							watershed_size, 
							cpu_buf_size, 
							ca_active_profiling,
							0);
	}


	/* ERROR HANDLING */
	if (oie_rt != OIE_OK) {
		// At this point we have already launched progress manager,
		// and the application. If the error occur, stop them immediately.
		if(m_pProgress)
		{
			m_pProgress->stopNow();
			m_pProgress->terminate ();
		}

		switch(oie_rt)
		{
		case -OIE_DAEMON_RUNNING:
			errMsg += "Oprofile daemon is currently running.\n"
				  "Please make sure the profile is not running.\n\n"
				  "Do you want to kill it?";
			if(QMessageBox::Yes == QMessageBox::critical(this, "CodeAnalyst Error", 
				errMsg,
				QMessageBox::Yes,
				QMessageBox::No))
			{
				m_opIf.do_kill_daemon ();
			}
			break;
		case -OIE_LOCK_EXIST:
			errMsg += QString("Stale /var/lib/oprofile/lock exists ") +
				  "and no Oprofile daemon currently running.\n" +
				  "Do you want to remove the lock file?";
			if(QMessageBox::Yes == QMessageBox::critical(this, "CodeAnalyst Error", 
				errMsg,
				QMessageBox::Yes,
				QMessageBox::No))
			{
				m_opIf.remove_lock_file();
			}
			break;
		case -OIE_DAEMON_NOTSTART:
		case -OIE_DRV:
		case -OIE_ERR:
		default:
			errMsg += m_opIf.get_std_err();
			QMessageBox::critical(this, "CodeAnalyst Error", errMsg);
			break;	
		};
	}else{
		rt = true;
	}
	return rt;
}


bool ApplicationWindow::runEbs ()
{
	CATuneOptions ao;
	CpuEvent event;
	EBP_OPTIONS *pEventSession = static_cast <EBP_OPTIONS *> (m_pSession);
	bool rt = true;
	unsigned int novmlinux = true;
	QString vmlinux_dir = "";
	unsigned int buffer_size    = OP_DEFAULT_BUFFER_SIZE;
	unsigned int watershed_size = OP_DEFAULT_WATERSHED_SIZE;
	unsigned int cpu_buf_size   = OP_DEFAULT_CPU_BUFFER_SIZE;
	int groups, valid_events;

	m_activeCounterMask  = 0;
	clearOPEBSProperties();
	groups = static_cast <EBP_OPTIONS *>(m_pSession)->countEventGroups;
	if(groups > MAX_EVENTGROUP_MULTIPLEXING)
	{
		QMessageBox::critical(this,"CodeAnalyst Error",
			QString("ERROR:\n")
			+ "Invalid event configuration. CodeAnalyst only supports up to " 
			+ QString::number(MAX_EVENTGROUP_MULTIPLEXING) + "event groups.\n"
			+ "Please reconfigure.\n");	
		rt = false;	
	}

	if(((EBP_OPTIONS*)(m_pSession))->checkForDupEventWithDiffCount())
	{
		QMessageBox::critical(this,"CodeAnalyst Error",
			QString("ERROR:\n")
			+ "Invalid event configuration. CodeAnalyst do not allow "
			+ "same events with different counts.\n" 
			+ "Please reconfigure.\n");	
		rt = false;	
	}

	if(rt)
	{
		for (int j=0; j<groups; j++) {
			valid_events =  pEventSession->pEvents[j].numberOfValidEvents;
			for (int i=0; i<valid_events; i++) {
				EventGroup * pEventGroup = &pEventSession->pEvents[j];
				int pos = j * MAX_EVENTNUM + i;

				op_ebs_properties[pos].eventmask = pEventGroup->eventSelect[i];
				op_ebs_properties[pos].count = pEventGroup->eventCount[i];
				op_ebs_properties[pos].unitmask = pEventGroup->eventUnitMask[i];
				op_ebs_properties[pos].os = pEventGroup->eventOS[i];
				op_ebs_properties[pos].usr = pEventGroup->eventUser[i];
				m_eventFile.findEventByValue(op_ebs_properties[pos].eventmask, event);
				op_ebs_properties[pos].op_name = event.op_name;
			}
		}

		m_opIf.set_op_events (op_ebs_properties);

		ao.getNoVmlinux (&novmlinux);
		ao.getVmlinuxDir (vmlinux_dir);
		ao.getBufferSize (&buffer_size);
		ao.getWatershedSize (&watershed_size);
		ao.getCpuBufSize (&cpu_buf_size);

		rt = startDaemon(novmlinux, vmlinux_dir, 
			buffer_size, watershed_size, 
			cpu_buf_size, OP_EBP);
	}
	return rt;
} //ApplicationWindow::runEbs ()


bool ApplicationWindow::runIbs ()
{
	CATuneOptions ao;
	op_ibs_property ibs_prop;
	bool rt 			= true;
	unsigned int novmlinux 		= true;
	QString vmlinux_dir 		= "";
	bool dcMissInfoEnable 		= false ;
	unsigned int buffer_size 	= OP_DEFAULT_BUFFER_SIZE;
	unsigned int watershed_size     = OP_DEFAULT_WATERSHED_SIZE;
	unsigned int cpu_buf_size 	= OP_DEFAULT_CPU_BUFFER_SIZE;
	m_activeCounterMask  		= 0;		//???
	profiling_types op_active_profiling = OP_NONE;

	/* Getting the IBS session option */
	IBS_OPTIONS *pIbsSession 	= static_cast <IBS_OPTIONS *> (m_pSession);

	if (pIbsSession->fetchSample) 
	{
		ibs_prop.fetch_count = pIbsSession->fetchInterval;
		op_active_profiling = (profiling_types)(OP_IBS_FETCH | op_active_profiling); 
	}
	if(pIbsSession->opSample)
	{
		ibs_prop.op_count = pIbsSession->opInterval;
		op_active_profiling = (profiling_types)(OP_IBS_OP | op_active_profiling); 
		if(pIbsSession->opCycleCount)
			ibs_prop.dispatched_ops = 0; 
		else
			ibs_prop.dispatched_ops = 1;
	}

	if (m_opIf.checkIbsSupportInDaemon() >= 2) {
		// NOTE: If the fetchList or opList is empty while it is being enabled, 
		//       we would add all fetch/op events to the list. 
		//       This would help keep the old IBS DCCOnfig compatible.
		bool fixFetchList = false;
		bool fixOpList    = false;

		if (pIbsSession->fetchSample && pIbsSession->fetchList.size() == 0 )
			fixFetchList = true;	

		if (pIbsSession->opSample && pIbsSession->opList.size() == 0 )
			fixOpList = true;	

		if (fixFetchList || fixOpList) {
			EventList evList;
			m_eventFile.findIbsEvent(evList);

			EventList::iterator ev_it = evList.begin();	
			EventList::iterator ev_end = evList.end();	
			for(;ev_it != ev_end; ev_it++) {
				if (0xF000 <= (*ev_it).value && (*ev_it).value < 0xF100 && fixFetchList) {
					//fetch
					pIbsSession->fetchList.append((*ev_it).value);
				} else if (0xF100 <= (*ev_it).value && fixOpList) {
					//op
					pIbsSession->opList.append((*ev_it).value);
				}
			}
		} 

		// Convert list of IBS Fetch event number to op-name for oprofiled
		QValueList<unsigned int>::iterator it     = pIbsSession->fetchList.begin();
		QValueList<unsigned int>::iterator it_end = pIbsSession->fetchList.end();
		for (; it != it_end; it++)
		{
			CpuEvent event;
			m_eventFile.findEventByValue(*it, event);

			if (0xF000 <= *it && *it < 0xF100) {
				//fetch
				ibs_prop.fetch_strList.append(event.op_name);
			}
		}
		
		// Convert list of IBS Op event number to op-name for oprofiled
		it     = pIbsSession->opList.begin();
		it_end = pIbsSession->opList.end();
		for (; it != it_end; it++)
		{
			CpuEvent event;
			m_eventFile.findEventByValue(*it, event);

			if (0xF100 <= *it) {
				//op
				ibs_prop.op_strList.append(event.op_name);
			}
		}
	}

	ibs_prop.dcMissInfoEnabled = pIbsSession->dcMissInfoEnabled;

	/*******************************************************
	* Configure oprofile and start the daemon
	*/
	ao.getNoVmlinux (&novmlinux);
	ao.getVmlinuxDir (vmlinux_dir);
	ao.getBufferSize (&buffer_size);
	ao.getWatershedSize (&watershed_size);
	ao.getCpuBufSize (&cpu_buf_size);

	m_opIf.set_op_ibs(&ibs_prop);
	rt = startDaemon(novmlinux, vmlinux_dir, buffer_size, 
			watershed_size, cpu_buf_size, 
			op_active_profiling);

	return rt;
}



bool ApplicationWindow::containEbsEvent(OP_EVENT_PROPERTY_MAP &events_name_map)
{
	OP_EVENT_PROPERTY_MAP::iterator it;

	for( it  = events_name_map.begin(); it != events_name_map.end(); it++)
	{
		if(!it->second.op_name.contains("IBS"))
		{
			return true;	
		}
	}

	return false;	
}

bool ApplicationWindow::containIbsEvent(OP_EVENT_PROPERTY_MAP &events_name_map)
{
	OP_EVENT_PROPERTY_MAP::iterator it;

	for( it  = events_name_map.begin(); it != events_name_map.end(); it++)
	{
		if(it->second.op_name.contains("IBS"))
		{
			return true;	
		}
	}

	return false;	
}

QString ApplicationWindow::getNextImportSessionName(TRIGGER trigger)
{
	RUN_OPTIONS go;
	QString qs_return;	

	/******************************
	 * Handling session name
	 */
	m_pATuneFile->getRunOptions (&go);

	// Get the next session name
	QString oldSessionName = go.sessionName;
	go.sessionName += " import";
	m_pATuneFile->setRunOptions (go);

	qs_return = m_pATuneFile->getNextSessionName (trigger);

	//Reset the session name
	go.sessionName = oldSessionName;
	m_pATuneFile->setRunOptions (go);
	return qs_return;

}

bool ApplicationWindow::onImportXmlFile(ImportWizard* import_wizard)
{
	bool ret = false;
	unsigned int numCpu;
	unsigned long cpuModel;
	unsigned long cpuFamily;
	EventMaskEncodeMap events_map;
	OP_EVENT_PROPERTY_MAP ebp_events_name_map;
	OP_EVENT_PROPERTY_MAP ibs_events_name_map;


	// - Get the XML file from ImportWizard
	QString import_xml_path = import_wizard->get_xml_path();
	if(!QFile::exists(import_xml_path))
	{
		m_importErrorMsg = QString("XML File ") 
				+ import_xml_path
				+ " does not exist.";
		return false;
	}
	
	// - Get the cpuinfo file from ImportWizard
	QString import_xml_cpuinfo = import_wizard->get_xml_cpuinfo_path();

	// Get system info
	if(!QFile::exists(import_xml_cpuinfo))
	{
		m_importErrorMsg = QString("cpuinfo File ") 
				+ import_xml_cpuinfo 
				+ " does not exist.";
		return false;
	}
	
	numCpu    = XpFileContains(import_xml_cpuinfo, "processor");
	cpuFamily = getFamilyFromCpuInfo(import_xml_cpuinfo);
	cpuModel  = getModelFromCpuInfo(import_xml_cpuinfo);
	QString file = helpGetEventFile(cpuFamily, cpuModel);

	CEventsFile * eventFileForImport = new CEventsFile();
	if(!eventFileForImport)
	{
		m_importErrorMsg = QString("Error getting CA Eventfile.");
		return false;
	}

	eventFileForImport->open(file);

	// - Read XML file 
	opxmldata_handler op_hdr;

	// Opdata_handler initialization:
	op_hdr.init();

	// *****************************************************************
	// Initialize Progress Dialog
	m_translate_progress_dlg = new TranslateDlg (this, "TranslateDlg", true);
	if (NULL == m_translate_progress_dlg) {
		m_importErrorMsg = QString(DLG_ALLOC_ERROR_MSG);
		return ret;
	}

	m_translate_progress_dlg->setCaption ("Modules processed");

	connect (m_translate_progress_dlg, SIGNAL (dialogClosed()),
		this, SLOT (onModuleProcessingClosed()));


	m_ca_display = new ca_translate_gui_display();
	if(!m_ca_display)
	{
		m_importErrorMsg = QString("Error creating CA progress dialog.");
		return false;
	}

	m_ca_display->setDlgEvent (this);

	m_translate_progress_dlg->show();
	
	op_hdr.set_display(m_ca_display);

	// *****************************************************************
	// Read XML file
	if(!op_hdr.readXML(import_xml_path.ascii()))	
	{
		m_importErrorMsg = QString("Error reading XML file: ") 
						+ import_xml_path + "."; 
		goto closeReader;
	}

	// Handle EBP Events
	if(op_hdr.getEbsSamples(ebp_events_name_map) < 0)
	{
		m_importErrorMsg = QString("Error getting EBS sample."); 
		goto closeReader;
	}

	if(op_hdr.getIbsSamples(ibs_events_name_map) < 0)
	{
		m_importErrorMsg = QString("Error getting IBS sample."); 
		goto closeReader;
	}

	// Sanity check
	if( ebp_events_name_map.size() == 0
	&&  ibs_events_name_map.size() == 0)
	{
		m_importErrorMsg = 
			QString("The imported XML file contains insufficient information.\n") +
			"Please make sure to use option \"opcontrol --separate=lib,cpu\"\n" +
			"when running profile.\n";
		goto closeReader;
	}


	if(ebp_events_name_map.size() > 0
	&& ibs_events_name_map.size() > 0)
	{
		QMessageBox::warning( this, "CodeAnalyst Warning",
			"The imported XML file contains both PMC and IBS events.\n"
			"CodeAnalyst will separate them into PMC and IBS session.");
	}

	if( ebp_events_name_map.size() > 0 )
	{
		//***********************************************************
		// Create EBP Profile
		EBP_OPTIONS *pEbp = new EBP_OPTIONS ();
		RETURN_FALSE_IF_NULL (pEbp, this);
		
		//***********************************************************
		// Calculate the number of groups for PMC events 
		pEbp->countEventGroups = ebp_events_name_map.size() / MAX_EVENTNUM;
		ebp_events_name_map.size() % MAX_EVENTNUM ? pEbp->countEventGroups++ : 0;	
		pEbp->pEvents = new EventGroup[pEbp->countEventGroups];

		PopulateEbpSettingsForImportSession(pEbp, 
					ebp_events_name_map, 
					eventFileForImport, 
					events_map);

		//***********************************************************
		// Process XML file
		op_hdr.setEventEncodeMap(events_map);

		m_ca_display->enableOkButton(false);

		if(!op_hdr.processXMLData())
		{
			m_importErrorMsg = QString("Error processing EBP XML file: ") 
					+ import_xml_path + ".\n" 
					+ "Please make sure the XML file was created using:\n"
					+ "    opreport -Xwdl -o <output.file>";
			goto closeReader;
		}

		//***********************************************************
		// Create Directory inside the project directory
		pEbp->sessionName = getNextImportSessionName(EVENT_TRIGGER);
		pEbp->SessionFile = pEbp->sessionName + EBS_FILE_EXT;

		//  Create Directory inside the project directory
		QString dir_name;
		dir_name = QString(m_pATuneFile->getDir() 
				+ pEbp->SessionFile) + ".dir";

		QDir ebs_dir;
		ebs_dir.mkdir(dir_name);

		QString caSession;
		caSession = dir_name 
			+ QString("/") 
			+ QString(pEbp->SessionFile);
		
		//***********************************************************
		// Generate EBP file
		op_hdr.write_ebp_output(caSession.data(), numCpu,
					cpuFamily, cpuModel);
		
		// - Copy jo file over for java
		op_hdr.copyJoFile(caSession.section("/",0,-2));
	
		// - Adding EBP/IBS section to .caw file
		m_pATuneFile->addSession(pEbp);
		m_pATuneFile->saveAtuneFile ();
		initProjectWindow ();
		onSessionDblClicked (EVENT_TRIGGER, pEbp->sessionName);
	}

	events_map.clear();
	// Handle IBS Events
	if( ibs_events_name_map.size() > 0 )
	{
		//***********************************************************
		// Create IBS Profile
		IBS_OPTIONS *pIbs = new IBS_OPTIONS ();
		RETURN_FALSE_IF_NULL (pIbs, this);
		
		//***********************************************************
		PopulateIbsSettingsForImportSession(pIbs, 
					ibs_events_name_map, 
					eventFileForImport, 
					events_map,
					import_wizard);

		//***********************************************************
		// Process XML file
		op_hdr.setEventEncodeMap(events_map);

		m_ca_display->enableOkButton(false);

		if(!op_hdr.processXMLData())
		{
			m_importErrorMsg = QString("Error processing IBS XML file: ") 
						+ import_xml_path + "." ;
			goto closeReader;
		}

		//***********************************************************
		// Create Directory inside the project directory
		pIbs->sessionName = getNextImportSessionName(IBS_TRIGGER);
		pIbs->SessionFile = pIbs->sessionName + EBS_FILE_EXT;

		//  Create Directory inside the project directory
		QString dir_name;
		dir_name = QString(m_pATuneFile->getDir() 
				+ pIbs->SessionFile) + ".dir";

		QDir ebs_dir;
		ebs_dir.mkdir(dir_name);

		QString caSession;
		caSession = dir_name 
			+ QString("/") 
			+ QString(pIbs->SessionFile);
		
		//***********************************************************
		// Generate EBP file
		op_hdr.write_ebp_output(caSession.data(), numCpu,
					cpuFamily, cpuModel);
		
		// - Copy jo file over for java
		op_hdr.copyJoFile(caSession.section("/",0,-2));
	
		// - Adding EBP/IBS section to .caw file
		m_pATuneFile->addSession(pIbs);
		m_pATuneFile->saveAtuneFile ();
		initProjectWindow ();
		onSessionDblClicked (IBS_TRIGGER, pIbs->sessionName);
	}

	ret = true;
closeReader:

	m_ca_display->enableOkButton(true);

	// Closing
	eventFileForImport->close();
	if(eventFileForImport)
	{
		delete eventFileForImport;
		eventFileForImport = NULL;
	}

	return ret;
} //bool ApplicationWindow::onImportXmlFile(ImportWizard* import_wizard)

bool ApplicationWindow::onImportEbpFile(ImportWizard* import_wizard)
{
	QString caSession = "";
	IBS_OPTIONS *pIbs = NULL;
	EBP_OPTIONS *pEbs = NULL;
	TBP_OPTIONS *pTbs = NULL;
	QString dir_name = "";

	// - Get the Ebp file
	QString import_ebp_path = import_wizard->get_ebp_path();
	QString import_jit_dir_path = import_wizard->get_jit_dir_path();

	// - Read TBS file 
        TbsReader *ebp_reader = new TbsReader ();
        RETURN_FALSE_IF_NULL (ebp_reader, this);
        if(!ebp_reader->OpenTbsFile(import_ebp_path.data()))
	{
		m_importErrorMsg = QString("Error opening TBP/EBP file: ") 
				+ import_ebp_path + "." ;
		return FALSE;
	}
        
	// - Determine what type of EBP file (TBS, EBS or IBS)
	QValueList<unsigned long long> eventList = ebp_reader->getEventList();
	if(import_ebp_path.section('.',-1).lower() == QString("tbp")) 
	{
		pTbs = new TBP_OPTIONS ();
		RETURN_FALSE_IF_NULL (pTbs, this);

	}else if( !eventList.empty() && eventList[0] >= 0xF000 && eventList[0] <  0xFFFF ) {
		// Event range 0xF000 - 0xFFFF is reserved for IBS.
		pIbs = new IBS_OPTIONS ();
		RETURN_FALSE_IF_NULL (pIbs, this);

	} else{
		pEbs = new EBP_OPTIONS ();
		RETURN_FALSE_IF_NULL (pEbs, this);
		
		// Calculate the number of groups for PMC events 
		pEbs->countEventGroups = eventList.size() / MAX_EVENTNUM;
		eventList.size() % MAX_EVENTNUM ? pEbs->countEventGroups++ : 0;	
		pEbs->pEvents = new EventGroup[pEbs->countEventGroups];

		//***********************************************************
		// Populate the counter settings for the import session
		QValueList<unsigned long long>::iterator it = eventList.begin();
		QValueList<unsigned long long>::iterator it_end = eventList.end();
		for (int i = 0, group = 0, ebp_event_index = 0; it != it_end; it++, i++) 
		{
			/* Trying to Fill/Fake the EBP_Options */
			group = i / MAX_EVENTNUM;	
			ebp_event_index = i % MAX_EVENTNUM;
			pEbs->pEvents[group].eventSelect[ebp_event_index] = 0;
			pEbs->pEvents[group].eventUnitMask[ebp_event_index] = 0;
			pEbs->pEvents[group].eventOS[ebp_event_index] = 0;
			pEbs->pEvents[group].eventUser[ebp_event_index] = 0;
			pEbs->pEvents[group].eventCount[ebp_event_index] = 0;
			pEbs->pEvents[group].edgeDetect[ebp_event_index] = 0;
			pEbs->pEvents[group].numberOfValidEvents = ebp_event_index + 1;
		}
	}

	// ---------------------------------------------------------------
	// - Handle different type of EBP file
	if(pTbs != NULL)
	{
		// HANDLE TBP FILE
		pTbs->sessionName = getNextImportSessionName(TIMER_TRIGGER);
		pTbs->SessionFile = pTbs->sessionName + TBS_FILE_EXT;

		// - Create Directory inside the project directory
		dir_name = QString(m_pATuneFile->getDir() 
				+ pTbs->SessionFile) + ".dir";
		
		QDir tbs_dir;
		tbs_dir.mkdir(dir_name);

		caSession = dir_name 
			+ QString("/") 
			+ QString(pTbs->SessionFile);
 	
		// - Copy EBP/IBS file over.
		QString cmd;	
		cmd = "/bin/cp -f \"" + import_ebp_path + "\" \"" + caSession +"\"";  
		//fprintf(stderr,"DEBUG: Copy file : %s\n",cmd.data());
		if(system(cmd.data()) != 0)
		{
			m_importErrorMsg = QString("Error: Cannot copy ") 
					+ import_ebp_path
					+ " to "+caSession+".\n";
			return false;
		}

		// - Copy jit dir for java.
		if(!import_jit_dir_path.isEmpty())
		{
			cmd = "/bin/cp -rf \"" + import_jit_dir_path + "\" \"" + dir_name +"\"";  
			//fprintf(stderr,"DEBUG: Copy dir : %s\n",cmd.data());
			if(system(cmd.data()) != 0)
			{
				m_importErrorMsg = QString("Error: Cannot copy ") 
					+ import_jit_dir_path
					+ " to "+dir_name+".\n";
				return false;
			}
		}
		
		// - Adding EBP/IBS section to .caw file
		m_pATuneFile->addSession(pTbs);
		m_pATuneFile->saveAtuneFile ();
		initProjectWindow ();
		onSessionDblClicked (TIMER_TRIGGER, pTbs->sessionName);

	} else if(pIbs != NULL) {
		// HANDLE IBS EVENT FILE
		RETURN_FALSE_IF_NULL (pIbs, this);

		pIbs->sessionName = getNextImportSessionName(IBS_TRIGGER);
		pIbs->SessionFile = pIbs->sessionName + EBS_FILE_EXT;

		// - Create Directory inside the project directory
		dir_name = QString(m_pATuneFile->getDir() 
				+ pIbs->SessionFile) + ".dir";
		
		QDir ebs_dir;
		ebs_dir.mkdir(dir_name);

		caSession = dir_name 
			+ QString("/") 
			+ QString(pIbs->SessionFile);
 	
		// - Copy EBP/IBS file over.
		QString cmd;	
		cmd = "/bin/cp -f \"" + import_ebp_path + "\" \"" + caSession +"\"";  
		//fprintf(stderr,"DEBUG: Copy file : %s\n",cmd.data());
		if(system(cmd.data()) != 0)
		{
			m_importErrorMsg = QString("Error: Cannot copy ") 
					+ import_ebp_path
					+ " to "+caSession+".\n";
			return false;
		}

		// - Copy jit dir for java.
		if(!import_jit_dir_path.isEmpty())
		{
			cmd = "/bin/cp -rf \"" + import_jit_dir_path + "\" \"" + dir_name +"\"";  
			//fprintf(stderr,"DEBUG: Copy dir : %s\n",cmd.data());
			if(system(cmd.data()) != 0)
			{
				m_importErrorMsg = QString("Error: Cannot copy ") 
					+ import_jit_dir_path
					+ " to "+dir_name+".\n";
				return false;
			}
		}
		
		// - Adding EBP/IBS section to .caw file
		m_pATuneFile->addSession(pIbs);
		m_pATuneFile->saveAtuneFile ();
		initProjectWindow ();
		onSessionDblClicked (IBS_TRIGGER, pIbs->sessionName);
	
	}else if(pEbs != NULL){
		// HANDLE EBP EVENT FILE
		RETURN_FALSE_IF_NULL (pEbs, this);

		pEbs->sessionName = getNextImportSessionName(EVENT_TRIGGER);
		pEbs->SessionFile = pEbs->sessionName + EBS_FILE_EXT;

		// - Create Directory inside the project directory
		dir_name = QString(m_pATuneFile->getDir() 
				+ pEbs->SessionFile) + ".dir";
		
		QDir ebs_dir;
		ebs_dir.mkdir(dir_name);

		caSession = dir_name 
			+ QString("/") 
			+ QString(pEbs->SessionFile);
 	
		// - Copy EBP/IBS file over.
		QString cmd;	
		cmd = "/bin/cp -f \"" + import_ebp_path + "\" \"" + caSession +"\"";  
		//fprintf(stderr,"DEBUG: Copy file : %s\n",cmd.data());
		if(system(cmd.data()) != 0)
		{
			m_importErrorMsg = QString("Error: Cannot copy ") 
					+ import_ebp_path
					+ " to "+caSession+".\n";
			return false;
		}

		// - Copy jit dir for java.
		if(!import_jit_dir_path.isEmpty())
		{
			cmd = "/bin/cp -rf \"" + import_jit_dir_path + "\" \"" + dir_name +"\"";  
			//fprintf(stderr,"DEBUG: Copy dir : %s\n",cmd.data());
			if(system(cmd.data()) != 0)
			{
				m_importErrorMsg = QString("Error: Cannot copy ") 
					+ import_jit_dir_path
					+ " to "+dir_name+".\n";
				return false;
			}
		}
	
		// - Adding EBP/IBS section to .caw file
		m_pATuneFile->addSession(pEbs);
		m_pATuneFile->saveAtuneFile ();
		initProjectWindow ();
		onSessionDblClicked (EVENT_TRIGGER, pEbs->sessionName);
	}


	return true;
} //bool ApplicationWindow::onImportEbpFile


bool ApplicationWindow::onImportIbsData ( 
	OP_EVENT_PROPERTY_MAP events_name_map,
	QString qs_return,
	ImportWizard * import_wizard,
	CEventsFile* eventFile)
{
	bool ret = false;

	OP_EVENT_PROPERTY_MAP ibs_events_name_map;
	EventMaskEncodeMap events_map;
	QString caSession = "";
	vector < string > events_name;
	QString import_cpuinfo_path = import_wizard->get_cpuinfo_path();

	IBS_OPTIONS *pIbs = new IBS_OPTIONS ();
	if(!pIbs){
		m_importErrorMsg = QString("Error importing profile data into IBS session.") ;
		return ret;
	}

	pIbs->sessionName = qs_return;
	pIbs->SessionFile = pIbs->sessionName + EBS_FILE_EXT;

	QDir ebs_dir;
	QString ibs_dir_name = QString(m_pATuneFile->getDir() + pIbs->SessionFile);
	ibs_dir_name += ".dir";
	ebs_dir.mkdir(ibs_dir_name);

	caSession = ibs_dir_name;
	caSession += QString("/");
	caSession += QString(pIbs->SessionFile);

	m_activeCounterMask = 0;

	/* Filter for IBS event */
	OP_EVENT_PROPERTY_MAP::iterator ev_it = events_name_map.begin ();
	OP_EVENT_PROPERTY_MAP::iterator ev_end = events_name_map.end ();

	for(; ev_it != ev_end ;ev_it++) {
		if(ev_it->second.op_name.startsWith("event:IBS_")) {
			ibs_events_name_map[ev_it->first] = ev_it->second;
		}
	}

	PopulateIbsSettingsForImportSession( pIbs, ibs_events_name_map,
				eventFile, events_map, import_wizard);

	/*****************************************************************
	* Set the session type with the corresponded options
	*/	

	m_pSession = static_cast<SESSION_OPTIONS *>(new IBS_OPTIONS ());

	IBS_OPTIONS *pIbsSession = static_cast <IBS_OPTIONS *> (m_pSession);
	pIbsSession->set(pIbs);

	/*****************************************************************
	* Translating Oprofile Samples Phase
	*/	
	if (!createOprofileTranslateObjects()) {
		m_importErrorMsg = QString("Translate Error: ") 
				+ "failed to create translate object.";
		return ret;
	}

	QString op_sample_dir =  import_wizard->get_op_sample_dir();

	m_translate->init(
		op_sample_dir, 
		XpFileContains(import_cpuinfo_path.data(), "processor"),
		ibs_dir_name, events_map, m_ca_display);

	m_translate_progress_dlg = new TranslateDlg (this, "TranslateDlg", true);
	if (NULL == m_translate_progress_dlg) {
		m_importErrorMsg = QString(DLG_ALLOC_ERROR_MSG);
		return ret;
	}

	m_translate_progress_dlg->setCaption ("Modules processed");
	connect (m_translate_progress_dlg, SIGNAL (dialogClosed()),
		this, SLOT (onModuleProcessingClosed()));

	m_ca_display->setDlgEvent (this);
	m_translate_progress_dlg->show();


	/*****************************************************************
 	 * Initialize the process filter for import
 	 */
	m_processFilter.clear();
	if(import_wizard->isApplyProcessFilter())
	{
		m_processFilter = import_wizard->getProcessFilter();
	}

	// Save to project setting
	m_pATuneFile->setProcessFilter(import_wizard->getProcessFilter());
	
	bool rt = m_translate->translate_op_to_ca(
					getFamilyFromCpuInfo(import_cpuinfo_path.data()),
						getModelFromCpuInfo(import_cpuinfo_path.data()), 
						m_processFilter, caSession);

	/*****************************************************************
	* Adding imported session and open the session window  
	*/
	if (rt) 
	{
		m_pATuneFile->addSession(pIbs);
		m_pATuneFile->saveAtuneFile ();
		initProjectWindow ();
		onSessionDblClicked (IBS_TRIGGER, pIbs->sessionName);
	} else{
		folderRemove(ibs_dir_name);
		m_importErrorMsg = QString(NO_SAMPLE_MSG);
		return ret;
	}
	ret = true;
	return ret;
} //bool ApplicationWindow::onImportIbsData 

bool ApplicationWindow::PopulateIbsSettingsForImportSession(
			IBS_OPTIONS *pIbs,
			OP_EVENT_PROPERTY_MAP &ibs_events_name_map,
			CEventsFile * eventFile,
			EventMaskEncodeMap &events_map,
			ImportWizard * import_wizard)
{
	bool ret = false;

	/*******************************************************
	* Extract the property information of IBS_FETCH and IBS_OP events.
	* Here, we do sanity check if it contains multiple configurations for
	* IBS_FETCH or IBS_OP samples
	*/ 
	unsigned long fetchInterval = 0;	
	unsigned long opInterval    = 0;
	bool multipleFetch          = false;
	bool multipleOp             = false;

	OP_EVENT_PROPERTY_MAP::iterator it = ibs_events_name_map.begin ();
	OP_EVENT_PROPERTY_MAP::iterator end = ibs_events_name_map.end ();
	for (int i = 0;  it != end; it++, i++) {
		CpuEvent cpu_event;
		string name = it->first.data ();
		eventFile->findEventByOPName(name.data(), cpu_event);
		m_activeCounterMask |= 1 << i; //????

		// Get the IBS_OP and IBS_FETCH properties
		QString ev_name = it->first.data();
		if(ev_name.contains("IBS_FETCH_")) {
			pIbs->fetchList.append(cpu_event.value);
			if(fetchInterval > 0 && fetchInterval != it->second.count) 
				multipleFetch = true;
			else
				fetchInterval = it->second.count;
		} else if(ev_name.contains("IBS_OP_")) {
			pIbs->opList.append(cpu_event.value);
			if(opInterval > 0 && opInterval != it->second.count)
				multipleOp = true;
			else
				opInterval = it->second.count;
		}

		// NOTE: Instead of calling enumerateAllOPEvent,
		// we can just simply do this here.
		//events_name.push_back(name);
		EventEncodeType eet;
		eet.sortedIndex = i; 
		eet.weight = 1;
		eet.eventCount = 0; 
		eet.eventMask = EncodeEventMask(cpu_event.value,
					   	it->second.unitmask);
		events_map.insert(EventMaskEncodeMap::value_type(name, eet));
	}

	if(multipleFetch)
	{
		QMessageBox::information (this, 
			"CodeAnalyst Error",
			"Importing session contains multiple"
			"IBS FETCH configurations.\n"
			"Please run opcontrol --reset before profiling.");
		return ret;
	}

	if(multipleOp)
	{
		QMessageBox::information (this, 
			"CodeAnalyst Error",
			"Importing session contains multiple"
			"IBS OP configurations.\n"
			"Please run opcontrol --reset before profiling.");
		return ret;
	}

	// Set the IBS_OP and IBS_FETCH properties
	pIbs->fetchInterval = fetchInterval;
	pIbs->opInterval    = opInterval;

	if(fetchInterval != 0) 
		pIbs->fetchSample = true;
	else
		pIbs->fetchSample = false;

	if(opInterval != 0) 
	{
		pIbs->opSample = true;

		/***********************************
 		 * Check for IBS-OPs dispatched_ops flag
 		 * - In case of remote import, we have to check 
 		 *   the "cainfo/dispatched_ops" file in the package.
 		 * - If doesn't exist, default to Op Cycle count
 		 */
		QString import_dispatched_ops_path;
		QTextStream stream;
		QString line;

		import_dispatched_ops_path = import_wizard->get_cainfo_dispatched_ops_path();

		//Default value
		pIbs->opCycleCount = true;

		QFile dispatched_ops_file;
		dispatched_ops_file.setName(import_dispatched_ops_path);
		if(dispatched_ops_file.exists())
		{
			// Open file
			if(dispatched_ops_file.open(IO_ReadOnly))
			{
				stream.setDevice(&dispatched_ops_file);
				line = stream.readLine();
				if(line == QString("1"))	
					pIbs->opCycleCount = false;

				// Close file
				dispatched_ops_file.close();
			}
		}
	}else
		pIbs->opSample = false;
	
	ret = true;
	return ret;
} // bool ApplicationWindow::PopulateIbsSettingsForImportSession

void ApplicationWindow::PopulateEbpSettingsForImportSession(
			EBP_OPTIONS *pEbs,
			OP_EVENT_PROPERTY_MAP &pmc_events_name_map,
			CEventsFile * eventFile,
			EventMaskEncodeMap &events_map)
{
	EventMaskMap eventMaskMap;
	OP_EVENT_PROPERTY_MAP::iterator it  = pmc_events_name_map.begin ();
	OP_EVENT_PROPERTY_MAP::iterator end = pmc_events_name_map.end ();
	for (int i = 0, group = 0, ebp_event_index = 0; it != end; it++, i++) 
	{
		CpuEvent cpu_event;
		string name = it->first.data ();
		eventFile->findEventByOPName(name.data(), cpu_event);
		m_activeCounterMask |= 1 << i; //?????


		// We need to check for any duplicate events (same event no and mask)
		EventMaskType cur = EncodeEventMask(cpu_event.value, it->second.unitmask);

		if(eventMaskMap.find(cur) == eventMaskMap.end())
		{
			eventMaskMap.insert(EventMaskMap::value_type(cur,it->second.count));
		}else{
			QMessageBox::warning(this, "CodeAnalyst Warning", 
				"CodeAnalyst found duplicates of event:0x" + QString::number(cpu_event.value,16) 
				+ ", mask:0x" + QString::number(it->second.unitmask,16)
				+ ", with count:" + QString::number(it->second.count)
				+ ",\n"
				+ "This events will not be imported.\n"
				+ "If this is an error, please re-run the profile.");
			continue;
		}	

		/* Fill the EBP_Options */
		group = i / MAX_EVENTNUM;	
		ebp_event_index = i % MAX_EVENTNUM;
		pEbs->pEvents[group].eventSelect[ebp_event_index] = cpu_event.value;
		pEbs->pEvents[group].eventUnitMask[ebp_event_index] = it->second.unitmask;
		pEbs->pEvents[group].eventOS[ebp_event_index] = it->second.os;
		pEbs->pEvents[group].eventUser[ebp_event_index] = it->second.usr;
		pEbs->pEvents[group].eventCount[ebp_event_index] = it->second.count;
		pEbs->pEvents[group].edgeDetect[ebp_event_index] = 0;
		pEbs->pEvents[group].numberOfValidEvents = ebp_event_index + 1;


		// NOTE: Instead of calling enumerateAllOPEvent,
		// we can just simply do this here.
		if (events_map.find(name) == events_map.end()) {
			//events_name.push_back(name);
			EventEncodeType eet;
			eet.sortedIndex = events_map.size();
			eet.weight = 1;
			eet.eventCount = it->second.count; 
			eet.eventMask = EncodeEventMask(cpu_event.value,
						it->second.unitmask);

			events_map.insert(EventMaskEncodeMap::value_type(name, eet));
		}
	}

} //void ApplicationWindow::PopulateEbpSettingsForImportSession

bool ApplicationWindow::onImportEbpData ( 
	OP_EVENT_PROPERTY_MAP events_name_map,
	QString qs_return,
	ImportWizard * import_wizard,
	CEventsFile * eventFile)
{
	bool ret = false;

	OP_EVENT_PROPERTY_MAP pmc_events_name_map;
	EventMaskEncodeMap events_map;
	QString caSession = "";
	vector < string > events_name;
	QString import_cpuinfo_path = import_wizard->get_cpuinfo_path();

	EBP_OPTIONS *pEbs = new EBP_OPTIONS ();
	if(!pEbs)
	{
		m_importErrorMsg = QString("Error importing profile data into EBP session.") ;
		return ret;
	}

	pEbs->sessionName = qs_return;
	pEbs->SessionFile = pEbs->sessionName + EBS_FILE_EXT;
	pEbs->msMpxInterval = 0;

	QDir ebs_dir;
	QString ebs_dir_name = QString(m_pATuneFile->getDir() + pEbs->SessionFile);
	ebs_dir_name += ".dir";
	ebs_dir.mkdir(ebs_dir_name);

	caSession = ebs_dir_name;
	caSession += QString("/");
	caSession += QString(pEbs->SessionFile);

	/***********************************************************
	* TODO: This code should combine with the similar code in SS_FINISH case
	* of function setSamplingState()
	*/
	m_activeCounterMask = 0;

	//***********************************************************
	// Filter for PMC event 
	OP_EVENT_PROPERTY_MAP::iterator ev_it = events_name_map.begin ();
	OP_EVENT_PROPERTY_MAP::iterator ev_end = events_name_map.end ();

	for(; ev_it != ev_end ;ev_it++)
	{
		if(!ev_it->second.op_name.contains("IBS"))
		{
			pmc_events_name_map[ev_it->first] = ev_it->second;                
		}
	}

	//***********************************************************
	// Calculate the number of groups for PMC events 
	pEbs->countEventGroups = pmc_events_name_map.size() / MAX_EVENTNUM;
	pmc_events_name_map.size() % MAX_EVENTNUM ? pEbs->countEventGroups++ : 0;	
	pEbs->pEvents = new EventGroup[pEbs->countEventGroups];

	PopulateEbpSettingsForImportSession(pEbs, pmc_events_name_map, 
						eventFile, events_map);
	
	//***********************************************************
	// Initialize EBP Session setting

	m_pSession = static_cast<SESSION_OPTIONS *>(new EBP_OPTIONS ());

	EBP_OPTIONS *pEventSession = static_cast <EBP_OPTIONS *> (m_pSession);
	pEventSession->set(pEbs);

	/*****************************************************************
	* Translating Oprofile Samples Phase
	*/	
	if (!createOprofileTranslateObjects()) {
		m_importErrorMsg = QString("Translate Error: ") 
				+ "failed to create translate object.";
		return ret;
	}

	QString op_sample_dir =  import_wizard->get_op_sample_dir();

	m_translate->init(
		op_sample_dir,
		XpFileContains(import_cpuinfo_path.data(), "processor"),
		ebs_dir_name, events_map, m_ca_display);

	m_translate_progress_dlg = new TranslateDlg (this, "TranslateDlg", true);
	if (NULL == m_translate_progress_dlg) {
		m_importErrorMsg = QString(DLG_ALLOC_ERROR_MSG);
		return ret;
	}

	m_translate_progress_dlg->setCaption ("Modules processed");
	connect (m_translate_progress_dlg, SIGNAL (dialogClosed()),
		this, SLOT (onModuleProcessingClosed()));

	m_ca_display->setDlgEvent (this);
	m_translate_progress_dlg->show();

	/*****************************************************************
 	 * Initialize the process filter for import
 	 */
	m_processFilter.clear();	
	if(import_wizard->isApplyProcessFilter())
	{
		m_processFilter= import_wizard->getProcessFilter();
	}

	// Save to project setting
	m_pATuneFile->setProcessFilter(import_wizard->getProcessFilter());
	
	bool rt = m_translate->translate_op_to_ca(
				getFamilyFromCpuInfo(import_cpuinfo_path.data()),
					getModelFromCpuInfo(import_cpuinfo_path.data()),
					m_processFilter, caSession);

	/*****************************************************************
	* Adding imported session and open the session window  
	*/
	if (rt) 
	{
		m_pATuneFile->addSession(pEbs);
		m_pATuneFile->saveAtuneFile ();
		initProjectWindow ();
		onSessionDblClicked (EVENT_TRIGGER, pEbs->sessionName);
	} else {
		folderRemove(ebs_dir_name);
		m_importErrorMsg = QString(NO_SAMPLE_MSG);
		return ret;
	}
	ret = true;
	return ret;
} //bool ApplicationWindow::onImportEbpData

void ApplicationWindow::onImportData ()
{
	bool ret = false;

	ImportWizard * import_wizard = new ImportWizard(this);

	// Setup Process Filter list for the import wizard
	import_wizard->setProcessFilter(m_pATuneFile->getProcessFilter());	
	if (QDialog::Rejected == import_wizard->exec())
		return;

	// EBP File Import
	if(import_wizard->is_ebp_import()) {
		ret = onImportEbpFile(import_wizard);		
	} 
	// XML File Import
	else if(import_wizard->is_xml_import()) {
		ret = onImportXmlFile(import_wizard);		
	} 
	// LOCAL/REMOTE Import
	else {
		ret = onImportLocalRemoteFile(import_wizard);		
	}

	if(!ret)
	{
		QMessageBox::critical(this, "Error Importing Profile", 
			"There is an error in importing profile data.\n"
			"Please check your setting and try again.\n\n"
			"Reason:\n" + m_importErrorMsg);
	}
} // void ApplicationWindow::onImportData 


bool ApplicationWindow::onImportLocalRemoteFile (ImportWizard *import_wizard)
{
	QString op_sample_dir = "";
	QString project_path;
	RUN_OPTIONS     go;
	QStringList path_list;
	OP_EVENT_PROPERTY_MAP events_name_map;
	QString qs_return;
	CEventsFile * eventFileForImport;

	clearOPEBSProperties();

	/***********************************************************
	* Sanity check for blank importing path
	*/ 
	op_sample_dir = import_wizard->get_op_sample_dir();
	if (op_sample_dir.isEmpty())
	{
		m_importErrorMsg = QString("Error: Please specify Oprofile sample dir.\n") 
				+ "( i.e. /var/lib/oprofile/samples/current )" ;
		return false;
	}

	path_list = QStringList::split (DIRSLASH, op_sample_dir);
	project_path = m_pATuneFile->getPath ();
	project_path.truncate (project_path.findRev (XPS_PATH_SLASH) + 1);

	/***********************************************************
	* Acquiring a list of events from the oprofile samples and
	* determine whether this is ibs events.
	*/
	QDir d (op_sample_dir);

	// Sanity check to see if this directory really contains oprofile samples
	// by checking for directory {root} or {kern}
	if ( !d.exists()
	|| ( !d.exists("{root}") && !d.exists("{kern}")))
	{
		m_importErrorMsg = QString("Error: Please specify Oprofile sample dir.\n") 
				+ "( i.e. /var/lib/oprofile/samples/current )\n" 
				+ " or check if directory " 
				+ op_sample_dir + "/{root} or {kern} exists.";
		return false;	
	}

	/***********************************************************
	* If this is a remote import, we have to make sure that
	* we using the event files corresponded to the platform
	*/
	if(import_wizard->is_remote_import())
	{
		QString import_cpuinfo_path = import_wizard->get_cpuinfo_path();
		QString file; 

		if(QFile::exists(import_cpuinfo_path))
		{
			unsigned long cpuModel  = getModelFromCpuInfo(import_cpuinfo_path.data());
			unsigned long cpuFamily = getFamilyFromCpuInfo(import_cpuinfo_path.data());
			file = helpGetEventFile(cpuFamily, cpuModel);
		}

		eventFileForImport = new CEventsFile();
		eventFileForImport->open(file);
	}else{
		eventFileForImport = &m_eventFile;	
	}
	// ***********************************************************

	if (!validateDirContainsOPSamples (d, *eventFileForImport))
	{
		m_importErrorMsg = QString(NO_SAMPLE_MSG) ;
		return false;
	}
	findOprofileEvents (d, events_name_map);

	/***********************************************************
	* Determining the type of session.  In case, the import session
	* contains both IBS and EBP events, we separate them into two
	* session 
	*/
	bool ret = false;
	bool hasEbs = containEbsEvent(events_name_map);
	bool hasIbs = containIbsEvent(events_name_map);

	if (hasEbs && hasIbs)
	{
		QMessageBox::warning( this, "CodeAnalyst Warning",
			"Imported data contain both PMC and IBS events.\n"
			"CodeAnalyst will separate them into PMC and IBS session.");
	}

	if(hasEbs)
	{
		/******************************
		* Handling session name
		*/
		m_pATuneFile->getRunOptions (&go);

		// Get the next session name
		QString oldSessionName = go.sessionName;
		go.sessionName += " import";
		m_pATuneFile->setRunOptions (go);

		qs_return = m_pATuneFile->getNextSessionName (EVENT_TRIGGER);

		//Reset the session name
		go.sessionName = oldSessionName;
		m_pATuneFile->setRunOptions (go);

		/******************************
		* Import EBP data
		*/
		ret = onImportEbpData ( events_name_map,
			qs_return,
			import_wizard,
			eventFileForImport);
		if(!ret)
			goto out;
	} 

	if(hasIbs)
	{
		/******************************
		* Handling session name
		*/
		m_pATuneFile->getRunOptions (&go);

		// Get the next session name
		QString oldSessionName = go.sessionName;
		go.sessionName += " import";
		m_pATuneFile->setRunOptions (go);

		qs_return = m_pATuneFile->getNextSessionName (IBS_TRIGGER);

		//Reset the session name
		go.sessionName = oldSessionName;
		m_pATuneFile->setRunOptions (go);

		/******************************
		* Import EBP data
		*/
		ret = onImportIbsData ( events_name_map,
			qs_return,
			import_wizard,
			eventFileForImport);
	}

	/*
	* NOTE: If this is a remote import, we also copy the /proc/cpuid
	*	 file in the session directory which will be used 
	* 	 for viewing session property 
	*/ 
	if(ret && import_wizard->is_remote_import())
	{
		QString import_cpuinfo_path = import_wizard->get_cpuinfo_path();
		if(QFile::exists(import_cpuinfo_path))
		{
			QString session_cpuinfo = QString("\"") 
				+ m_pATuneFile->getDir() 
				+ qs_return
				+ ".ebp"
				+ DIR_FILE_EXT 
				+ "/cpuinfo\"";

			QString command = "cp " + import_cpuinfo_path
				+ " " + session_cpuinfo;
			system(command.data());
		}

		//Also, close the event file.
		eventFileForImport->close();
		if(NULL != eventFileForImport){
			delete eventFileForImport;
			eventFileForImport = NULL;	
		}
	}

out:
	//***********************************************************
	if (NULL != import_wizard) {
		delete import_wizard;
		import_wizard = NULL;
	}

	return ret;
} //void ApplicationWindow::onImportLocalRemoteFile


void ApplicationWindow::clearOPEBSProperties()
{
	for (int i = 0; i < MAX_EVENTNUM_MULTIPLEXING; i++)
	{
		op_ebs_properties[i].eventmask = 0;
		op_ebs_properties[i].count = 0;
		op_ebs_properties[i].unitmask = 0;
		op_ebs_properties[i].os = 0;
		op_ebs_properties[i].usr = 0;
		op_ebs_properties[i].op_name = "";
	}
}


bool ApplicationWindow::validateDirContainsOPSamples (const QDir & _d, CEventsFile & _e)
{
	QDir d = _d;
	return (findOPSamplesFile (d, _e));
}


bool ApplicationWindow::findOPSamplesFile (const QDir & _d, CEventsFile & _e)
{
	QDir d = _d;
	QStringList entries = d.entryList ();
	bool ret = false;

	for (QStringList::Iterator it = entries.begin ();
		it != entries.end (); ++it)
	{
		if (*it == "." || *it == "..")
			continue;
		QFileInfo info (d, *it);
		if (info.isDir ())
		{
			if (!(ret = findOPSamplesFile (QDir (info.filePath ()), _e)))
				continue;
			else
			{
				return ret;
			}
		}
		else
		{
			QString event_name = parseOPSampleFileName (info.fileName ());
			if (_e.isEventExist(event_name))
			{
				return true;
			}
		}
		ret = false;
	}
	return ret;
}


/**
* Given a directory, parse the file names in the directory
* to find the events Oprofile profiled.
*/
bool ApplicationWindow::findOprofileEvents (const QDir & _d,
			OP_EVENT_PROPERTY_MAP &events_name_map)
{
	QDir d = _d;

	QStringList entries = d.entryList ();
	QStringList::Iterator it     = entries.begin ();
	QStringList::Iterator it_end = entries.end ();
	for (; it != it_end ; ++it)
	{
		if (*it == "." || *it == "..")
			continue;
		QFileInfo info (d, *it);
		if (info.isDir ())
		{
			if (!findOprofileEvents (QDir (info.filePath ()), events_name_map))
				return false;
		}
		else
		{
			parseOPSampleFileName (info.fileName (), events_name_map);
		}
	}

	QString name = d.dirName ();

	if (!d.cdUp ())
		return false;

	return true;
}


QString ApplicationWindow::parseOPSampleFileName (const QString & fname)
{

	QStringList strlist = QStringList::split (".", fname);

	if (0 != strlist.size ())
	{
		return strlist[op_field_name];
	} else {
		return QString ("");
	}
}


void ApplicationWindow::parseOPSampleFileName (const QString & fname,
		   OP_EVENT_PROPERTY_MAP &events_name_map)
{

	bool ok;

	QStringList strlist = QStringList::split (".", fname);

	if (0 != strlist.size ())
	{

		QString key = "event:" + strlist[op_field_name] + " "
			+ "count:" + strlist[op_field_count] + " "
			+ "unit-mask:" + strlist[op_field_unitmask];

		string parsed_name 			= string (key.data ());
		events_name_map[parsed_name].count 	= strlist[op_field_count].toUInt (&ok);
		events_name_map[parsed_name].unitmask 	= strlist[op_field_unitmask].toUInt (&ok);
		events_name_map[parsed_name].os 	= true;
		events_name_map[parsed_name].usr 	= true;
		events_name_map[parsed_name].op_name 	= parsed_name.c_str ();
	}
}


//===========================================================================
//onCascade

//@mfunc This is a custom function because the qt version of cascade() doesn't
//      work right.

//@comm Got the offset values from Qt::QWorkspace.cpp file

//===========================================================================
void ApplicationWindow::onCascade ()
{
	// from QWorkspace.cpp
	const int xoffset = 13;
	const int yoffset = 23;
	const int width = 640;
	const int height = 480;
	int x = 0;
	int y = 0;

	ws->cascade ();
	int numWnd = ws->windowList ().count ();

	for (int i = 0; i < numWnd; ++i)
	{
		QWidget *pWnd = ws->windowList ().at (i);

		if (!pWnd->isHidden ())
		{
			pWnd->showNormal ();
			pWnd->parentWidget ()->setGeometry (x, y, width, height);
			if ((y + yoffset + height) <= ws->height ())
			{
				x += xoffset;
				y += yoffset;
			}
			else
			{
				x = 0;
				y = 0;
			}
		}
	}
}// ApplicationWindow::onCascade

void ApplicationWindow::setDefaultSize(QWidget* pWnd)
{
	const int width = 640;
	const int height = 480;
	int x = 0;
	int y = 0;

	pWnd->showNormal ();
	pWnd->parentWidget ()->setGeometry (x, y, width, height);
}

bool ApplicationWindow::isCorrectDriverForGH()
{
	bool ret = false;
	QString line;
	QTextStream stream;
	
	if( m_this_cpuFamily == GREYHOUND_FAMILY)
	{
		QFile cpu_type("/dev/oprofile/cpu_type");
		if (!cpu_type.open(IO_ReadOnly))
			goto out;

		stream.setDevice(&cpu_type);
		line = stream.readLine();
		if( line == QString("x86-64/family10")	
		||  line == QString("x86-64/family10h"))	
			ret = true;	

		cpu_type.close();
	}else{
		ret = true;
	}
out:
	return ret;
}

/* 
* folderRemove:
* Recursive contents within the folder and the folder
* itself.
* It is the same code from oprofile_interface.cpp
*/
bool ApplicationWindow::folderRemove (const QDir & _d)
{
	QDir d = _d;

	QStringList entries = d.entryList(QDir::All|QDir::Hidden);
	QStringList::Iterator it = entries.begin();

	for (; it != entries.end (); ++it)
	{
		if (*it == "." || *it == "..")
			continue;
		QFileInfo info (d, *it);
		if (info.isDir ())
		{
			if (!folderRemove (QDir (info.filePath ())))
				return false;
		}
		else
		{
			d.remove (info.filePath ());
		}
	}

	QString name = d.dirName ();

	if (!d.cdUp ())
		return false;

	d.rmdir (name);
	return true;
}


void ApplicationWindow::enumerateDir(QString & dirName, PID_MAP_T & map)
{
	QDir dir(dirName, QString::null, QDir::Name, QDir::Dirs );
	QStringList dirList = dir.entryList();

	QStringList::Iterator it = dirList.begin();
	for (; it != dirList.end(); it++)
	{
		bool ok = false;
		QString s = * it;
		unsigned int pid = 0;
		pid = s.toUInt(&ok);

		if (!ok)
			continue;

		map[pid] = 0;
	}
}


void ApplicationWindow::onToggleChartDensity ()
{
	QString toggle_text;
	DWORD density;
	//Toggle here
	if (!m_pWindowsMenu->isItemChecked (m_toggleDensityId))
		density = SHOW_CHART_DENSITY;
	else
		density = NO_SHOW_CHART_DENSITY;

	//I wrapped this in this section, so that the ATuneOptions would be written
	// before the charts checked to see if they should be visible or not.
	{
		CATuneOptions ao;
		ao.setShowChartDensity (density);
	}

	m_pWindowsMenu->setItemChecked (m_toggleDensityId,
		(SHOW_CHART_DENSITY == density));

	//ideally, this signal would be connected to the density charts, however
	//due to data hiding, the signal passes through several levels.
	emit densityVisibilityChanged ();
}


//Because the dialog is non-modal and gets deleted when it's closed to prevent
// memory leaks, there is a signal when it is closed
void ApplicationWindow::onModuleProcessingClosed()
{
	m_translate_progress_dlg = NULL;
}


int ApplicationWindow::OnSessionSettings()
{
	if (SS_IDLE != m_SamplingState)
	{
		QMessageBox::warning (this, "CodeAnalyst Warning", "You cannot change "
			"the session options while sampling.");
		return QDialog::Rejected;
	}
	SessionOptionsDlg *pDlg = new SessionOptionsDlg (m_pATuneFile, this);
	if (NULL == pDlg)
	{
		QMessageBox::critical (this, NOMEM_HEADER, NOMEM_MSG);
		return QDialog::Rejected;
	}

	//send the list of available profile configurations to the dialog
	pDlg->initFromProject (&m_profiles);
	int result = pDlg->exec();

	setSamplingState (SS_IDLE);

	QString profile = m_pATuneFile->getLastProfile ();

	if (QDialog::Accepted == result)
	{
		m_pProfileBox->clear();
		m_pProfileBox->insertStringList (m_profiles.getListOfProfiles ());
		m_pProfileBox->setCurrentText (profile);
	}
	RUN_OPTIONS gen;
	m_pATuneFile->getRunOptions( &gen );
	return result;
}


void ApplicationWindow::OnProfileManage ()
{
	ManageCfgsDlg *pManageDlg = new ManageCfgsDlg (&m_profiles, this);
	RETURN_IF_NULL (pManageDlg, this);

	//IF they changed something in the dialog box, update the display
	pManageDlg->exec();
	//reload profile list...
	m_pProfileBox->clear();
	m_pProfileBox->insertStringList (m_profiles.getListOfProfiles ());

	// Set current profile
	if(!m_pATuneFile)
		return;

	QString profile = m_pATuneFile->getLastProfile ();
	if (!profile.isEmpty ())
	{
		for (int i = m_pProfileBox->count()-1 ; i >= 0 ; i--)
		{
			if(m_pProfileBox->text(i) == profile)
			{
				m_pProfileBox->setCurrentItem (i);
			}
		}
	}
}
