//$Id: application.h,v 1.11 2006/05/15 22:09:22 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef APPLICATION_H
#define APPLICATION_H

#include <vector>
#include <string>
#include <map>

#include "atunefile.h"
#include "projectwindow.h"
#include "progressmgr.h"
#include "tbsStruct.h"
#include "sessionnav.h"
#include "ImportWizard.h"
#include "oprofile_interface.h"
#include "translatedlg.h"
#include "catranslate_mod.h"
#include "opdata_handler.h"
#include "catranslate_display.h"
#include "newProjectDlg.h"
#include "ProfileCollection.h"
#include "eventsfile.h"

/// Headers from Oprofile
//#include "op_config.h"


#ifndef OP_BASE_DIR
#define OP_BASE_DIR "/var/lib/oprofile/"
#endif 

#ifndef OP_SAMPLES_DIR
#define OP_SAMPLES_DIR OP_BASE_DIR "samples/"
#endif

#ifndef OP_SAMPLES_CURRENT_DIR
#define OP_SAMPLES_CURRENT_DIR OP_SAMPLES_DIR "current/"
#endif

#ifndef OP_LOCK_FILE
#define OP_LOCK_FILE OP_BASE_DIR "lock"
#endif

#ifndef OP_LOG_FILE
#define OP_LOG_FILE OP_SAMPLES_DIR "oprofiled.log"
#endif

#ifndef OP_DUMP_STATUS
#define OP_DUMP_STATUS OP_BASE_DIR "complete_dump"
#endif

#ifndef OP_JAVA_DIR
#define OP_JAVA_DIR  OP_BASE_DIR "Java/"
#endif

#ifndef OP_JIT_DIR
#define OP_JIT_DIR  OP_BASE_DIR "jit/"
#endif

using namespace std;


////////////////////////////////////
// Menu ID's under the Sampling Menu
//
enum {
	SM_START = 0,
	SM_PAUSE,
	SM_STOP,
	SM_ABORT,
	SM_OPTIONS,
	MAX_MENU_NUM
	//TODO

};

///////////////////////////////////
// Menu ID's under the Tools Menu
enum {
	TM_SETTINGS = 0,
	TM_CAOPTIONS,
	TM_EDITPROFILES,
	TM_EDITVIEWS,
	TM_DIFFANALYST,
	TM_LAST
};

///////////////////////////////////
// Position of the Tool Bar menu item
#define FILE_TB_ID	0
#define SAMP_TB_ID	1
#define TRACE_TB_ID	2

#define MAX_TOOLBARS	4

///////////////////////////////////
// Sampling States
//
enum SAMPLING_STATE
{
	SS_DELAY,
	SS_RUNNING,
	SS_PAUSED,
	SS_IDLE,
	SS_RESUMED,
	SS_FINISHED
};


///////////////////////////////////
// OP_FILE_NAME_FIELD
//
enum OP_FILE_NAME_FIELD
{
	op_field_name = 0,
	op_field_count,
	op_field_unitmask,
};


//////////////////////////////////////////////////////////////////////////
// MruEntry
//
//////////////////////////////////////////////////////////////////////////
typedef struct _MruEntry
{
	QString filePath;
	int	index;
} MruEntry, *PMruEntry;

typedef QValueList<MruEntry> MruList;


//////////////////////////////////////////////////////////////////////////
// PID_MAP_T
//
//////////////////////////////////////////////////////////////////////////
typedef QMap<unsigned int, unsigned int>  PID_MAP_T;


const unsigned int CPU_UNHALTED_CYCLES = 0x76;
const QString OP_DEFAULT_SAMPLE_DIR =  OP_SAMPLES_DIR;
const QString OP_DEFAULT_DIR = OP_BASE_DIR;
const QString DLG_ALLOC_ERROR_MSG = "Could not allocate memory for the dialog "
"box";



//////////////////////////////////////////////////////////////////////////
class ApplicationWindow: public QMainWindow
{
	Q_OBJECT
public:
	ApplicationWindow();
	~ApplicationWindow();

public:
	QString getCurrentFilePath();
	bool setSamplingState( SAMPLING_STATE State );
	static bool findOPSamplesFile(const QDir &_d, CEventsFile & _e);
	static bool validateDirContainsOPSamples (const QDir &_d, CEventsFile & _e);
	static void parseOPSampleFileName(const QString & fname, 
		OP_EVENT_PROPERTY_MAP & events_name);
	static QString parseOPSampleFileName(const QString & fname);
	static bool folderRemove (const QDir & _d);

protected:
	void customEvent( QCustomEvent *pCustEvent );
	void closeEvent( QCloseEvent *pCloseEvent );
	void saveMRUToReg();

private slots:
	void targetExit();
	void onFileNew();
	void onAboutToQuit();

	void onFileOpen();
	void onFileSave();
	void onFileClose(bool *isOk=NULL);

	void onHelpAbout();
	void onHelpContents();
	void onHelpSysInfo();

	void windowsMenuAboutToShow();
	void windowsMenuActivated( int id );
	QWidget* findWindow( QString caption );
	void onCascade();
	void onToggleChartDensity ();

	void initProjectWindow();

	void onSamplingStart();
	void onSamplingPause();
	void onSamplingStop ();

	void OnProfileChange (const QString &profile);

	void onMruItemSelected(int id);


	void onSessionDeletedOnly (TRIGGER trigger, QString SessionName);
	void onSessionDeleted (TRIGGER trigger, QString SessionName);
	void onSessionDblClicked (TRIGGER trigger, QString SessionName);
	void onSessionRenamed (TRIGGER trigger, QString oldName, QString newName);
	void onTbsSessionProperties( QString SessionName );
	void onEbsSessionProperties( QString SessionName );
	void onIbsSessionProperties( QString SessionName );
	void OnCopySessionToCurrent (TRIGGER trigger, QString SessionName);

	int OnSessionSettings();
	int onToolsATuneOptions();
	void OnProfileManage ();
	void OnViewManage ();
	void OnOpenDiffAnalyst(DiffSessionInfo *sess = NULL);

	void onWindowActivated( QWidget *pWnd );
	void onDataViewChanged( QString exportString, QListView *pList );
	void onModuleProcessingClosed();
	void onExportData();
	void onImportData();
	bool onImportEbpFile(ImportWizard * import_wizard) ;
	bool onImportXmlFile(ImportWizard * import_wizard) ;
	bool onImportLocalRemoteFile(ImportWizard * import_wizard) ;
	bool onImportEbpData ( OP_EVENT_PROPERTY_MAP events_name_map,
		QString qs_return,
		ImportWizard * import_wizard,
		CEventsFile * eventFile);
	bool onImportIbsData ( OP_EVENT_PROPERTY_MAP events_name_map,
		QString qs_return,
		ImportWizard * import_wizard,
		CEventsFile * eventFile);

	void closeAllWindows();

signals:
	void densityVisibilityChanged ();
	void shownChanged(QString viewName);

private:
	void print_op_events();
	void enumerateAllOpEvents(vector<string>& events, EventMaskEncodeMap & events_map); 
	void enumerateAllIbsEvents(vector<string>& events, EventMaskEncodeMap & events_map); 
	void setDefaultSize(QWidget* pWnd);


	void setGUIButtonsStart();
	int launchProgressManager();	
	bool runEbs ();
	bool runTbs ();
	bool runIbs ();
	QString getNextImportSessionName(TRIGGER trigger);
	void LoadFile( QString FileName );
	void resizeAllWindows( QSize NewSize );
	void parseMruString( const char *mrulist );
	void insertMruItems();
	void updateMruFile( QString NewFilePath );
	void removeMruFile( QString filePath );
	bool mruFileContains( QString FilePath );
	void updateMruItemIndex( QString file, int index);
	void registerHotKeys();
	bool sampling();
	bool isCorrectDriverForGH();
	float translateMSToTicks(float ms);

	bool findOprofileEvents (const QDir & _d, OP_EVENT_PROPERTY_MAP & events_name);

	void clearOPEBSProperties();
	void handleCADriverInterfaceError(unsigned int errorCode);
	bool ProcessTIPRD (QString ca_result_filename );
	void enumerateDir(QString & dirName, PID_MAP_T & map);

	void ReadEventsFromPRD(int * nEvent);

	BOOL startDaemon(unsigned int novmlinux,
		QString vmlinux_dir, 
		unsigned int buffer_size,
		unsigned int watershed_size,
		unsigned int cpu_buf_size, 
		oprofile_interface_ns::profiling_types);

	//todp: place this in the factory class
	bool createOprofileTranslateObjects();
	void deleteOprofileTranslateObjects();

	bool containIbsEvent(OP_EVENT_PROPERTY_MAP & event_name_map);
	bool containEbsEvent(OP_EVENT_PROPERTY_MAP &events_name_map);
	void PopulateEbpSettingsForImportSession(EBP_OPTIONS *pEbs,
			OP_EVENT_PROPERTY_MAP &pmc_events_name_map,
			CEventsFile * eventFile,
			EventMaskEncodeMap &events_map);
	bool PopulateIbsSettingsForImportSession(IBS_OPTIONS *pIbs,
			OP_EVENT_PROPERTY_MAP &pmc_events_name_map,
			CEventsFile * eventFile,
			EventMaskEncodeMap &events_map,
			ImportWizard * import_wizard);

private:
	int		m_ExportDataId;
	int		m_ImportDataId;
	int		m_SaveId;
	int		m_CloseId;
	int             m_toggleDensityId;
	ProjectWindow	*m_pProjWnd;
	QListView	*m_pTree;
	ATuneFile	*m_pATuneFile;
	int		m_SamplingState;
	int		m_FileSepId;
	bool		m_bDocumentOpen;
	bool		m_bHotKeysLoaded;
	bool		m_bFirstDialog;
	bool		m_bLegacyTimer;
	int		m_nDocNum;
	QLabel		*m_pStatusMsg;
	ProgressMgr	*m_pProgress;
	SESSION_OPTIONS *m_pSession;
	TranslateDlg    *m_translate_progress_dlg;
	QProgressBar	*m_pStatusProgress;
	QSplitter	*m_pSplitter;
	QVBox		*m_pVBox;
	QWidgetList	m_Windows;
	QListView	*m_pExportList;
	MruList		m_MruList;
	unsigned int	m_tick_per_sec;
	unsigned int 	m_activeCounterMask; 
	oprofile_interface m_opIf;
	unsigned long 	m_this_cpuFamily;
	unsigned long 	m_this_cpuModel;

	/*
	* Oprofile only handles up to MAX_EVENTNUM, the GUI
	* needs to break events to different groups and do
	* multiple runs to circumvent this limitation.
	* ob_ebs_properties are passed to the daemon, thus only
	* can contain up to four events.
	*/
	op_event_property op_ebs_properties[MAX_EVENTNUM_MULTIPLEXING];
	CEventsFile m_eventFile;
	uid_t m_euid;

	QComboBox *m_pProfileBox;
	ProfileCollection m_profiles;

	PID_MAP_T m_RecordedPidMap;
	PID_MAP_T m_PidMap;

	unsigned int m_novmlinux;
	QString m_vmlinux;
	unsigned int m_sample_buf_size;
	unsigned int m_cpu_buf_size;

	SAMPLING_STATE m_prevState;

	CATranslate * m_translate;
	ca_translate_display * m_ca_display;



	/* Which group of the events of EBP_OPTIONS->pEvents is 
	* being profiled by the GUI, only valid when the state of
	* GUI is SS_RUNNING.
	*/
	int m_totalEventGroup;
	QWorkspace	*ws;
	QToolBar	*fileTools;
	QToolBar	*pSamplingToolBar;
	QToolBar	*pNavToolBar;
	QToolBar *m_pToolsToolBar;
	QPopupMenu	*m_pWindowsMenu;
	QPopupMenu	*m_pSamplingMenu;
	QPopupMenu	*m_pToolsMenu;
	QPopupMenu	*m_pFileMenu;
	QPopupMenu	*m_pHelpMenu;
	int		m_SamplingMenuId[MAX_MENU_NUM];
	QToolButton * m_SamplingToolId[MAX_MENU_NUM];
	QToolButton * m_ToolsToolId[TM_LAST];
	QToolButton * m_SaveButt;
	int		m_ToolsMenuId[TM_LAST];
	QStringList	m_processFilter;
	QString		m_importErrorMsg;
};

#endif
