//$Id: atunefile.cpp,v 1.3 2005/02/14 16:52:29 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2008 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "stdafx.h"
#include <assert.h>
#include <errno.h>

#include "application.h"
#include "atunefile.h"
#include "auxil.h"
#include "xp.h"
#include "EventMaskEncoding.h"

ATuneFile::ATuneFile( QString FileName, bool create )
{
	Q_UNUSED (create);
	QString XpPath = FileName;

	m_bOpened    = FALSE;
	m_pFile  = NULL;
	m_szFileDir  = "";
	m_szFileName = "";
	m_szFilePath = "";

	m_pFile = new QFile( XpPath );

	m_szFilePath = XpPath;
	m_szFileName = XpPath.section ('/', -1);
	m_szFileDir = XpPath.section ('/', 0, -2, QString::SectionIncludeTrailingSep);
	m_applyProcFilter = false;
}


ATuneFile::~ATuneFile()
{
	if( m_pFile != NULL ) {
		delete m_pFile;
		m_pFile = NULL;
	}
}


/* static */ bool ATuneFile::create (const QString fileName)
{
	bool ret = false;

	QString tempFileName = fileName;

	//Replaces \ with / in directories
	tempFileName.replace ('\\', '/');

	//recursively create directories, if needed
	int sections = tempFileName.contains ("/");
	for (int sects = 1; sects < sections; sects ++) {
		QDir    dir;
		dir.setPath( tempFileName.section ("/", 0, sects) );
		if( !dir.exists() ) {
			dir.mkdir (dir.path());
		}
	}

	//create default project
	ATuneFile * pTemp = new ATuneFile (fileName, TRUE );
	if(!pTemp) return ret;

	ret = pTemp->saveAtuneFile();
	delete pTemp;
	pTemp = NULL;
	return ret;	
}//CawFile::create


QCString ATuneFile::getPath()
{
	return m_szFilePath;
}


QCString ATuneFile::getName()
{
	return m_szFileName;
}


QString ATuneFile::getDir()
{
	return m_szFileDir;
}


// reads the General options located in the .caw file
// The General section has the [GENERAL] Key before it
//
void ATuneFile::readGeneralSection( QTextStream *stream )
{
	QString line;
	QString parm;
	QString val;

	line = stream->readLine();

	while( line.contains ("=") > 0 ) {
		parm  = line.section ('=', 0, 0 );
		val = line.section ('=', 1 );

		if (helpReadRunOptions (parm, val, &m_runOptions)) { 
			//intentionally empty, the handling is done in the function
		} else if (parm == "LastProfile") {
			m_lastProfileCfg = val;
		} else if (parm ==("LaunchHistory")) {
			m_launchHistory += val;
		} else if (parm ==("WorkDirHistory")) {
			m_workDirHistory += val;
		} else if (parm ==("ProcessFilter")) {
			m_processFilter += val;
		} else if (parm ==("ApplyProcFilter")) {
			m_applyProcFilter = val.toUInt();
		}

		line = stream->readLine();
	}
}


//checks whether the file is a valid file
int ATuneFile::needsConversion (QString first_line)
{
	if (first_line.startsWith ("FileVersion")) 
	{
		m_file_version = first_line.section ('=', 1, 1).toInt();
		return m_file_version;
	}
	return 1;
}

bool ATuneFile::openAtuneFile()
{
	// Opens an existing file or creates a new one
	m_bOpened = m_pFile->open( IO_ReadWrite );

	if( !m_bOpened ) {
		if (NULL != m_pFile) {
			delete m_pFile;
			m_pFile = NULL;
		}
		return FALSE;
	}

	QTextStream *pStream = new QTextStream( m_pFile );
	QCString line;

	if (!pStream->eof()) {
		line = pStream->readLine();
		/*
		if (needsConversion (line)) {
		// ask to convert
		int answer = QMessageBox::question (NULL, "CodeAnalyst",
		"The CodeAnalyst project needs to be updated, may I update it?",
		QMessageBox::Yes, QMessageBox::No);
		//If okay,
		if (QMessageBox::Yes == answer) {
		delete pStream;
		m_pFile->close();
		delete m_pFile;

		//convert
		convertFromVer1 ();

		//re-open
		m_pFile = new QFile (m_szFilePath);

		if( !m_pFile->open( IO_ReadOnly ) ) {
		delete m_pFile;
		return FALSE;
		}

		pStream = new QTextStream( m_pFile );
		if (NULL == pStream) {
		delete m_pFile;
		QMessageBox::critical (NULL, NOMEM_HEADER, NOMEM_MSG);
		return false;
		}
		}
		else {
		//else abort
		delete pStream;
		m_pFile->close();
		delete m_pFile;
		m_pFile = NULL;

		return false;
		}
		}
		//re-set the file pointer
		m_pFile->at (0);
		*/

		int convert = needsConversion (line);
		//No conversion necessary from version 2
		int answer = QMessageBox::Yes;
		if (convert != PROJECT_INT_FILE_VERSION)
		{
			// ask to convert
			answer = QMessageBox::question (NULL, "CodeAnalyst",
				"The CodeAnalyst project needs to be updated, may I update it?",
				QMessageBox::Yes, QMessageBox::No);
		}
		switch (convert)
		{
		case 1:
			{
				//If okay,
				if (QMessageBox::Yes == answer)
				{
					delete pStream;
					m_pFile->close();
					delete m_pFile;

					//convert
					convertFromVer1 ();

					//re-open
					m_pFile = new QFile (m_szFilePath);
					RETURN_FALSE_IF_NULL (m_pFile, NULL);

					if( !m_pFile->open( IO_ReadOnly ) )
					{
						delete m_pFile;
						return FALSE;
					}

					pStream = new QTextStream( m_pFile );
					if (NULL == m_pFile) 
					{
						delete m_pFile;
						QMessageBox::critical (NULL, NOMEM_HEADER, NOMEM_MSG);
						return false;
					}
					pStream->setEncoding (QTextStream::UnicodeUTF8);

				} else {
					//else abort
					delete pStream;
					m_pFile->close();
					delete m_pFile;

					return false;
				}
				break;
			}
		case 2:
			if (QMessageBox::Yes != answer)
			{
				//abort
				delete pStream;
				m_pFile->close();
				delete m_pFile;

				return false;
			}
			//otherwise, fall through
		case PROJECT_INT_FILE_VERSION:
		default:
			break;
		}
		//re-set the file pointer
		m_pFile->at (0);
	}

	while( !pStream->eof() ) {
		line = pStream->readLine();

		if( line.find( "[GENERAL]" ) != -1 ) {
			readGeneralSection( pStream );
		}

		else if( line.find( "[TBP]") != -1 ) {
			readTbpSection( pStream );
		}

		else if( line.find("[EBP]") != -1 ) {
			readEventSection( pStream );
		}

		else if( line.find("[IBS]") != -1 ) {
			readIbsSection( pStream );
		}

	}

	delete pStream;
	m_pFile->close();

	if(m_file_version != PROJECT_INT_FILE_VERSION)
		saveAtuneFile();

	return TRUE;
} // ATuneFile::openAtuneFile()


// closeAtuneFile the file if it has been opened
// delete the file if it has been allocated
//
void ATuneFile::closeAtuneFile()
{
	m_bOpened = FALSE;
}


//This helper function just helps me from copy/pasting this common bit
void ATuneFile::helpWriteRunOptions (QTextStream *pStream,
									 RUN_OPTIONS *pRun)
{
	*pStream << "Name=" << pRun->sessionName << endl;
	*pStream << "Launch=" << pRun->launch << endl;
	*pStream << "WorkDir=" << pRun->workDir << endl;
	*pStream << "StopOnApp=" << pRun->stopOnAppExit << endl;
	*pStream << "BreakAfter=" << pRun->termAppOnDur << endl;
	*pStream << "Duration=" << pRun->duration << endl;
	*pStream << "StartPaused=" << pRun->startPaused << endl;
	*pStream << "StartDelay=" << pRun->startDelay << endl;
	*pStream << "Affinity=0x" << QString::number(pRun->affinity,16)<< endl;
}


//This avoids copy/pasting this code in multiple sections,
//	returns true if this function consumed the line
bool ATuneFile::helpReadRunOptions (QString parm, 
									QString value,
									RUN_OPTIONS *pRun)
{
	if (NULL == pRun)
		return false;
	bool handled = false;
	if( parm == "Name" ) {
		pRun->sessionName = value;
		handled = true;
	} else if( parm == "Launch" )
	{
		pRun->launch = value;
		handled = true;
	} else if( parm == "WorkDir" )
	{
		pRun->workDir = value;
		handled = true;
	} else if( parm == "StopOnApp" )
	{
		pRun->stopOnAppExit = value.toUInt();
		handled = true;
	} else if( parm == "BreakAfter" )
	{
		pRun->termAppOnDur = value.toUInt();
		handled = true;
	} else if( parm == "Duration" )
	{
		pRun->duration = value.toInt();
		handled = true;
	} else if (parm == "StartPaused" )
	{
		pRun->startPaused = value.toUInt();
		handled = true;
	} else if (parm == "StartDelay" )
	{
		pRun->startDelay = value.toInt();
		handled = true;
	} else if (parm == "Affinity" ) 
	{
		pRun->affinity = value.toUInt(0,0);
		handled = true;
	}
	return handled;
}												  //ATuneFile::helpReadRunOptions


// saveAtuneFile all the options in the options structure to the file
//
bool ATuneFile::saveAtuneFile()
{
	bool ret = false;
	if( !m_pFile->open(IO_ReadWrite | IO_Truncate | IO_Translate) ) {
		QMessageBox::information (0, "Error", "Failed to create/save CodeAnalyst Project file." );

		return ret;
	}

	QTextStream stream( m_pFile );

	stream << "FileVersion=" << PROJECT_FILE_VERSION << endl;

	// General Options
	stream << "[GENERAL]"   << endl;
	helpWriteRunOptions (&stream, &m_runOptions);
	stream << "LastProfile=" << m_lastProfileCfg << endl;
	//handle QStringLists
	stream << "LaunchHistory=" << m_launchHistory.join ("\nLaunchHistory=") << endl;
	stream << "WorkDirHistory=" << m_workDirHistory.join ("\nWorkDirHistory=") << endl;
	stream << "ApplyProcFilter=" << m_applyProcFilter << endl;
	stream << "ProcessFilter=" << m_processFilter.join ("\nProcessFilter=") << endl;

	stream << endl;

	// List of TBP Sessions
	writeTbpSection (&stream);
	stream << endl;

	// List of EBP Sessions
	writeEventSection (&stream);
	stream << endl;

	// List of IBS Sessions
	writeIbsSection (&stream);
	stream << endl;

	m_pFile->close();
	
	ret = true;
	return ret;
}


//If there already exists "SessionName" for the trigger, this will try
//	"SessionName 1", "SessionName 2", etc.
QString ATuneFile::getNextSessionName (TRIGGER trigger)
{
	QString result = m_runOptions.sessionName;
	int additional = 1;
	char tmp[10] = "";

	//since ibs sessions also use .ebp extensions, we can't have overlap or the 
	//session gets overwritten
	QString ext;
	switch (trigger)
	{
	case TIMER_TRIGGER:
		ext = TBS_FILE_EXT;
		break;
	case EVENT_TRIGGER:
	case IBS_TRIGGER:
		ext = EBS_FILE_EXT;
		break;
	default:
		break;
	};
	QString existTest = m_szFileDir + result + ext + DIR_FILE_EXT;

	while ((m_sessions[trigger].contains (result))
		|| ((IBS_TRIGGER == trigger) && (m_sessions[EVENT_TRIGGER].contains (result)))
		|| ((EVENT_TRIGGER == trigger) && (m_sessions[IBS_TRIGGER].contains (result)))
		|| QFile::exists (existTest))
	{
		sprintf(tmp,"%02d",additional++);
		result = m_runOptions.sessionName + " " + QString(tmp);
		existTest = m_szFileDir + result + ext + DIR_FILE_EXT;
	}
	return result;
}


bool ATuneFile::IsSessionNameTaken (TRIGGER trigger, QString SessionName)
{
	if (trigger >= NO_TRIGGER) {
		return true;
	}
	else {
		return m_sessions[trigger].contains (SessionName);
	}
}


//This adds an existing profile to the project - note that the ATuneFile will
//	take care of deleting the allocated memory
void ATuneFile::addSession (SESSION_OPTIONS *const pSession)
{
	if (NULL == pSession)
		return;
	m_sessions[pSession->Trigger].insert (pSession->sessionName, pSession);
}


void ATuneFile::deleteSession (TRIGGER trigger, QString sessionName)
{
	SESSION_OPTIONS *pSession = m_sessions[trigger][sessionName];
	QString filename = pSession->SessionFile;
	m_sessions[trigger].remove (sessionName);

	QString dirName = m_szFileDir + filename + DIR_FILE_EXT;
	QDir dir (dirName);
	if (dir.exists()) {
		QDir dir = QDir (dirName);
		ApplicationWindow::folderRemove(dir);
	}

	saveAtuneFile();
}//ATuneFile::DeleteSession


bool ATuneFile::getSessionCfg (TRIGGER trigger, QString name,
							   SESSION_OPTIONS *pSession)
{
	if (NULL == pSession)
		return false;
	if (m_sessions[trigger].contains (name)) {
		*pSession = * (m_sessions[trigger] [name]);
		return true;
	}
	else {
		return false;
	}
}


bool ATuneFile::getSessionDetails (QString name, SESSION_OPTIONS *pSession)
{
	if (NULL == pSession)
		return false;

	TRIGGER trigger = pSession->Trigger;
	if (m_sessions[trigger].contains (name)) {
		switch (trigger) {
			case TIMER_TRIGGER:
				{
					TBP_OPTIONS *pTbp = static_cast<TBP_OPTIONS *>(pSession);
					TBP_OPTIONS *pOther = static_cast<TBP_OPTIONS *>(m_sessions[trigger] [name]);
					*pTbp = *pOther;
				}
				break;

			case IBS_TRIGGER:
				{
					IBS_OPTIONS *pIbs = static_cast<IBS_OPTIONS *>(pSession);
					IBS_OPTIONS *pOther = static_cast<IBS_OPTIONS *>(m_sessions[trigger] [name]);
					pIbs->set (pOther);
				}
				break;

			default:
			case EVENT_TRIGGER:
				{
					EBP_OPTIONS *pEbp = static_cast<EBP_OPTIONS *>(pSession);
					EBP_OPTIONS *pOther = static_cast<EBP_OPTIONS *>(m_sessions[trigger] [name]);
					pEbp->set (pOther);
				}
				break;
		}
		return true;
	}
	else {
		return false;
	}
}


//This will alter the session name and file name in the internal lists, in
//  addition to renaming all applicable files (session data, ti, prd, dir)
void ATuneFile::renameSession (TRIGGER trigger, QString oldname, QString newname)
{
	QString newFile;
	QString oldFile;
	QString ext;

	newFile = m_szFileDir + newname;
	oldFile = m_szFileDir + oldname;
	SESSION_OPTIONS *pSession = m_sessions[trigger][oldname];
	m_sessions[trigger].remove (oldname);
	switch (trigger) {
		default: //should never happen
		case TIMER_TRIGGER:
			ext = TBS_FILE_EXT;
			break;
		case IBS_TRIGGER:
		case EVENT_TRIGGER:
			ext = EBS_FILE_EXT;
			break;
	}
	newFile += ext;
	oldFile += ext;
	pSession->SessionFile = newname + ext;
	pSession->sessionName = newname;
	m_sessions[trigger].insert (newname, pSession);

	//rename the files
	QDir dir;
	//first rename the directory
	newFile += DIR_FILE_EXT;
	oldFile += DIR_FILE_EXT;
	dir.rename (oldFile, newFile);
	//now all files in dir
	oldFile = newFile + "/" + oldname + ext;
	newFile += "/" + newname + ext;
	dir.rename (oldFile, newFile);
	dir.rename ((oldFile + PRD_FILE_EXT), (newFile + PRD_FILE_EXT));
	dir.rename ((oldFile + TI_FILE_EXT), (newFile + TI_FILE_EXT));
}//ATuneFile::RenameSession


//Since the indexes of the maps are names, we just get the list of indexes
QStringList ATuneFile::getSessionList (TRIGGER trigger)
{
	QStringList sessionList;
	SessionMap::Iterator it = m_sessions[trigger].begin();
	for( ; it != m_sessions[trigger].end(); ++it ) {
		sessionList += it.key();
	}
	return sessionList;
}


void ATuneFile::writeTbpSection (QTextStream *pStream )
{
	*pStream << "[TBP]" << endl;
	// Individual TBP Session options
	RUN_OPTIONS *pRun;
	SessionMap::iterator it = m_sessions[TIMER_TRIGGER].begin();
	for( ; it != m_sessions[TIMER_TRIGGER].end(); ++it ) {
		TBP_OPTIONS *pTbsSession = static_cast <TBP_OPTIONS *>(it.data());
		pRun = static_cast<RUN_OPTIONS *> (pTbsSession);

		//each session separated by this blank line
		*pStream << endl;
		helpWriteRunOptions (pStream, pRun);
		*pStream << "ProfileFile=" << pTbsSession->SessionFile << endl;
		*pStream << "Cpus=" << pTbsSession->Cpus << endl;
		*pStream << "Interval=" << pTbsSession->msInterval << endl;
	}
	*pStream << endl << "[End]" << endl;
}


//Reads all the timer-based profile sessions
void ATuneFile::readTbpSection( QTextStream *pStream )
{
	QString line = pStream->readLine();

	//This is allocated on the first blank line
	TBP_OPTIONS *pTbp = NULL;

	while (!line.startsWith ("[End]")) {
		if (pStream->atEnd ()) {
			break;
		}

		QString parm = line.section ('=', 0, 0);
		QString value = line.section ('=', 1);
		if (helpReadRunOptions (parm, value, static_cast<RUN_OPTIONS *>(pTbp))) { 
			//This case is intentionally empty, the handling is done in
			//	helpReadRunOptions, we need this to set up the other elses
		} else if (parm.startsWith ("ProfileFile")) {
			pTbp->SessionFile = value;
		} else if (parm.startsWith ("Cpus")) {
			pTbp->Cpus = value.toInt();
		} else if (parm.startsWith ("Interval")) {
			pTbp->msInterval = value.toFloat();
		} else if (line.isEmpty()) {
			//write info to map
			if (NULL != pTbp) {
				m_sessions[TIMER_TRIGGER].insert (pTbp->sessionName,
					static_cast<SESSION_OPTIONS *>(pTbp));
			}

			// create next, these will be deleted when the object closes
			pTbp = new TBP_OPTIONS;
			RETURN_IF_NULL (pTbp, NULL);
		}
		line = pStream->readLine();
	}
	if (NULL != pTbp) {
		if (!pTbp->SessionFile.isEmpty()) {
			//write info to map
			m_sessions[TIMER_TRIGGER].insert (pTbp->sessionName,
				static_cast<SESSION_OPTIONS *>(pTbp));
		} else
			delete pTbp;
	}
}												  //ATuneFile::ReadTbpSection


void ATuneFile::writeEventSection (QTextStream *pStream )
{
	*pStream << "[EBP]" << endl;

	// Individual Ebp Session options
	RUN_OPTIONS *pRun;
	SessionMap::iterator it = m_sessions[EVENT_TRIGGER].begin();
	for( ; it != m_sessions[EVENT_TRIGGER].end(); ++it ) {
		EBP_OPTIONS *pEventSession = static_cast <EBP_OPTIONS *>(it.data());
		pRun = static_cast<RUN_OPTIONS *> (pEventSession);

		//each session separated by this blank line
		*pStream << endl;
		helpWriteRunOptions (pStream, pRun);
		*pStream << "ProfileFile=" << pEventSession->SessionFile << endl;
		*pStream << "Cpus=" << pEventSession->Cpus << endl;
		*pStream << "MultiplexInterval=" << QString::number(pEventSession->msMpxInterval) << endl;
		//write event cfg
		*pStream << "EventGroupCount=" << pEventSession->countEventGroups << endl;
		for (int i = 0; i < pEventSession->countEventGroups; i++) {
			for (int j = 0; j < pEventSession->pEvents[i].numberOfValidEvents; j++) {
				//Note that this needs to match the EventGroup struct
				*pStream << "OneEvent=" << i << "," << j << ",";
				*pStream << pEventSession->pEvents[i].eventSelect[j] << ",";
				*pStream << pEventSession->pEvents[i].eventUnitMask[j] << ",";
				*pStream << pEventSession->pEvents[i].eventOS[j] << ",";
				*pStream << pEventSession->pEvents[i].eventUser[j] << ",";
				*pStream << QString::number (pEventSession->pEvents[i].eventCount[j]) << ",";
				*pStream << pEventSession->pEvents[i].edgeDetect[j] << endl;
			}
		}
	}
	*pStream << endl << "[End]" << endl;
}//ATuneFile::writeEventSection


void ATuneFile::readEventSection( QTextStream *pStream )
{
	QString line = pStream->readLine();
	EBP_OPTIONS *pEventOpts = NULL;

	while (!line.startsWith ("[End]")) {
		QString parm = line.section ('=', 0, 0);
		QString value = line.section ('=', 1);
		//note that the first line should be blank, so pEventOpts will be valid
		if (helpReadRunOptions (parm, value, static_cast<RUN_OPTIONS *>(pEventOpts))) {
			//This case is intentionally empty, the handling is done in
			//	helpReadRunOptions, we need this to set up the other elses
		} else if (parm.startsWith ("ProfileFile")) {
			pEventOpts->SessionFile = value;
		} else if (parm.startsWith ("Cpus")) {
			pEventOpts->Cpus = value.toInt();
		} else if (parm.startsWith ("MultiplexInterval")) {
			pEventOpts->msMpxInterval = value.toULongLong();
		} else if (parm.startsWith ("EventGroupCount")) {
			//Since this is always written before the events listing,
			//	there will always be allocated memory before being read
			pEventOpts->countEventGroups = value.toInt();
			pEventOpts->pEvents = new EventGroup[pEventOpts->countEventGroups];
			RETURN_IF_NULL (pEventOpts->pEvents, NULL);
			memset (pEventOpts->pEvents, 0, (sizeof (EventGroup) * pEventOpts->countEventGroups));
		} else if (parm.startsWith ("OneEvent")) {
			//OneEvent=group,index,ev select,unitmask,eventOs,eventUsr,eventCount
			//The section depends on the order it was written
			int group = value.section (",", 0, 0).toInt();
			int index = value.section (",", 1, 1).toInt();
			pEventOpts->pEvents[group].numberOfValidEvents++;

			//Note that this should match the EventGroup struct
			pEventOpts->pEvents[group].eventSelect[index] =
				value.section (",", 2, 2).toUInt ();
			pEventOpts->pEvents[group].eventUnitMask[index] =
				value.section (",", 3, 3).toUInt ();
			pEventOpts->pEvents[group].eventOS[index] =
				value.section (",", 4, 4).toUInt ();
			pEventOpts->pEvents[group].eventUser[index] =
				value.section (",", 5, 5).toUInt ();
			pEventOpts->pEvents[group].eventCount[index] =
				value.section (",", 6, 6).toULongLong ();
			pEventOpts->pEvents[group].edgeDetect[index] =
				value.section (",", 7, 7).toUInt ();
		} else if (line.isEmpty())
		{
			//write info to map
			if (NULL != pEventOpts) {
				m_sessions[EVENT_TRIGGER].insert (pEventOpts->sessionName,
					static_cast<SESSION_OPTIONS *>(pEventOpts));
			}

			// create next, these will be deleted when the object closes
			pEventOpts = new EBP_OPTIONS;
			RETURN_IF_NULL (pEventOpts, NULL);
		}
		line = pStream->readLine();
	}
	if (NULL != pEventOpts) {
		if (!pEventOpts->SessionFile.isEmpty()) {
			//write info to map
			m_sessions[EVENT_TRIGGER].insert (pEventOpts->sessionName,
				static_cast<SESSION_OPTIONS *>(pEventOpts));
		}
		else {
			delete pEventOpts;
		}
	}
}												  //ATuneFile::readEventSection


void ATuneFile::getRunOptions (RUN_OPTIONS * pRunOptions)
{
	*pRunOptions = m_runOptions;
}


void ATuneFile::setRunOptions (const RUN_OPTIONS runOptions)
{
	m_runOptions = runOptions;
	//update launch history
	if (!m_runOptions.launch.isEmpty()) {
		if (m_launchHistory.grep (m_runOptions.launch).empty()) {
			m_launchHistory.push_front (m_runOptions.launch);
			if (m_launchHistory.size () > MAX_HISTORY) {
				m_launchHistory.pop_back();
			}
		}
		else {
			m_launchHistory.remove (m_runOptions.launch);
			m_launchHistory.push_front (m_runOptions.launch);
		}
	}
	//update workdir history
	if (!m_runOptions.workDir.isEmpty()) {
		if (m_workDirHistory.grep (m_runOptions.workDir).empty()) {
			m_workDirHistory.push_front (m_runOptions.workDir);
			if (m_workDirHistory.size () > MAX_HISTORY) {
				m_workDirHistory.pop_back();
			}
		}
		else {
			m_workDirHistory.remove (m_runOptions.workDir);
			m_workDirHistory.push_front (m_runOptions.workDir);
		}
	}
}												  //ATuneFile::setRunOptions


QStringList ATuneFile::getLaunchHistory ()
{
	return m_launchHistory;
}


QStringList ATuneFile::getWorkDirHistory ()
{
	return m_workDirHistory;
}

QStringList ATuneFile::getProcessFilter()
{
	return m_processFilter;
}

void ATuneFile::setProcessFilter(QStringList filterList)
{
	m_processFilter = filterList;		
}

bool ATuneFile::isApplyProcessFilter()
{
	return m_applyProcFilter;
}

void ATuneFile::setApplyProcessFilter(bool t)
{
	m_applyProcFilter = t;
}

QString ATuneFile::getLastProfile ()
{
	QString ret = m_lastProfileCfg;
	if (ret.isEmpty())
		ret = CURRENT_TIME;
	return ret;
}


//update last profile
void ATuneFile::setLastProfile (QString profileName)
{
	m_lastProfileCfg = profileName;
}


void ATuneFile::convertFromVer1 ()
{
	//read from old
	QFile * pOld = new QFile (m_szFilePath);
	RETURN_IF_NULL (pOld, NULL);
	pOld->open( IO_ReadOnly );
	QTextStream *pOldStream = new QTextStream( pOld );
	RETURN_IF_NULL (pOldStream, NULL);

	while( !pOldStream->eof() ) {
		QString line = pOldStream->readLine();

		if( line.startsWith( "[GENERAL]" )) {
			readConvertGeneral( pOldStream );
		} else if( line.startsWith( "[TBP]"))
		{
			readConvertTbs( pOldStream );
		} else if (line.contains (TBS_FILE_EXT))
		{
			//6 = ".tbp]"
			readConvertTbs (pOldStream, line.mid (1, (line.length () - 6)));
		} else if( line.startsWith ("[EBP]"))
		{
			readConvertEbp( pOldStream );
		} else if (line.contains (EBS_FILE_EXT))
		{
			//6 = ".ebp]"
			readConvertEbp (pOldStream, line.mid (1, (line.length () - 6)));
		}
	}

	//Close file, so we can overwrite it
	pOld->close();
	delete pOldStream;
	delete pOld;

	//Write saved info in current format
	saveAtuneFile();
}												  //ATuneFile::convertFromVer1


void ATuneFile::readConvertGeneral (QTextStream *pStream)
{
	QString line = pStream->readLine();
	if (NULL == pStream)
		return;

	while ((!line.isEmpty()) && (!pStream->eof())) {
		QString parm = line.section ('=', 0, 0);
		QString value = line.section ('=', 1);
		if( parm.startsWith ("WorkDir" )) {
			m_runOptions.workDir = value;
		} else if( parm.startsWith ("Name" ))
		{
			m_runOptions.sessionName = value;
		} else if( parm.startsWith ("Duration" ))
		{
			m_runOptions.duration = value.toUInt();
		} else if( parm.startsWith ("Program" ))
		{
			m_runOptions.launch = value;
		} else if( parm.startsWith ("StopOnApp" ))
		{
			m_runOptions.stopOnAppExit = value.toUInt();
		} else if (parm.startsWith ("StartPaused" ))
		{
			m_runOptions.startPaused = value.toUInt();
		} else if( parm.startsWith ("BreakAfter" ))
		{
			m_runOptions.termAppOnDur = value.toUInt();
		} else if( parm.startsWith ("Delay" ))
		{
			m_runOptions.startDelay = value.toUInt();
		} else
		{
			//skipped, no longer userful
		}

		line = pStream->readLine();
	}
}												  //ATuneFile::readConvertGeneral


void ATuneFile::readConvertTbs (QTextStream *pStream, QString sessionName)
{
	QString line = pStream->readLine();
	if (NULL == pStream)
		return;

	while ((!line.isEmpty()) && (!pStream->eof())) {
		QString parm = line.section ('=', 0, 0);
		QString value = line.section ('=', 1);
		if (sessionName.isEmpty()) {
			//first one just reads all session names into it.
			TBP_OPTIONS * pTbp = new TBP_OPTIONS;
			RETURN_IF_NULL (pTbp, NULL);

			pTbp->sessionName = value.remove (TBS_FILE_EXT);

			m_sessions[TIMER_TRIGGER].insert (pTbp->sessionName,
				static_cast<SESSION_OPTIONS *>(pTbp));
		}
		else {
			//fill in details of each session previously read and added
			TBP_OPTIONS *pTbp = static_cast<TBP_OPTIONS *>(m_sessions[TIMER_TRIGGER][sessionName]);

			pTbp->SessionFile = sessionName + TBS_FILE_EXT;
			QString oldFile = m_szFileDir + pTbp->SessionFile;
			QString newFile = m_szFileDir + pTbp->SessionFile + DIR_FILE_EXT
				+ "/" + pTbp->SessionFile;
			QDir moveDir;
			moveDir.rename (oldFile, newFile, true);
			//prd
			oldFile = m_szFileDir + pTbp->SessionFile + PRD_FILE_EXT;
			newFile = m_szFileDir + pTbp->SessionFile + DIR_FILE_EXT
				+ "/" + pTbp->SessionFile + PRD_FILE_EXT;
			moveDir.rename (oldFile, newFile, true);
			//ti
			oldFile = m_szFileDir + pTbp->SessionFile + TI_FILE_EXT;
			newFile = m_szFileDir + pTbp->SessionFile + DIR_FILE_EXT
				+ "/" + pTbp->SessionFile + TI_FILE_EXT;
			moveDir.rename (oldFile, newFile, true);
			//Skip session name

			line = pStream->readLine();
			pTbp->msInterval = line.section ('=', 1).toFloat();

			line = pStream->readLine();
			pTbp->duration = line.section ('=', 1).toUInt();

			line = pStream->readLine();
			//Skip buffer size

			line = pStream->readLine();
			pTbp->Cpus = line.section ('=', 1).toUInt();

			line = pStream->readLine();
			pTbp->launch  = line.section ('=', 1);

			line = pStream->readLine();
			pTbp->startDelay = line.section ('=', 1).toUInt();

			line = pStream->readLine();
			pTbp->stopOnAppExit = line.section ('=', 1).toUInt();

			line = pStream->readLine();
			pTbp->termAppOnDur = line.section ('=', 1).toUInt();

			//added working directory to options 5/22/03
			line = pStream->readLine();
			if (!line.isEmpty ()) {
				pTbp->workDir = line.section ('=', 1);
				line = pStream->readLine();
				if (!line.isEmpty()) {
					//Skip jitInfo

					//added StartPaused 1/10/05
					line = pStream->readLine();
					if (!line.isEmpty()) {
						pTbp->startPaused = line.section ('=', 1).toUInt();
					}
					else {
						break;
					}
				}
				else {
					break;
				}
			}
			else {
				break;
			}
		}										  //If we're reading list of session names
		line = pStream->readLine();
	}											  //while there are names left
}												  //ATuneFile::readConvertTbs


void ATuneFile::readConvertEbp (QTextStream *pStream, QString sessionName)
{
	if ((NULL == pStream) || (pStream->eof()))
		return;
	QString line = pStream->readLine();

	while ((!line.isEmpty()) && (!pStream->eof())) {
		QString value = line.section ('=', 1);
		if (sessionName.isEmpty()) {
			//first one just reads all session names into it.
			EBP_OPTIONS * pSession = new EBP_OPTIONS;
			RETURN_IF_NULL (pSession, NULL);

			pSession->sessionName = value.remove (EBS_FILE_EXT);

			m_sessions[EVENT_TRIGGER].insert (pSession->sessionName,
				static_cast<SESSION_OPTIONS *>(pSession));
		}
		else {
			//fill in details of each session previously read and added
			EBP_OPTIONS *pSession = static_cast<EBP_OPTIONS *>(m_sessions[EVENT_TRIGGER][sessionName]);

			pSession->SessionFile = sessionName + EBS_FILE_EXT;
			QString oldFile = m_szFileDir + pSession->SessionFile;
			QString newFile = m_szFileDir + pSession->SessionFile + DIR_FILE_EXT
				+ "/" + pSession->SessionFile;
			QDir moveDir;
			moveDir.rename (oldFile, newFile, true);

			//Skip session name

			//Read program path
			line = pStream->readLine();
			pSession->launch  = line.section ('=', 1);

			//Delay
			line = pStream->readLine();
			pSession->startDelay = line.section ('=', 1).toUInt();

			//Duration
			line = pStream->readLine();
			pSession->duration = line.section ('=', 1).toUInt();

			line = pStream->readLine();
			if (line.startsWith ("CPUS")) {
				pSession->Cpus = line.section ("=", 1).toUInt();
				line = pStream->readLine();
			}
			else {
				pSession->Cpus = XpGetNumCpus();
			}
			pSession->stopOnAppExit = line.section ('=', 1).toUInt();

			line = pStream->readLine();
			pSession->termAppOnDur = line.section ('=', 1).toUInt();

			//Old files only could have 1 group
			pSession->countEventGroups = 1;
			pSession->pEvents = new EventGroup[1];
			pSession->msMpxInterval = 0;
			pSession->pEvents[0].numberOfValidEvents = 0;

			//4 = max # of events in a group at the time
			for (int i = 0; i < 4; i++) {
				line = pStream->readLine();
				line = line.section ('=', 1);
				//skip combo index, source mask
				pSession->pEvents[0].eventSelect[i] = line.section (',', 2, 2).toUShort();
				pSession->pEvents[0].eventUnitMask[i] = line.section (',', 3, 3).toUShort();
				pSession->pEvents[0].eventCount[i] = line.section (',', 4, 4).toULongLong();
				pSession->pEvents[0].eventOS[i] = line.section (',', 5, 5).toUShort();
				pSession->pEvents[0].eventUser[i] = line.section (',', 6, 6).toUShort();
				pSession->pEvents[0].edgeDetect[i] = false;
				if ((pSession->pEvents[0].eventSelect[i] != 0) ||
					(pSession->pEvents[0].eventOS[i] != 0) ||
					(pSession->pEvents[0].eventUser[i] != 0)) {
						pSession->pEvents[0].numberOfValidEvents++;
				}
			}

			//added working directory to options 5/22/03
			line = pStream->readLine();
			if (!line.isEmpty()) {
				pSession->workDir = line.section ('=', 1);
				line = pStream->readLine();
				if (!line.isEmpty()) {
					//skip jit info
					//added StartPaused 1/10/05
					line = pStream->readLine();
					if (!line.isEmpty()) {
						pSession->startPaused = line.section ('=', 1).toUInt();
					}
					else {
						break;
					}
				}
				else {
					break;
				}
			}
			else {
				break;
			}
		}//If we're reading list of session names
		line = pStream->readLine();
	}//while there are names left
}//ATuneFile::readConvertEbp

void ATuneFile::writeIbsSection (QTextStream *pStream)
{
	*pStream << "[IBS]" << endl;
	// Individual IBS Session options
	RUN_OPTIONS *pRun;
	SessionMap::iterator it = m_sessions[IBS_TRIGGER].begin();
	for( ; it != m_sessions[IBS_TRIGGER].end(); ++it )
	{
		IBS_OPTIONS *pIbsSession = static_cast <IBS_OPTIONS *>(it.data());
		pRun = static_cast<RUN_OPTIONS *> (pIbsSession);

		//each session separated by this blank line
		*pStream << endl;
		helpWriteRunOptions (pStream, pRun);
		*pStream << "ProfileFile=" << pIbsSession->SessionFile << endl;
		*pStream << "Cpus=" << pIbsSession->Cpus << endl;
		*pStream << "Fetch=" << pIbsSession->fetchSample << endl;
		*pStream << "OpSample=" << pIbsSession->opSample << endl;
		*pStream << "IntervalFetch=" << pIbsSession->fetchInterval << endl;
		*pStream << "IntervalOp=" << pIbsSession->opInterval << endl;
		*pStream << "OpCycleCount=" << pIbsSession->opCycleCount << endl;
		
		QValueList<unsigned int>::iterator it;
		QValueList<unsigned int>::iterator it_end;
		
		*pStream << "FetchList=";
		it = pIbsSession->fetchList.begin();
		it_end = pIbsSession->fetchList.end();
		while(it != it_end) {
			*pStream << "0x" << QString::number(*it,16);
			it++;
			if (it != it_end) {
				*pStream << ",";
			} else {
				break;
			}
		}
		*pStream << endl;
		
		*pStream << "OpList=";
		it = pIbsSession->opList.begin();
		it_end = pIbsSession->opList.end();
		while(it != it_end) {
			*pStream << "0x" << QString::number(*it,16);
			it++;
			if (it != it_end) {
				*pStream << ",";
			} else {
				break;
			}
		}
		*pStream << endl;
	}
	*pStream << endl << "[End]" << endl;
}


void ATuneFile::readIbsSection (QTextStream *pStream )
{
	QString line = pStream->readLine();

	//This is allocated on the first blank line
	IBS_OPTIONS *pIbs = NULL;

	while (!line.startsWith ("[End]"))
	{
		if (pStream->atEnd ())
		{
			break;
		}

		QString parm = line.section ('=', 0, 0);
		QString value = line.section ('=', 1);
		if (helpReadRunOptions (parm, value, static_cast<RUN_OPTIONS *>(pIbs)))
		{
			//This case is intentionally empty, the handling is done in 
			//	helpReadRunOptions, we need this to set up the other elses
		} else if (!parm.compare("ProfileFile"))
		{
			pIbs->SessionFile = value;
		} else if (!parm.compare("Cpus"))
		{
			pIbs->Cpus = value.toInt();
		} else if (!parm.compare ("Fetch"))
		{
			pIbs->fetchSample = value.toUInt();
		} else if (!parm.compare ("OpSample"))
		{
			pIbs->opSample = value.toUInt();
		} else if (!parm.compare ("IntervalFetch"))
		{
			pIbs->fetchInterval = value.toULong();
		} else if (!parm.compare ("IntervalOp"))
		{
			pIbs->opInterval = value.toULong();
		} else if (!parm.compare ("OpCycleCount")) {
			//If this wasn't written, it's just fine because the 
			//profile was taken before the rev c was available
			pIbs->opCycleCount = (value.toUInt() == 1)?true:false;
		} else if (!parm.compare ("FetchList")) {
			QStringList fetchStrList = QStringList::split(QString(","),value); 
			QStringList::iterator it     = fetchStrList.begin();
			QStringList::iterator it_end = fetchStrList.end();
			for(; it != it_end; it++)
				pIbs->fetchList.append((*it).toUInt(0,16));
		} else if (!parm.compare ("OpList")) {
			QStringList opStrList = QStringList::split(QString(","),value); 
			QStringList::iterator it     = opStrList.begin();
			QStringList::iterator it_end = opStrList.end();
			for(; it != it_end; it++)
				pIbs->opList.append((*it).toUInt(0,16));
		} else if (line.isEmpty())
		{
			//write info to map
			if (NULL != pIbs)
			{
				m_sessions[IBS_TRIGGER].insert (pIbs->sessionName, 
					static_cast<SESSION_OPTIONS *>(pIbs));
			}

			// create next, these will be deleted when the object closes
			pIbs = new IBS_OPTIONS;
			RETURN_IF_NULL (pIbs, NULL);
		}
		line = pStream->readLine();
	}
	if (NULL != pIbs)
	{
		if (!pIbs->SessionFile.isEmpty())
		{
			//write info to map
			m_sessions[IBS_TRIGGER].insert (pIbs->sessionName, 
				static_cast<SESSION_OPTIONS *>(pIbs));
		} else
			delete pIbs;
	}
}

void ATuneFile::setAffinityMask(AffinityMask_t afMask)
{
	m_runOptions.affinity = afMask;
}

AffinityMask_t ATuneFile::getAffinityMask()
{
	return m_runOptions.affinity;
}

bool EBP_OPTIONS::checkForDupEventWithDiffCount()
{
	bool ret = false;
	if(!pEvents || countEventGroups == 0)
		return ret;

	if(pEvents[0].numberOfValidEvents == 1)
		return ret;
	
	int g = 0, i = 0;
	EventMaskMap eventMaskMap;
	for(g = 0; g < countEventGroups ; g++)
	{
		for(i = 0; i < pEvents[g].numberOfValidEvents; i++)
		{
			EventMaskType cur = EncodeEventMask(pEvents[g].eventSelect[i]
							, pEvents[g].eventUnitMask[i]);
			EventMaskMap::iterator it = eventMaskMap.find(cur);	
			if(it != eventMaskMap.end())
			{
				if(it.data() != pEvents[g].eventCount[i])
				{
					ret = true;	
					return ret;	
				}
			}else{
				eventMaskMap.insert(EventMaskMap::value_type(cur,pEvents[g].eventCount[i]));
			}		
		}	
	}
	return ret;	
}
