//$Id: atunefile.h,v 1.4 2006/08/17 14:24:57 ssuthiku Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 - 2007 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/
#ifndef _ATUNEFILE_H
#define _ATUNEFILE_H

#include "stdafx.h"
#include "tbsStruct.h"
#include "DCConfigAPI.h"

#define GENERAL_SESSION	0
#define MAX_SESSIONS	100
#define MAX_HISTORY 10

// forward declarations
class CWorkspaceOptions;
struct EBS_SESSION_OPTIONS;
struct TBS_SESSION_OPTIONS;

// typedefs
typedef DWORD	AffinityMask_t;
const QString PROJECT_FILE_VERSION = "5";
const int PROJECT_INT_FILE_VERSION =  5 ;
const QString CURRENT_TIME = "Current time-based profile";

////////////////////////////////////////////////////////////////////////////
// Structures
//
struct RUN_OPTIONS
{
	QString	sessionName;
	QString launch;
	QString workDir;
	bool stopOnAppExit;
	bool termAppOnDur;
	bool startPaused;
	int duration;
	int startDelay;
	AffinityMask_t affinity;   

	RUN_OPTIONS ()
	{
		sessionName = "Session";
		stopOnAppExit = false;
		termAppOnDur = false;
		startPaused = false;
		duration = -1;		// To mark N/A
		startDelay = -1;	// To mark N/A
		affinity = 0; 		// To mark N/A
	};
};


struct SESSION_OPTIONS : public RUN_OPTIONS
{
	//Note that the SessionFile is not the path. 
	QString	SessionFile;  
	TRIGGER	Trigger;
	int Cpus;

	SESSION_OPTIONS() : RUN_OPTIONS()
	{
		Trigger = NO_TRIGGER;
		Cpus = 1;
	};
};

typedef QMap<QString, SESSION_OPTIONS *> SessionMap;

///////////////////////////////////////////////////////////////////////////////////
//
struct TBP_OPTIONS : public SESSION_OPTIONS
{
	float		msInterval;

	TBP_OPTIONS()  : SESSION_OPTIONS()
	{ 
		msInterval	= 10.0;
		Trigger = TIMER_TRIGGER;
	}

	TBP_OPTIONS operator=(const SESSION_OPTIONS opt )
	{
		sessionName 	= opt.sessionName;
		launch		= opt.launch;
		workDir		= opt.workDir;
		stopOnAppExit	= opt.stopOnAppExit;
		termAppOnDur	= opt.termAppOnDur;
		startPaused	= opt.startPaused;
		duration	= opt.duration;
		startDelay	= opt.startDelay;
		affinity 	= opt.affinity;
		return *this;
	};

	TBP_OPTIONS set(TBP_OPTIONS *pOther)
	{
		sessionName 	= pOther->sessionName;
		launch		= pOther->launch;
		workDir		= pOther->workDir;
		stopOnAppExit	= pOther->stopOnAppExit;
		termAppOnDur	= pOther->termAppOnDur;
		startPaused	= pOther->startPaused;
		duration	= pOther->duration;
		startDelay	= pOther->startDelay;
		affinity 	= pOther->affinity;
		return *this;
	};

};

class EBP_OPTIONS : public SESSION_OPTIONS
{
public:
	EventGroup *pEvents; //from DCConfigAPI.h
	int countEventGroups;
	unsigned int msMpxInterval;	

	EBP_OPTIONS ()  : SESSION_OPTIONS()
	{
		pEvents = NULL;
		Trigger = EVENT_TRIGGER;
		countEventGroups = 0;
		msMpxInterval = 0;	// To mark N/A
	};

	~EBP_OPTIONS ()
	{
		if (NULL != pEvents)
		{
			delete [] pEvents;
		}
	};

	void set (EBP_OPTIONS *pOther )
	{
		sessionName 	= pOther->sessionName;
		launch 		= pOther->launch;
		workDir 	= pOther->workDir;
		stopOnAppExit 	= pOther->stopOnAppExit;
		termAppOnDur 	= pOther->termAppOnDur;
		startPaused 	= pOther->startPaused;
		duration 	= pOther->duration;
		startDelay 	= pOther->startDelay;
		affinity 	= pOther->affinity;

		if (NULL != pEvents)
		{
			delete [] pEvents;
		}
		pEvents = new EventGroup [pOther->countEventGroups];
		if (NULL != pEvents)
		{
			int msize = sizeof (EventGroup) * pOther->countEventGroups;
			memcpy (pEvents, pOther->pEvents, msize);
		}
		countEventGroups = pOther->countEventGroups;
		msMpxInterval    = pOther->msMpxInterval;
	};

	EBP_OPTIONS operator=(const SESSION_OPTIONS opt )
	{
		/*
		if (NULL != pEvents) {
		int msize = sizeof (EventGroup) * opt.countEventGroups;
		memcpy (pEvents, opt.pEvents, msize);
		}
		countEventGroups = opt.countEventGroups;
		msMpxInterval = opt.msMpxInterval;
		*/
		sessionName 	= opt.sessionName;
		launch 		= opt.launch;
		workDir 	= opt.workDir;
		stopOnAppExit 	= opt.stopOnAppExit;
		termAppOnDur 	= opt.termAppOnDur;
		startPaused 	= opt.startPaused;
		duration 	= opt.duration;
		startDelay 	= opt.startDelay;
		affinity	= opt.affinity;
		return *this;
	};

	bool checkForDupEventWithDiffCount();
	
};

class IBS_OPTIONS : public SESSION_OPTIONS
{
public:
	bool fetchSample;
	bool opSample;
	bool opCycleCount;
	unsigned long fetchInterval;
	unsigned long opInterval;
	bool dcMissInfoEnabled;
	QValueList<unsigned int> fetchList;
	QValueList<unsigned int> opList;

	IBS_OPTIONS()  : SESSION_OPTIONS()
	{ 
		fetchSample 	= false;
		opSample 	= false;
		opCycleCount 	= true;
		dcMissInfoEnabled = false;
		fetchInterval	= 1000000;
		opInterval 	= 1000000;
		Trigger 	= IBS_TRIGGER;
	};

	IBS_OPTIONS operator=(const SESSION_OPTIONS opt )
	{
		sessionName 	= opt.sessionName;
		launch 		= opt.launch;
		workDir 	= opt.workDir;
		stopOnAppExit 	= opt.stopOnAppExit;
		termAppOnDur 	= opt.termAppOnDur;
		startPaused 	= opt.startPaused;
		duration 	= opt.duration;
		startDelay 	= opt.startDelay;
		affinity	= opt.affinity;

		return *this;
	};

	void set (IBS_OPTIONS *pOther )
	{
		sessionName	= pOther->sessionName;
		launch 		= pOther->launch;
		workDir 	= pOther->workDir;
		stopOnAppExit 	= pOther->stopOnAppExit;
		termAppOnDur 	= pOther->termAppOnDur;
		startPaused 	= pOther->startPaused;
		duration 	= pOther->duration;
		startDelay 	= pOther->startDelay;
		affinity	= pOther->affinity;
		fetchList	= pOther->fetchList;
		opList		= pOther->opList;

		fetchSample 	= pOther->fetchSample;
		opSample 	= pOther->opSample;
		fetchInterval	= pOther->fetchInterval;
		opInterval 	= pOther->opInterval;
		opCycleCount	= pOther->opCycleCount;
		dcMissInfoEnabled = pOther->dcMissInfoEnabled;
		Trigger 	= pOther->Trigger;
	};

};

///////////////////////////////////////////////////////////////////////////////////
//

enum CPU_TYPE {
	CPU_ATHLON = 0,
	CPU_ATHLON_XP,
	CPU_OPTERON,
	CPU_ATHLON_64
};


///////////////////////////////////////////////////////////////////////////////////
//

/////////////////////////////////////////////////////////////////////////
// ATuneFile class
//
class ATuneFile {
public:
	ATuneFile (QString FileName, bool bCreate);
	~ATuneFile ();

public:
	static bool create (const QString fileName);
	void getRunOptions (RUN_OPTIONS * pRunOptions);
	void setRunOptions (const RUN_OPTIONS runOptions);
	QStringList getLaunchHistory ();
	QStringList getWorkDirHistory ();
	QString getLastProfile ();
	void setLastProfile (QString profileName);
	QStringList getProcessFilter();
	void setProcessFilter(QStringList filterList);
	bool isApplyProcessFilter();
	void setApplyProcessFilter(bool t);

	QString getNextSessionName (TRIGGER trigger);
	bool IsSessionNameTaken (TRIGGER trigger, QString SessionName);

	void deleteSession (TRIGGER trigger, QString SessionName);
	void renameSession (TRIGGER trigger,  QString oldname, QString newname );
	void addSession (SESSION_OPTIONS *const pSession);
	bool getSessionCfg (TRIGGER trigger, QString name, 
		SESSION_OPTIONS *pSession);
	bool getSessionDetails (QString name, SESSION_OPTIONS *pSession);
	QStringList getSessionList (const TRIGGER trigger);

	bool openAtuneFile ();
	void closeAtuneFile ();
	bool saveAtuneFile ();

	QCString getPath ();
	QCString getName ();
	QString getDir ();
	QCString getWorkspaceDir ();
	AffinityMask_t getAffinityMask ();
	void setAffinityMask(AffinityMask_t afMask);

private:
	int needsConversion (QString first_line);
	void convertFromVer1 ();
	void readConvertGeneral (QTextStream *pStream);
	void readConvertTbs (QTextStream *pStream, QString sessionName = QString::null);
	void readConvertEbp (QTextStream *pStream, QString sessionName = QString::null);

	bool helpReadRunOptions (QString parm, QString value, RUN_OPTIONS *pRun);
	void readGeneralSection (QTextStream * pStream);
	void readTbpSection	   ( QTextStream *pStream );
	void readEventSection( QTextStream *pStream );
	void readIbsSection (QTextStream *pStream );

	void helpWriteRunOptions (QTextStream *pStream, RUN_OPTIONS *pRun);
	void writeTbpSection (QTextStream *pStream );
	void writeEventSection (QTextStream *pStream );
	void writeIbsSection (QTextStream *pStream );

	SessionMap m_sessions[NO_TRIGGER]; //yay, abstraction!

	RUN_OPTIONS m_runOptions;
	QString m_lastProfileCfg;
	QStringList m_launchHistory;
	QStringList m_workDirHistory;
	QStringList m_processFilter;
	bool m_applyProcFilter; 

	QCString m_szFileName;
	QString m_szFileDir;
	QCString m_szFilePath;

	QFile *m_pFile;
	bool m_bOpened;
	bool m_bSymInit;
	int m_file_version;
};

#endif
