//$Id: atuneoptionsdlg.h,v 1.3 2006/05/15 22:09:22 jyeh Exp $
//interface for the CATuneOptionsDlg class.

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2006 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

//Revision history
//$Log: atuneoptionsdlg.h,v $
//Revision 1.3  2006/05/15 22:09:22  jyeh
//Merge with brach Flat_file
//
//Revision 1.2.2.3  2005/08/19 14:56:44  jyeh
//Added OP091.
//
//Revision 1.2.2.2  2005/06/15 19:11:09  jyeh
//Daily update.
//
//Revision 1.2.2.1  2005/06/07 18:51:26  jyeh
//daily update
//
//Revision 1.2  2005/02/14 16:52:40  jyeh
//Updated header.
//
//Revision 1.1.1.1  2004/10/05 17:03:23  jyeh
//initial import into CVS
//
//Revision 1.14.2.2  2004/08/30 20:22:37  franksw
//Fix for bug21897
//
//Revision 1.14.2.1  2004/02/13 18:04:19  franksw
//Fix for bug 19181, bad defaults on redhat
//
//Revision 1.14  2004/01/21 16:36:12  jyeh
//Added code to validate Oprofile Kernel note size.
//
//Revision 1.13  2003/12/15 17:27:03  franksw
//Code Review of Sourceview
//
//Revision 1.12  2003/12/08 20:18:23  franksw
//Changes from code review
//
//Revision 1.11  2003/12/02 22:01:17  franksw
//Fix for virtual functions
//
//Revision 1.10  2003/12/01 21:06:06  franksw
//cleaned up atuneoptions
//
//Revision 1.9  2003/11/26 20:31:07  jyeh
//Added checks to ensure the buffer is between the min and the max size.
//
//Revision 1.8  2003/11/25 23:22:26  franksw
//Added color options
//
//Revision 1.7  2003/11/18 21:48:32  jyeh
//Added OnNoVmlinux().
//
//Revision 1.6  2003/11/13 17:53:55  franksw
//code cleanup
//

#ifndef _ATUNEOPTIONSDLG_H
#define _ATUNEOPTIONSDLG_H

#include <qpushbutton.h>

#include "stdafx.h"
#include "atuneoptions.h"

//generated from iatuneoptionsdlg.ui
#include "iatuneoptionsdlg.h"

// All numbers are from Oprofile 0.9.3
enum CAtuneOptionsDlgEnumType
{
	// From libop/op_config_24.h in 
	OP_MIN_PRE_WATERMARK = 8192,

	/** maximum number of entry in samples eviction buffer */
	OP_MAX_BUF_SIZE = 4194304,

	/** minimum number of entry in samples eviction buffer */
	OP_MIN_BUF_SIZE = 40960,

	// From libop/op_config.h
	OP_MIN_CPU_BUF_SIZE = 2048,

	OP_MAX_CPU_BUF_SIZE = 131072
};

enum OP_PLUG_IN
{
	OP_091,
};

class ElementPushButton : public QPushButton
{
	Q_OBJECT
public:
	ElementPushButton (const QString & text, QWidget * parent,
		const char * name = 0);
	void setCpu ( int cpu);
	void setEvent (int event);

signals:
	void pushClicked ( int cpu, int event );

private slots:
	void onClicked ();

private:
	int m_cpu;
	int m_event;
};


class CATuneOptionsDlg  : public IATuneOptionsDlg
{
	Q_OBJECT
public:
	CATuneOptionsDlg ( QWidget* parent, const char* name, bool modal,
		WFlags fl= WStyle_Customize | WStyle_NormalBorder
		| WStyle_Title | WStyle_SysMenu);
	virtual ~CATuneOptionsDlg();

public slots:
	// overrides
	void onApply();
	void onOk();
	void onNoVmlinux();
	void onProjectDirectoryBrowse();
	void onResetBufferSize();

	void onVmlinuxBrowse();

	bool validateOptions();

private:
	void initializeOptions();
	void saveOptions();
	void initializePluginSelection(CATuneOptions & ao);
	void initializeSearchPaths();
	void saveSearchPaths (CATuneOptions &ao);

private slots:
	//Todo: Do this with arrays instead...
	void onSourcePathBrowse (int index, int);

private:
	//related to dynamic creation
	QLineEdit * m_src_dirs [SOURCE_SEARCH_COUNT];
	ElementPushButton * m_src_dir_browse [SOURCE_SEARCH_COUNT];
};

#endif // #ifndef _ATUNEOPTIONSDLG_H
