//$Id: eventsfile.h,v 1.2 2005/02/14 16:52:40 jyeh Exp $
/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2008 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef _EVENTS_FILE
#define _EVENTS_FILE

#include <qfile.h>
#include <qxml.h>
#include <list>

#include "typedefs.h"

using namespace std;

struct UnitMask {
	QString name;               // human readable name
	unsigned int value;         // unit mask value
};

typedef list < UnitMask > UnitMaskList;

struct CpuEvent {
	QString name;               // human readable name for event
	QString abbrev;		// Abbreviated name
	QString source;             // source unit for event
	unsigned int value;         // event value
	// number of valid unit masks for the event
	unsigned int nUnitMasks;
	// list of valid unit masks for the vent
	UnitMaskList m_UnitMaskList;
	QString op_name;
	int m_minValidModel;	// The minimum model number for which the event is valid.

	CpuEvent(){
		value = 0;
		nUnitMasks = 0; 
		m_minValidModel = 0;
	};
};

typedef list < CpuEvent > EventList;

enum QtArrayEventEnumType {
	LABEL_EVENT = 0,
	LABEL_COUNT,
	LABEL_SOURCE,
	LABEL_OS,
	LABEL_USR,
	CA_EVENT_NAME = 0,
	EVENT_TYPE,
	LINE_COUNT = 0,
	LINE_SOURCE,
	SPACER_COUNT = 6,
	LAYOUT_COUNT = 10,
	CHECK_UNIT_MASK = 0,
	CHECK_OS = 8,
	CHECK_USR,
	RADIO_INDEX = 0,
	RADIO_ALPHABETIC
};

/*===================================================================
* Events File Class
* ~~~~~~~~~~~~~~~~~
* Provides functions for interfacing with the xml based .cef 
* (CPU Events File)
*/
class CEventsFile:public QXmlDefaultHandler {
public:
	CEventsFile ();
	virtual ~ CEventsFile ();

	// client interface
	bool open (const char *events_file_path);
	void close ();
	void printEvents ();

	bool isEventExist( QString eventName );
	bool findEventByName (QString eventName, CpuEvent & event);
	bool findEventByValue (unsigned int value, CpuEvent & event);
	bool findEventByOPName (QString eventName, CpuEvent & event);
	bool findIbsEvent(EventList &evList);

	EventList::const_iterator FirstEvent ();
	EventList::const_iterator EndOfEvents ();
	UnitMaskList::const_iterator FirstUnitMask (const CpuEvent & event);
	UnitMaskList::const_iterator EndOfUnitMasks (const CpuEvent & event);



	/* overrides from QXmlDefaultHandler */
	bool startDocument ();
	bool startElement (const QString & namespaceURI,
		const QString & localName, const QString & qName,
		const QXmlAttributes & atts);
	bool endElement (const QString & namespaceURI,
		const QString & localName, const QString & qName);
	bool characters (const QString & ch);

private:
	EventList m_EventList;
	CpuEvent m_CurrentEvent;
	QString m_CurrentSourceUnit;
	QString m_current_op_name;
	DWORD m_nEvents;

};

#endif // #ifndef _EVENTS_FILE
