/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2009 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "config.h"
#include "helperAPI.h"

#include <stdlib.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <qprocess.h>
#include <qtimer.h>
#include <qmessagebox.h>

#include <elf.h>

#define PACKED_DIR_NAME "capacked"

/* 
* folderRemove:
* Recursive contents within the folder and the folder
* itself.
* It is the same code from oprofile_interface.cpp
*/
bool folderRemove (const QDir & _d)
{
	QDir d = _d;

	QStringList entries = d.entryList(QDir::All|QDir::Hidden);
	QStringList::Iterator it = entries.begin();

	for (; it != entries.end (); ++it)
	{
		if (*it == "." || *it == "..")
			continue;
		QFileInfo info (d, *it);
		if (info.isDir ())
		{
			if (!folderRemove (QDir (info.filePath ())))
				return false;
		}
		else
		{
			d.remove (info.filePath ());
		}
	}

	QString name = d.dirName ();

	if (!d.cdUp ())
		return false;

	d.rmdir (name);
	return true;
}

void untar_packed_file(QString * packed_path, QString * temp_dir )
{
	QDir dir;
	if (!QFile::exists(*packed_path)) {
		return;    
	}

	srand(time(0));
	temp_dir->sprintf("/tmp/CA%ld/", random());
	dir.mkdir(*temp_dir);

	// todo hook the output to the dialog
	// todo use a randomly generated directory
	QString command = "tar zxf " + *packed_path + " -C " + *temp_dir;
	system(command.data());
}


void mkdir_p(QString & absPath)
{
	QStringList list =  QStringList::split("/", absPath);
	QStringList::iterator it = list.begin();

	QString dir_path = "/";
	for (; it != list.end(); ++it) {
		dir_path = dir_path + (*it) + "/";
		QDir dir(dir_path);

		if (!dir.exists())  {
			if (!dir.mkdir(dir_path))
				qDebug("mkdir failed");
		}

	}
}


void copy_files(QFileInfo & info, QString & dest_dir)
{
	// Copy any non-directory file to the new path
	QDir dir = QDir(info.filePath());
	dir.setFilter(QDir::Files);
	QStringList file_list = dir.entryList(); 
	QStringList::Iterator itFile = file_list.begin();
	for (; itFile != file_list.end(); ++itFile) {
		QFileInfo info(dir, (*itFile));

		QString command = "cp \"" + info.absFilePath() + "\" \"" + dest_dir + "\"";

		system(command.data());
	}
}


void recurse_dirs(QString & parent_dir_str, 
			QString & dest_dir_str, 
			QString & subs_dir,
			bool prev_dep)
{
	QDir current_dir = QDir(parent_dir_str);   

	current_dir.setFilter(QDir::Dirs);
	QStringList old_dir_list = current_dir.entryList();
	QStringList::Iterator it = old_dir_list.begin();

	for (; it != old_dir_list.end(); ++it) {
		bool dep = false;
		if (*it == "." || *it == "..")
			continue;

		QFileInfo info(current_dir, *it);
		QString path = info.filePath(); 

		QString temp = dest_dir_str + "/" + *it;

		if ((*it).contains("{dep}"))
			dep = true;
		else if (((*it).contains("{root}") || (*it).contains("{kern}")) 
		       &&  prev_dep == true)
			temp += subs_dir;
		else
			dep =false;

		mkdir_p(temp);
		copy_files(info, temp);
		recurse_dirs(path, temp, subs_dir, dep);
	}
}


unsigned int rename_files(QString * temp_dir, 
			QString * op_sample_path,
			QString * cpuinfo_path,
			QString * dispatched_ops_path)
{
	//todo capacked is hard coded right now.
	QString top_dir = *temp_dir;
	QString op_current_dir_str = *temp_dir + PACKED_DIR_NAME + "/current";
	QDir op_current_dir(op_current_dir_str );

	// Move the sample file to correct directory structure
	// For example: sample files in  
	// /tmp/CA_982/capacked/current/{root}/bin/basename 
	// needs to be moved to 
	// /tmp/CA_982/capacked/current/{root}/tmp/CA_982/capacked/binary/bin/basename/

	// Since we will be recursing capacked/current directory tree tree
	// the new structure is crecated at at capacked/new_current
	QString dest_topdir_str = top_dir + "capacked/samples/current/{root}" 
		+ *temp_dir + "capacked/binary";

	*op_sample_path = top_dir + "capacked/samples/current/";

	*cpuinfo_path = top_dir + "capacked/cpuinfo";

	*dispatched_ops_path = top_dir + "capacked/cainfo/dispatched_ops";
	
	QString subs_dir = *temp_dir + "capacked/binary/{root}/";

	// Loop through the original directory structure and create 
	// correspondent directory under the new directory tree structure
	recurse_dirs(op_current_dir_str, dest_topdir_str, subs_dir, false);

	return 0;
}


int getBitness(const char * path)
{ 	
	char elf_header[EI_NIDENT];
	FILE *fp;
	int ret = -1;

	if (!path)
		goto out;

	fp = fopen(path, "r");

	/**
	* The reporting libraries does not return correct path
	* of kernel module under 2.6.1.  This function will not
	* under this situation.
	*/
	if (NULL == fp) 
		goto out;

	fread(&elf_header, 16, 1, fp);
	if (ferror(fp)) {
                fclose (fp);
                goto out;
        }
        fclose(fp);

        if (0 != strncmp(elf_header, ELFMAG, SELFMAG))
                goto out;

        switch (elf_header[EI_CLASS]) {
                case ELFCLASS32:
                        ret = 32; 
                        break;
                case ELFCLASS64:
                        ret = 64;
                        break;
        }
out:
	return ret; 

}


int ca_exec_command (QStringList & args, 
			QString & stdOut,
			QString & stdErr,
			int & status,
			bool isBlock)
{
	int rt = -1;

	QProcess * proc = new QProcess();
	if (!proc)
		goto out;	

	proc->setArguments (args);

//	fprintf(stderr,"ca_exec_command = %s\n", proc->arguments().join(" ").ascii());

	if(!proc->start ()) {
		goto out;
	}

	// Since the process being launch are non-blocking daemon,
	// sysctl, mknod, etc, just wait for the command to finish
	while (proc->isRunning ()) { 
		if (!isBlock)
			break;
	}

	// Get exit status/stdOut/stdErr
	status = proc->exitStatus();
	stdErr = QString(proc->readStderr());
	stdOut = QString(proc->readStdout());

	// Check for normal Exit
	if (proc->normalExit () && status == 0)
		rt = 0;
out:
	if (proc)
		delete proc;
	return rt;
}


int ca_exec_sudo_command (QStringList & args, 
			QString & stdOut,
			QString & stdErr,
			int & status,
			bool isBlock)
{
	int rt = -1;

	QStringList cmd;
	cmd.append("sudo");
	cmd.append("-S");
	cmd += args;

	QProcess * proc = new QProcess();
	if (!proc)
		goto out;	

	proc->setArguments (cmd);

//	fprintf(stderr,"ca_exec_command = %s\n", proc->arguments().join(" ").ascii());

	if(!proc->start ()) {
		goto out;
	}

	/* NOTE:
	 * sudo, should never output to the stderr.
	 * We consider this error and will just exit out.
	 */
	while (proc->isRunning ()) { 
		if (!isBlock)
			break;
		if (proc->readStderr().size() != 0){
			proc->tryTerminate();
			QTimer::singleShot(1000, proc, SLOT(kill()));
			break;
		}
	}

	// Get exit status/stdOut/stdErr
	status = proc->exitStatus();
	stdErr = QString(proc->readStderr());
	stdOut = QString(proc->readStdout());

	// Check for normal Exit
	if (proc->normalExit () && status == 0)
		rt = 0;
out:
	if (proc)
		delete proc;
	return rt;
}


bool cssConversion(QString cgOutput)
{
	// TODO: [Suravee] Currently we directly call cgreport
	// since we want cgprocessing code to be a standalone utility.
	// and can eliminate the build dependency.  We might want
	// to integrate this into the gui in the future.
	QString cgreportPath  	= QString(CA_BIN_DIR) + "/cgreport"; 
	QString cgFile 		= QString("/var/lib/oprofile/samples/ca_oprofile.cg");

	if (!QFile::exists (cgreportPath)) {
		QMessageBox::critical(NULL, "CSS Conversion Error",
			QString("CodeAnalyst could not process call-stack sampling data.\n") +
			"Please install " + cgreportPath + ".\n");
		return false;
	}

	if (!QFile::exists (cgFile)) {
		QMessageBox::critical(NULL, "CSS Conversion Error",
			cgFile + " not found.\n" +
			"Please rerun the profile with proper CSS setting.\n");
		return false;
	}

	QStringList cmd;
	cmd.append(cgreportPath);
	cmd.append(QString("-i"));
	cmd.append(cgFile);
	cmd.append(QString("-o"));
	cmd.append(cgOutput + "/callgrind.out");

	QString stdOut, stdErr;
	int status;
	return (ca_exec_command(cmd, stdOut, stdErr, status) == 0
		&& status == 0);
}
