//$Id: oprofile_interface.h,v 1.4 2005/02/14 16:52:40 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef OPROFILE_INTERFACE_H_
#define OPROFILE_INTERFACE_H_

#include "stdafx.h"
#include <qstring.h>
#include <qthread.h>
#include <qobject.h>
#include <qprocess.h>

#define MUX_DRIVER_INTERFACE 	"time_slice"

namespace oprofile_interface_ns {

	enum profiling_types {
		OP_NONE= 0,
		OP_EBP = 0x01,
		OP_IBS_FETCH = 0x02,
		OP_IBS_OP = 0x04,
	};

	enum oprofile_interface_errorno {
		OIE_OK=0,
		OIE_LOCK_EXIST,
		OIE_DAEMON_NOTSTART,
		OIE_DAEMON_RUNNING,
		OIE_DRV,
		OIE_ERR
	};

};

using namespace oprofile_interface_ns;

/**
* Interface between CodeAnalyst and Oprofile. 
* The class essentially does the same thing as opcontrol. 
*
* Note: The function names do not follow the QT coding
* standard to reflect the equivalent name in opcontrol 
* script.
*/
class oprofile_interface:QObject {
	Q_OBJECT
public:

	oprofile_interface ();
	int do_start_daemon (bool vmlinux, 
			QString & vmlinux_dir,
			unsigned int buffer_size, 
			unsigned int buffer_watershed_size, 
			unsigned int cpu_buf_size,
			profiling_types active_profiling, 
			unsigned int multiplex_interval);

	void set_op_events (op_event_property * properties);
	void set_op_ibs ( op_ibs_property * properties);
	bool set_css(unsigned int cssDepth, 
			unsigned int cssInterval,
			unsigned int cssTgid,
			unsigned int cssBitness);
	void do_dump ();
	void do_kill_daemon ();
	void do_pause_daemon ();
	void do_resume_daemon();


	QString get_kernel_range (QString vmlinux_dir);
	QString & get_std_err();

	bool do_setup ();

	bool remove_lock_file();
	
	int getNumEventCounters(){return m_numEventCountersAvailable;};

	bool havePermissionToProfile();

	bool loadOprofileDriver();
	
	bool checkOprofileDriverStatus();

	int checkIbsSupportInDaemon();

	int checkDispatchOpInDaemon();

	int checkDCMissInfoInDaemon();
	
	bool checkMuxSupportInDaemon();

	bool checkMuxSupportInDriver();

	bool checkCssSupportInDaemon();

	bool checkCssSupportInDriver();

	static bool set_param (QString field, 
					unsigned long int value,
					QString & stdOut,
					QString & stdErr,
					int & status);
	
private:

	bool set_param (QString field, unsigned long int value);
	
	int exec_command (QStringList & args);
	
	int exec_sudo_command (QStringList & args);

	int do_reset();
	
	int  help_start_daemon (bool vmlinux, 
			QString & vmlinux_dir, 
			profiling_types active_profiling);

	bool get_daemon_pid(unsigned int &pid);


	bool do_sysctl_setup (unsigned int buffer_size, 
				unsigned int buffer_watershed_size, 
				unsigned int cpu_buf_size,
				profiling_types active_profiling, 
				unsigned int multiplex_interval);

	void customEvent (QCustomEvent * pCustEvent);

	void generate_op_event_arg (QString & arg);

	bool set_ctr_param (int event, QString field, unsigned long int value);


	bool do_ebs_setup(profiling_types active_profiling);
	
	bool do_ibs_setup(profiling_types active_profiling);

	QString simplifyPath(QString path);
	
	bool is_driver_loaded();
	
	bool is_oprofilefs_mounted();

private:
	op_ibs_property 	* m_pIbsProperties;
	op_event_property 	m_EventProperties[MAX_EVENTNUM_MULTIPLEXING];
	QString 		m_stderr;
	QString 		m_stdout;
	int 			m_status;
	int 			m_numEventCountersAvailable;
	bool 			m_isDaemonStarted;
	unsigned int 		m_daemon_pid;
	int			m_ibsVersion;
	int			m_hasDispatchOpInDaemon;
	int			m_hasDCMissInfoInDaemon;
	
};

#endif
