//$Id: projectwindow.cpp,v 1.3 2005/02/14 16:52:29 jyeh Exp $
// implementation for the ProjectWindow class.

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "projectwindow.h"

ProjectWindow::ProjectWindow ( QWidget * pParentWnd ) 
: QWidget ( pParentWnd, "ProjectWindow" )
{
	m_current_item = NULL;
	m_tbp = NULL;
	m_ebp = NULL;
	m_ibs = NULL;
	m_tree = NULL;
	m_current_item  = NULL;	

	// Initialize the Tree View with Items
	m_tree = new QListView( this );
	m_tree->setVScrollBarMode ( QScrollView::AlwaysOn  );
	m_tree->setSorting (-1);
	m_tree->addColumn ( NO_PROJECTS_OPENED );
        m_tree->setResizeMode(QListView::AllColumns);
	m_tree->setRootIsDecorated ( TRUE );

	// Connect the signals to slots
	QObject::connect (m_tree, SIGNAL (doubleClicked (QListViewItem *) ),
		SLOT ( onDblClicked (QListViewItem *) ));
	QObject::connect (m_tree, SIGNAL (selectionChanged (QListViewItem *)),
		SLOT ( onSelChange (QListViewItem *) ));
	QObject::connect (m_tree, SIGNAL (expanded (QListViewItem *) ),
		SLOT ( onExpanded  (QListViewItem *) ));
	QObject::connect(m_tree, SIGNAL (rightButtonClicked (QListViewItem *,
		const QPoint &, 
		int)),
		SLOT (onRightButtonClicked (QListViewItem *, 
		const QPoint &, int)) );
	QObject::connect(m_tree, SIGNAL (contextMenuRequested (QListViewItem *,
		const QPoint &, 
		int)),
		SLOT (onRightButtonClicked (QListViewItem *, 
		const QPoint &,
		int)));

	m_menu = new QPopupMenu ( m_tree );

	if (NULL != m_menu) {
		m_menu->insertItem ("&Properties", this, 
			SLOT (onPropertiesSession()));
		m_menu->insertItem( "&Copy configuration to current", this, 
			SLOT (OnCopyToCurrent()) );
		m_menu->insertItem ("&Rename", this, SLOT (onRenameSession()) );
		m_menu->insertItem ("Create D&iff Session", this, 
			SLOT (onOpenSessionInDiffAnalyst()));
		m_menu->insertSeparator();
		m_menu->insertItem ("&Delete", this, 
			SLOT (deleteCurrentSessionItem()));
	}

	m_menu_parent = new QPopupMenu ( m_tree );

	if (NULL != m_menu_parent) {
	    m_menu_parent->insertItem ("&Delete all Sessions", this, 
				SLOT (deleteAllSessionItem()));
	}
} //ProjectWindow::ProjectWindow


ProjectWindow::~ProjectWindow()
{
	if (NULL != m_menu)
		delete m_menu;
}


void ProjectWindow::onExpanded ( QListViewItem * item )
{
	if (TBP_SESSION_STR == item->text (0)) {
		QCString TipString;
		QRect	 ItemRect;
		QListViewItem * p;

		for (p = item->firstChild(); NULL != p; p = p->nextSibling()) {
			m_tree->ensureItemVisible ( p );
			ItemRect = m_tree->itemRect ( p );

			TipString.sprintf ( "%s", item->text (0).data() ); 
		}
	}
}


// Delete all items in the list view
//
void ProjectWindow::deleteAllItems()
{
	m_tree->clear();
	m_tree->setColumnText ( 0, NO_PROJECTS_OPENED );
}


// Set the title of the list view
//
void ProjectWindow::setCaption ( QString str )
{
	m_tree->setColumnText (0, str);
}


void ProjectWindow::onDblClicked ( QListViewItem * item )
{
	if ( NULL == item ) return;
	if ( NULL ==  item->parent() ) return;

	if( item->parent()->text(0) == TBP_SESSION_STR )
	{
		emit SessionDblClicked (TIMER_TRIGGER, item->text(0) );
	} 
	else if( item->parent()->text(0) == EBP_SESSION_STR )
	{
		emit SessionDblClicked (EVENT_TRIGGER, item->text(0) );
	} 
	else if( item->parent()->text(0) == IBS_SESSION_STR )
	{
		emit SessionDblClicked (IBS_TRIGGER, item->text(0) );
	} 
}


// Called when the selection has changed in the tree view
// We want to emit a signal with the sessions name to the app window
//
void ProjectWindow::onSelChange ( QListViewItem * item )
{
	if ( NULL == item->parent() ) return;

	// Track the current item
	m_current_item = item;

	if ( TBP_SESSION_STR == item->parent()->text (0) ) {
		emit newSessionSelected ( item->text (0) );
	}
}


void ProjectWindow::resizeEvent ( QResizeEvent * e )
{
	Q_UNUSED (e);
	m_tree->resize ( width(), parentWidget()->height() );
}


void ProjectWindow::keyReleaseEvent ( QKeyEvent * e )
{
	if ( Qt::Key_Delete == e->key() ) {
		deleteCurrentSessionItem();
	}
}


void ProjectWindow::insertTbpSamplingSession ( QString session_name )
{	
	static QListViewItem * last_item = NULL;
	QListViewItem *item;
	if (NULL != last_item)
		item = new QListViewItem ( m_tbp, last_item, session_name );
	else
		item = new QListViewItem ( m_tbp, session_name );
	last_item = item;
}


void ProjectWindow::insertEbpSamplingSession ( QString session_name )
{	
	static QListViewItem * last_item = NULL;
	QListViewItem *item;
	if (NULL != last_item)
		item = new QListViewItem ( m_ebp, last_item, session_name );
	else
		item = new QListViewItem ( m_ebp, session_name );
	last_item = item;
}


void ProjectWindow::insertIbsSamplingSession ( QString session_name )
{	
	static QListViewItem * last_item = NULL;
	QListViewItem *item;
	if (NULL != last_item)
		item = new QListViewItem ( m_ibs, last_item, session_name );
	else
		item = new QListViewItem ( m_ibs, session_name );
	last_item = item;
}


void ProjectWindow::initialize()
{

	/* 
	* NOTE: Put the initialization in this order so that TBP is on top 
	*/
	m_ibs = new QListViewItem ( m_tree, IBS_SESSION_STR );
	m_ebp = new QListViewItem ( m_tree, EBP_SESSION_STR );
	m_tbp = new QListViewItem ( m_tree, TBP_SESSION_STR );
}


void ProjectWindow::expandTbp()
{
	m_tbp->setOpen ( TRUE );
}


void ProjectWindow::expandEbp()
{
	m_ebp->setOpen ( TRUE );
}

void ProjectWindow::expandIbs()
{
	m_ibs->setOpen ( TRUE );
}

//Note that this seems to have an issue on items in 64-bit Linux
//
void ProjectWindow::onRightButtonClicked ( QListViewItem * item, 
										  const QPoint &pt, int col )
{
	Q_UNUSED (col);
	// ONly show the menu for children, not parent itmes
	if ( NULL == item ) return;
	if ( NULL == item->parent() ) 
	{
		m_menu_parent->popup(pt);
		return;
	}

	m_menu->popup ( pt );
}


void ProjectWindow::onPropertiesSession()
{
	if (NULL == m_current_item) return;

	QString current_name = m_current_item->text (0);
	if (TBP_SESSION_STR == m_current_item->parent()->text (0)) 
	{
		emit tbpSessionProperties ( current_name );
	} 
	else if ( EBP_SESSION_STR == m_current_item->parent()->text (0) ) 
	{
		emit ebpSessionProperties ( current_name );
	}
	else if ( IBS_SESSION_STR == m_current_item->parent()->text (0) ) 
	{
		emit ibsSessionProperties ( current_name );
	}
}


void ProjectWindow::OnCopyToCurrent()
{
	QListViewItem * item = m_tree->selectedItem();
	if (item == NULL)
		return;

	QString CurrentName = item->text(0);
	QString type = item->parent()->text(0);
	if( TBP_SESSION_STR == type)
	{
		emit CopySessionToCurrent (TIMER_TRIGGER, CurrentName );
	} 
	else if (EBP_SESSION_STR == type )
	{
		emit CopySessionToCurrent (EVENT_TRIGGER, CurrentName );
	}
	else if (IBS_SESSION_STR == type )
	{
		emit CopySessionToCurrent (IBS_TRIGGER, CurrentName );
	}
}

void ProjectWindow::onRenameSession()
{
	QListViewItem * item = m_tree->selectedItem();
	if (item == NULL)
		return;
	QString CurrentName = item->text(0);
	QString FileName;
	QString NewName;

	//tryagain:
	while(1)
	{
		bool isOk = false;
		NewName = QInputDialog::getText( "Enter new name for the session", 
			"Rename Session '" + CurrentName + "'.",
			QLineEdit::Normal,CurrentName,&isOk,this );

		if( NewName.isEmpty())
			if( isOk)
				QMessageBox::critical(this,"Error","Please specify session name.\n");
			else
				return;
		else
			break; 
	}

	if(NewName.compare(CurrentName) == 0)
		return;	

	// Singal the name change
	if( item->parent()->text(0) == TBP_SESSION_STR )
	{
		emit SessionRenamed (TIMER_TRIGGER, CurrentName, NewName );
	} 
	else if( item->parent()->text(0) == EBP_SESSION_STR )
	{
		emit SessionRenamed (EVENT_TRIGGER, CurrentName, NewName );
	}
	else if( item->parent()->text(0) == IBS_SESSION_STR )
	{
		emit SessionRenamed (IBS_TRIGGER, CurrentName, NewName );
	}
}

void ProjectWindow::deleteCurrentSessionItem()
{
	QListViewItem *pItem = m_tree->selectedItem();

	//Don't delete a null selection or any of the headers!
	if( (NULL != pItem) && (pItem->parent() != NULL))
	{
		int answer = QMessageBox::information( this, "Delete?", 
			"Delete session '" + pItem->text(0) + "'?",
			QMessageBox::Yes, QMessageBox::No );

		if( QMessageBox::Yes == answer  )
		{
			QString parent = pItem->parent()->text(0);
			TRIGGER trigger = NO_TRIGGER;
			if( parent == TBP_SESSION_STR )
			{
				trigger = TIMER_TRIGGER;
			} 
			else if( parent == EBP_SESSION_STR )
			{
				trigger = EVENT_TRIGGER;
			}
			else if( parent == IBS_SESSION_STR )
			{
				trigger = IBS_TRIGGER;
			}
			emit SessionDeletedAndReinit (trigger, pItem->key(0, TRUE) );
		}
    }
}

void ProjectWindow::deleteAllSessionItem()
{
	QListViewItem *pItem = m_tree->selectedItem();
	if(!pItem) return;
	
	QListViewItem *pChild = pItem->firstChild();
	if(!pChild) return;

	QListViewItem *pNextChild = NULL;

	int answer = QMessageBox::information( this, "Delete All Sessions?", 
					       "Delete all sessions in '" + pItem->text(0) + "'?",
					       QMessageBox::Yes, QMessageBox::No );

	if( QMessageBox::Yes == answer  )
	{
	    QString parent = pItem->text(0);
	    TRIGGER trigger = TIMER_TRIGGER;
	    if( parent == TBP_SESSION_STR )
	    {
		trigger = TIMER_TRIGGER;
	    } 
	    else if( parent == EBP_SESSION_STR )
	    {
		trigger = EVENT_TRIGGER;
	    }
	    else if( parent == IBS_SESSION_STR )
	    {
		trigger = IBS_TRIGGER;
	    }

	    do{
		pNextChild = pChild->nextSibling();
		emit SessionDeletedOnly (trigger, pChild->key(0, TRUE) );
		pChild = pNextChild;
	    }while(pChild);
	    emit reinitProjectWindow();
	}
}

void ProjectWindow::onOpenSessionInDiffAnalyst()
{
	DiffSessionInfo sessInfo;
 
	QListViewItem * item = m_tree->selectedItem();
	if (item == NULL)
		return;

	sessInfo.sessionFile = item->text(0);
	QString type = item->parent()->text(0);
	if( TBP_SESSION_STR == type)
	{
		sessInfo.type = DiffSessionInfo::TBP_SESS;	
	} 
	else if (EBP_SESSION_STR == type )
	{
		sessInfo.type = DiffSessionInfo::EBP_SESS;	
	}
	else if (IBS_SESSION_STR == type )
	{
		sessInfo.type = DiffSessionInfo::IBS_SESS;	
	}

	emit openSessionInDiffAnalyst(&sessInfo);
}
