//$Id: projectwindow.h,v 1.2 2005/02/14 16:52:40 jyeh Exp $
// interface for the ProjectWindow class.

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/


//Revision history
//$Log: projectwindow.h,v $
//Revision 1.2  2005/02/14 16:52:40  jyeh
//Updated header.
//
//Revision 1.1.1.1  2004/10/05 17:03:23  jyeh
//initial import into CVS
//
//Revision 1.4  2003/12/11 00:11:56  franksw
//Code Review Project Window
//
//Revision 1.3  2003/11/17 23:24:40  franksw
//code cleanup
//

#ifndef _PROJECTWINDOW_H
#define _PROJECTWINDOW_H

#include <qlistview.h>
#include <qmap.h>
#include <qpopupmenu.h>
#include "NewDiffSessionDlg.h"
#include "stdafx.h"

static const QString TBP_SESSION_STR = "TBP Sessions";
static const QString EBP_SESSION_STR = "EBP Sessions";
static const QString IBS_SESSION_STR = "IBS Sessions";
static const QCString NO_PROJECTS_OPENED = "No Project Opened";

//////////////////////////////////////////////////////////////////////////////
// ProjectWindow
//
class ProjectWindow : public QWidget
{
	Q_OBJECT
public:

	ProjectWindow ( QWidget * pParentWnd );
	~ProjectWindow();

	void setCaption ( QString str );
	void deleteAllItems();
	void initialize();
	void insertTbpSamplingSession ( QString session_name );
	void insertEbpSamplingSession ( QString session_name );
	void insertIbsSamplingSession ( QString session_name );

protected:
	void resizeEvent ( QResizeEvent * e );
	void keyReleaseEvent ( QKeyEvent * e ); 

signals:
	void newSessionSelected ( QString session_name );

	void SessionDblClicked (TRIGGER trigger, QString SessionName);
	void reinitProjectWindow();	
	void SessionDeletedAndReinit (TRIGGER trigger, QString SessionName);
	void SessionDeletedOnly (TRIGGER trigger, QString SessionName);
	void SessionRenamed(TRIGGER trigger, QString OldSessionName, 
		QString NewSessionName );

	void tbpSessionProperties ( QString session_name );
	void ebpSessionProperties ( QString session_name );
	void ibsSessionProperties ( QString session_name );

	void CopySessionToCurrent (TRIGGER trigger, QString SessionName);
	void openSessionInDiffAnalyst(DiffSessionInfo *sess);

	public slots:
	void expandTbp();
	void expandEbp();
	void expandIbs();
	void onRenameSession();
	void onPropertiesSession();
	void OnCopyToCurrent();
	void onDblClicked ( QListViewItem * item );
	void onSelChange ( QListViewItem * item );	
	void onExpanded ( QListViewItem * item );
	void onRightButtonClicked ( QListViewItem * item, const QPoint &pt,
				    int col );
	void deleteCurrentSessionItem();
	void deleteAllSessionItem();

private:
	QListViewItem *	m_tbp;
	QListViewItem *	m_ebp;
	QListViewItem *	m_ibs;
	QListView *	m_tree;
	QListViewItem *	m_current_item;
	QPopupMenu * m_menu;
	QPopupMenu * m_menu_parent;

private slots:
	void onOpenSessionInDiffAnalyst();
	
};

#endif
