//$Id: atune.h,v 1.2 2006/05/15 22:09:45 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#ifndef SYSPROF_H
#define SYSPROF_H

#include "typedefs.h"
#define I386

enum ProjectWideEnums {
	MAX_EVENTNUM_MULTIPLEXING = 32,
	MAX_EVENTGROUP_MULTIPLEXING = 8,
	MAX_EVENTNUM 		= 4,
	MAX_CPUS 		= 8,
	TBP_EVENT_MASK 		= 0xff,
	ONCHIP_LOCAL_APIC 	= 0x200,
	SOURCE_SEARCH_COUNT 	= 3,
	GREYHOUND_FAMILY 	= 0x10,
	GRIFFIN_FAMILY 		= 0x11,
	OPTERON_FAMILY 		= 15,
	ATHLON_FAMILY 		= 6,
	OPTERON_MODEL 		= 5,
	HEX_BASE 		= 16,
	REV_C_MODEL		= 0x04
};

#define ATUNE_MAJOR	2
#define ATUNE_MINOR	2
#define ATUNE_REV	3

#define SIZE_PER_BUFFER     (2<<14)     // SIZE_PER_BUFFER = 32K
#define NUM_BUFFERS         4
#define	BUFFER_SIZE			SIZE_PER_BUFFER * NUM_BUFFERS
#define	BUFFER_ELEMENTS		(SIZE_PER_BUFFER/sizeof(PROFILE_DATA))

// interrupt sources
#define SOURCE_COUNTER0	0
#define SOURCE_COUNTER1	1
#define SOURCE_COUNTER2	2
#define SOURCE_COUNTER3	3
#define SOURCE_TIMER	4

#define MAGIC_DATA_SAMPLE	'S'
#define MAGIC_DATA_PROCESS	'P'
#define MAGIC_DATA_MODULE	'M'


#if defined(_x86_64_)

typedef unsigned long VADDR;
#define PATH_BUFFER UINT64
#define HANDLE_CAST	(UINT64)

#elif defined(I386)

typedef unsigned long VADDR;
#define PATH_BUFFER char *
#define HANDLE_CAST


#else

#error "Machine type required."

#endif

#define UNKNOWN_MODULE_PID	"unknown module pid"

//////////////////////////////////////////////////////////////////////////
// TRIGGER : ways in which to sample
// 
enum TRIGGER {
    TIMER_TRIGGER,
    EVENT_TRIGGER,
    EVENT_TRIGGER_PBM,
    IBS_TRIGGER,
    NO_TRIGGER
};

//////////////////////////////////////////////////////////////////////////
// Time Stamp Counter
//
// _TSC is defined in cadddef.h
typedef union _TSC {
    struct {
        DWORD dwLow;
        DWORD dwHigh;
    };

    UINT64 qwTimeStamp;

} TSC, *PTSC;

//////////////////////////////////////////////////////////////////////////
// 
typedef struct _PROFILER_INIT {
    char projectPath[256];      // path to the atune project

} PROFILER_INIT, *PPROFILER_INIT;


//////////////////////////////////////////////////////////////////////////
//
#pragma pack( push, 1 )
typedef struct _IMG_LOAD_INFO {
    DWORD Pid;
    VADDR Base;
    DWORD ImgSize;
    DWORD BufferSize;
    TSC load_tsc;
    PATH_BUFFER PathBuffer;
} IMG_LOAD_INFO, *PIMG_LOAD_INFO;
#pragma pack()

//////////////////////////////////////////////////////////////////////////
//
#pragma pack( push, 1 )
struct TBS_START_IOCTL {
    DWORD InitCount;
    DWORD BufferSize;
    char SessionFile[255];
};
#pragma pack()


//////////////////////////////////////////////////////////////////////////
//
#pragma pack( push, 1 )
typedef struct _EBS_IOCTL {
    bool bPreciseBranchMispredict;
    DWORD EventSelect[MAX_EVENTNUM];
    DWORD count[MAX_EVENTNUM];

} EBS_IOCTL, *PEBS_IOCTL;
#pragma pack()


//////////////////////////////////////////////////////////////////////////
#pragma pack( push, 1 )
struct TBS_LOG_HEADER {
    UINT64 NumSamples;
    UINT32 Ring0;
    UINT32 Ring3;
};
#pragma pack()


//////////////////////////////////////////////////////////////////////////
// profile process info
// 8 bytes
typedef struct PROFILE_PROCESS_EVENT {
    DWORD pid;
    BOOL create;

} PROFILE_PROCESS_EVENT, *PPROFILE_PROCESS_EVENT;

//////////////////////////////////////////////////////////////////////////
// profile module load event
//
typedef struct PROFILE_MODULE_EVENT {
    DWORD pid;
    VADDR base;
    DWORD size;
    BOOL create;

} PROFILE_MODULE_EVENT, *PPROFILE_MODULE_EVENT;

//////////////////////////////////////////////////////////////////////////
// Log Entry structure
// 31 Bytes
#pragma pack( push, 1 )
typedef struct PROFILE_SAMPLE {
    TSC tsc;                    // (0) Time Stamp for sample
    UINT32 PID;                 // (8) process id
    BYTE ProcNum;               // (12) cpu
    BYTE source;                // (13) source of interrupt

#if defined(_x86_64_)
    UINT64 CodeSeg;             // (14)
    VADDR EIP;                  // (22)
    BYTE Rsvd2[1];              // (30)
#elif defined(I386)
    UINT32 CodeSeg;             // (14) Code semgemnt for interrupted
                                // instruction
    UINT32 EIP;                 // (18) Address of instructions
    DWORD Rsvd0;                // (22)
    DWORD Rsvd1;                // (26)
    BYTE Rsvd2[1];              // (30)
#endif

} PROFILE_SAMPLE, *PPROFILE_SAMPLE;
#pragma pack()

//////////////////////////////////////////////////////////////////////////
// Profile Data structure
// 32 bytes
#pragma pack( push, 1 )
typedef struct PROFILE_DATA {
    BYTE magic;                 // P - Process S - Sample M - Module

    union {
        PROFILE_SAMPLE sample;
        PROFILE_PROCESS_EVENT psEvent;
        PROFILE_MODULE_EVENT moduleEvent;
    };

} PROFILE_DATA, *PPROFILE_DATA;
#pragma pack()

#endif // SYSPROF_H
