#include <qapplication.h>
#include <qprogressdialog.h>

#include "stdafx.h"
#include "catranslate_gui_display.h"

//#include "op_config.h"

#ifndef OP_BASE_DIR
#define OP_BASE_DIR "/var/lib/oprofile/"
#endif

#ifndef OP_SAMPLES_DIR
#define OP_SAMPLES_DIR OP_BASE_DIR "samples/"
#endif  

#ifndef OP_LOG_FILE
#define OP_LOG_FILE OP_SAMPLES_DIR "oprofiled.log"
#endif

ca_translate_gui_display::ca_translate_gui_display()
{
    m_progress_dlg_event = NULL;
}


ca_translate_gui_display::~ca_translate_gui_display()
{
}


void ca_translate_gui_display::init()
{
}


void ca_translate_gui_display::setDlgEvent(QWidget * progress_event)
{
    if (NULL != progress_event) {
        m_progress_dlg_event = progress_event;
    }
}

void ca_translate_gui_display::update_display(const char * text)
{
    QString msg = text;
    QApplication::postEvent (m_progress_dlg_event, 
        new QCustomEvent ((QEvent::Type)EVENT_UPDATE_PROGRESS,
        (void *) new QString (msg)));
    qApp->processEvents ();
}

void ca_translate_gui_display::update_display_error(const char * text)
{
    QString msg = text;
    QApplication::postEvent (m_progress_dlg_event, 
        new QCustomEvent ((QEvent::Type)EVENT_UPDATE_ERROR,
        (void *) new QString (msg)));
    qApp->processEvents ();
}

void ca_translate_gui_display::enableOkButton(bool b)
{
	QCustomEvent *custEvent = NULL;

	if(!b)
		custEvent = new QCustomEvent((QEvent::Type)EVENT_UPDATE_PROGRESS_BEGIN);
	else
		custEvent = new QCustomEvent((QEvent::Type)EVENT_UPDATE_PROGRESS_DONE);

	QApplication::postEvent (m_progress_dlg_event, custEvent);
	qApp->processEvents ();
}

void ca_translate_gui_display::display_oprofile_log()
{
    QFile * logFile =  new QFile(OP_LOG_FILE);
    QTextStream * logStream = NULL;
    QString tmp = "";
    QString sessionString;
    QStringList sessionStringsList;
    int curCpu = -1;
    //wait for the log file to be written to.
    qApp->processEvents (1000);
    if (NULL != logFile && logFile->exists()) {
        logFile->open(IO_ReadOnly);

        logStream = new QTextStream(logFile);

	if (NULL != logStream) {
		QApplication::postEvent (m_progress_dlg_event, 
			new QCustomEvent((QEvent::Type)EVENT_UPDATE_PROGRESS, 
			(void *) new QString("\n========== Oprofile Log ==========\n")));

		while(!logStream->atEnd()) 
		{
			QString line = logStream->readLine();
			line.append("\n");    

			// "Nr. sample dumps" is the first line that we care.
			// In oprofile 0.9.3, daemon doesn't clear the old log in the file.  
			// Here, we are looking for the last log in the file.
			if (line.startsWith("Nr. sample dumps"))
			{
				sessionString = line;
			}
			else if (line.startsWith("Nr."))
				sessionString.append(line);
		} 
        }

        QStringList finalLogList = QStringList::split("\n", sessionString);
       
        QStringList::iterator begin = finalLogList.begin(); 
        for (; begin != finalLogList.end(); begin++) {
            QString line = *begin;
            QEvent::Type message = (QEvent::Type) EVENT_UPDATE_PROGRESS;
		
		// Try to separate by CPU 
		if(line.contains("samples lost cpu buffer overflow"))
		{
			curCpu++;
			QString cpuLine("\n---------- CPU : ");
			cpuLine += QString::number(curCpu);
			cpuLine += " ----------\n";
			QApplication::postEvent (m_progress_dlg_event, 
                                     new QCustomEvent (message, 
                                         (void *) new QString (cpuLine)));
		} 
		
		// Highlight lost information in Red	 
		if (line.contains("lost")) 
		{
			//QString tmp = line.right(line.length() - line.findRev(":") - 2); 
			//if ("0" != tmp) {
				message = (QEvent::Type) EVENT_UPDATE_ERROR;
			//}
		}
		QApplication::postEvent (m_progress_dlg_event, 
                                     new QCustomEvent (message, 
                                         (void *) new QString (line)));
        }

        logFile->close();

        if (NULL != logFile) {
            delete logFile;
            logFile = NULL;
        }

        if (NULL != logStream) {
            delete logStream;
            logStream = NULL;
        }
    }
}
