//$Id: xpwin.cpp,v 1.2 2006/05/15 22:09:45 jyeh Exp $

/*
// CodeAnalyst for Open Source
// Copyright 2002 . 2005 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/


#include <errno.h>
#include <stdlib.h>
#include <string>
#include <sys/types.h> 
#include <sys/stat.h>
#include <qprocess.h>
#include <unistd.h>

#include "stdafx.h"
#include "xp.h"
#include "auxil.h"

//use 0xFF for TBP, use event mask for ebp

EventMaskMap g_EventMaskMap;


unsigned long getModelFromCpuInfo(const char * file)
{
	if (NULL == file)
		return 0;
	QFile devices (file);
	unsigned long model = 0;

	if (devices.open(IO_ReadOnly)) 
	{
		QTextStream stream(&devices);
		while (!stream.atEnd()) 
		{
			QString tmp = stream.readLine();
			if (tmp.contains ("model")) 
			{
				model = tmp.section(": ", 1, 1).toUInt();
				break;
			}
		}
	}

	return model;
}

unsigned long getFamilyFromCpuInfo(const char * file)
{
	if (NULL == file)
		return 0;
	QFile devices (file);
	unsigned long family = 0;

	if (devices.open(IO_ReadOnly)) 
	{
		QTextStream stream(&devices);
		while (!stream.atEnd()) 
		{
			QString tmp = stream.readLine();
			if (tmp.contains ("family")) 
			{
				family = tmp.section(": ", 1, 1).toUInt();
				break;
			}
		}
	}

	return family;
}

int XpFileContains(const char * file, const char * str)
{
    if ((NULL == file) || (NULL == str))
	return 0;
    QFile devices (file);
    int nTimes = 0;

    if (devices.open(IO_ReadOnly)) {
	QTextStream stream(&devices);
	while (!stream.atEnd()) {
	    QString tmp = stream.readLine();
	    if (tmp.contains (str)) {
		nTimes++;
	    }
	}
    }
    
    return nTimes;
}


unsigned int XpGetNumCpus()
{
	return XpFileContains ("/proc/cpuinfo", "processor");
}

unsigned int XpGetNumCoresPerSocket()
{
	unsigned int numCores = 1; 

	QFile devices ("/proc/cpuinfo");

	if (devices.open(IO_ReadOnly)) 
	{
		QTextStream stream (&devices);
		while (!stream.atEnd()) 
		{
			QString tmp = stream.readLine();
			if (tmp.contains ("cpu cores")) 
			{
				numCores = tmp.section(": ", 1, 1).toUInt();
			}
		}
	}
	    devices.close();

	return numCores;
}


/********************************************************
 Function: TimeStamp (assume athlon)
 RDTSC returns 64 bits of data in EAX,EDX
 We only use the lower 32 bits so, an overflow could occur every 8 seconds.
********************************************************/

unsigned int TimeStamp()
{
	
  unsigned int TickVal = 0;
	
  __asm__ __volatile__ (
			"rdtsc"
			: "=a" (TickVal)
			: /*no inputs*/
			: "%edx");
  
  return TickVal;	
}	


//////////////////////////////////////////////////////////////////////////////////
// XpGetAppPath()
// --------------
//
void XpGetAppPath(QString & path)
{
         path = QDir::currentDirPath();
}


//////////////////////////////////////////////////////////////////////////////////
// CpuId()
// -------
//  This function has to be compilied in either 32-bit mode or 64 bit mode.
//  If it's being compilied with 64-bit tools include the define -D_x86_64_
void CpuId( char *vendor_id, 
		unsigned long *family, 
		unsigned long *model, 
		unsigned long *stepping, 
		unsigned long *feature_flags)
{
    if ((NULL == vendor_id) || (NULL == family) || (NULL == model) || 
	(NULL == stepping) || (NULL == feature_flags))
	return;

	QFile devices ("/proc/cpuinfo");

	if (devices.open(IO_ReadOnly)) 
	{
		QTextStream stream (&devices);
		while (!stream.atEnd()) 
		{
			QString tmp = stream.readLine();
			if (tmp.contains ("vendor_id")) 
			{
				sprintf ( vendor_id, "%12s", tmp.section(": ", 1, 1).data());
			} 
			else if (tmp.contains ("cpu family")) 
			{
				*family = tmp.section(": ", 1, 1).toUInt();
			}  
			else if (tmp.contains ("model\t")) 
			{
				*model = tmp.section(": ", 1, 1).toUInt();
			} 
			else if (tmp.contains ("stepping")) 
			{
				*stepping = tmp.section(": ", 1, 1).toUInt();
			}
		}
	}
	    devices.close();

	__asm__ __volatile__ (
		"movl %%ebx, %%edi\n\t"
		"movl $1, %%eax\n\t"
		"cpuid\n\t"
		"movl %%edi, %%ebx\n\t"
		//feature_flags is in %dx
		: "=d" (*feature_flags)
		:
		: "eax","ecx");

} //CpuId

bool isCpuAMD ()
{
    QFile devices ("/proc/cpuinfo");

    if (devices.open(IO_ReadOnly)) 
    {
	QTextStream stream (&devices);
	while (!stream.atEnd()) 
	{
	    QString tmp = stream.readLine();
	    if (tmp.contains ("vendor_id")) 
	    {
		char vendor_id[13];
		sprintf ( vendor_id, "%12s", tmp.section(": ", 1, 1).data());
		vendor_id[12] = '\0';
		if (0 == strcmp (vendor_id, "AuthenticAMD"))
		    return true;
		break;
	    }
	}
    }
    devices.close();
    return false;
}

bool isCpuIbsOk()
{
	char VendorId[15];
	unsigned long family;
	unsigned long model;
	unsigned long stepping;
	unsigned long features;

	CpuId( VendorId, &family, &model, &stepping, &features );

	if( family == GREYHOUND_FAMILY)
		return true;
	else
		return false;
}

bool isDriverIbsOk()
{
	struct stat sb;
	if(stat("/dev/oprofile/ibs_fetch",&sb ) == -1)
	{
		return false;
	}

	return true;
}

bool isDriverMuxOk()
{
	struct stat sb;
	if(stat("/dev/oprofile/time_slice",&sb ) == -1)
	{
		return false;
	}

	return true;
}

bool isDriverIbsOpDispatchOpOk()
{
	struct stat sb;
	if(stat("/dev/oprofile/ibs_op/dispatched_ops",&sb ) == -1)
	{
		return false;
	}

	return true;
}

string helpGetEventFile(unsigned long cpuFamily,
		        unsigned long cpuModel)
{
	Q_UNUSED(cpuModel);

        char * dir = getenv("OPROFILE_EVENTS_DIR");
        string file;
        if (NULL == dir)
                file = CA_DATA_DIR + string("/events/");
	else
		file = dir;

        switch(cpuFamily) {
                case OPTERON_FAMILY:
                        file += K8_EVENTS_FILE;
                        break;
                case ATHLON_FAMILY:
                        file += K7_EVENTS_FILE;
                        break;
                case GRIFFIN_FAMILY:
                        file += GR_EVENTS_FILE;
                        break;
                case GREYHOUND_FAMILY:
                default:
			file += GH_EVENTS_FILE;
                        break;
        }
        return file;
}

unsigned int getTickSysFS()
{
        bool ok;
        unsigned int tps;
        QString line;
        QFile max_freq("/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq");
        QTextStream stream;

        if (!max_freq.open(IO_ReadOnly))
                return 0;

        stream.setDevice(&max_freq);
        line = stream.readLine();
        tps = line.toUInt(&ok);
        if (!ok)
                return 0;

        tps = tps * 1000;
        max_freq.close();

        return tps;
}

unsigned int getTickCPUInfo()
{
        float tps = 0;
        QTextStream stream;
        QString line;
        QFile cpuinfo("/proc/cpuinfo");

        if (!cpuinfo.open(IO_ReadOnly))
                return 0;

        stream.setDevice(&cpuinfo);
        do
        {
                line = stream.readLine();
                if (line.contains("cpu MHz"))
                {
                        bool ok;
                        QString stripped = line.stripWhiteSpace();
                        QString mhz = stripped.section(':', 1,1);
                        tps = mhz.toFloat(&ok);
                        if (!ok)
                                return 0;

                        tps = tps * 1000000;
                        break;
                }
        }
        while(!line.isNull());
        cpuinfo.close();

        return (unsigned int)tps;
}

// Calculate how many ticks in a second.
unsigned int getTick ()
{
        unsigned int tps = getTickSysFS();
        if (tps)
                return tps;
        else
                return getTickCPUInfo();
}


